/*

  The code below computes the variability indexes
  suggested by Stetson, Peter B. in 1996PASP..108..851S
  see http://adsabs.harvard.edu/abs/1996PASP..108..851S
  
*/
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <gsl/gsl_statistics_double.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_errno.h>

#include "limits.h"

#define WS_BINNING_DAYS 2.0


double sgn(double a){
 if( a<0.0 )
  return -1.0;
 return 1.0;
}

void seteson_JKL_from_sorted_lightcurve(size_t * input_array_index_p, double * input_JD, double * input_m, double * input_merr, int input_Nobs, int input_Nmax, double * output_J, double * output_K, double * output_L){
 int i,j,k,n;

 double J,J_forward,J_backward;
 double K;//,K_forward,K_backward;
 double L;//,L_forward,L_backward;

 double mean_magnitude,old_mean_magnitude;

 double a,b;

 double * P=malloc(4*input_Nobs*sizeof(double));
 double di,dj,dk;
 
 double sum, sum_w;

 double sqrt_n_nm1;

 double * w;
 w=malloc(4*input_Nobs*sizeof(double));
 if( w==NULL ){fprintf(stderr,"ERROR in stetson_variability_indexes.c - cannot allocate memory\n");exit(1);}
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 ////  Determine robust mean and weights through an iterative procedure following Stetson (1996) ////
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 // Initial guess
 for(i=0;i<input_Nobs;i++){w[i]=1.0/(input_merr[i]*input_merr[i]);}
 old_mean_magnitude=mean_magnitude=gsl_stats_wmean(w, 1, input_m, 1, input_Nobs);
 // re-weighting
 a=b=2.0;
 // 1000 here is the maximum number of iterations, just to make sure the process will not go on forever
 for(j=0;j<1000;j++){
  for(i=0;i<input_Nobs;i++){w[i]=w[i]/(1.0+ pow( fabs( (input_m[i]-mean_magnitude)/input_merr[i] )/a , b) );}
  mean_magnitude=gsl_stats_wmean(w, 1, input_m, 1, input_Nobs);
  if( fabs(old_mean_magnitude-mean_magnitude)<0.00001 )break; // erarly stop condition
  old_mean_magnitude=mean_magnitude;
 }
 //fprintf(stderr,"j=%d mean_magnitude=%lf\n",j,mean_magnitude);
 ////////////////////////////////////////////////////////////////////////////////////////////////////

 sqrt_n_nm1=sqrt(input_Nobs/(input_Nobs-1));

 // From now on w[] is re-used for the weights of P[], not m[]

 // Forward
 for(k=0,i=0;i<input_Nobs;i++){
  if( i<input_Nobs-3 ){
   if( input_JD[input_array_index_p[i+2]]-input_JD[input_array_index_p[i]]<WS_BINNING_DAYS ){
    // 3-point case
    di=sqrt_n_nm1*(input_m[input_array_index_p[i]]-mean_magnitude)/input_merr[input_array_index_p[i]];
    dj=sqrt_n_nm1*(input_m[input_array_index_p[i+1]]-mean_magnitude)/input_merr[input_array_index_p[i+1]];
    dk=sqrt_n_nm1*(input_m[input_array_index_p[i+2]]-mean_magnitude)/input_merr[input_array_index_p[i+2]];
    P[k]=di*dj;w[k]=2.0/3.0;k++;
    P[k]=dj*dk;w[k]=2.0/3.0;k++;
    P[k]=di*dk;w[k]=2.0/3.0;k++;
    i++;i++;//
    continue;
   }
  }
  if( i<input_Nobs-2 ){
   if( input_JD[input_array_index_p[i+1]]-input_JD[input_array_index_p[i]]<WS_BINNING_DAYS ){
    // 2-point case
    di=sqrt_n_nm1*(input_m[input_array_index_p[i]]-mean_magnitude)/input_merr[input_array_index_p[i]];
    dj=sqrt_n_nm1*(input_m[input_array_index_p[i+1]]-mean_magnitude)/input_merr[input_array_index_p[i+1]];
    P[k]=di*dj;w[k]=1.0;k++;
    i++;//
    continue;
   }
  }
  // 1-point case
  di=sqrt_n_nm1*(input_m[input_array_index_p[i]]-mean_magnitude)/input_merr[input_array_index_p[i]];
  P[k]=di*di-1.0;w[k]=1.0;k++; // should we assign the same weight to singleton observations???
 }

 n=k;
 sum_w=sum=0.0;
 for(k=0;k<n;k++){
  sum+=w[k]*sgn(P[k])*sqrt(fabs(P[k]));
  sum_w+=w[k];
 }
 
 J_forward=sum/sum_w;
 
 // Backward
 for(k=0,i=input_Nobs-1;i>-1;i--){
  if( i>1 ){
   if( input_JD[input_array_index_p[i]]-input_JD[input_array_index_p[i-2]]<WS_BINNING_DAYS ){
    // 3-point case
    di=sqrt_n_nm1*(input_m[input_array_index_p[i]]-mean_magnitude)/input_merr[input_array_index_p[i]];
    dj=sqrt_n_nm1*(input_m[input_array_index_p[i-1]]-mean_magnitude)/input_merr[input_array_index_p[i-1]];
    dk=sqrt_n_nm1*(input_m[input_array_index_p[i-2]]-mean_magnitude)/input_merr[input_array_index_p[i-2]];
    P[k]=di*dj;w[k]=2.0/3.0;k++;
    P[k]=dj*dk;w[k]=2.0/3.0;k++;
    P[k]=di*dk;w[k]=2.0/3.0;k++;
    i--;i--;//
    continue;
   }
  }
  if( i>0 ){
   if( input_JD[input_array_index_p[i]]-input_JD[input_array_index_p[i-1]]<WS_BINNING_DAYS ){
    // 2-point case
    di=sqrt_n_nm1*(input_m[input_array_index_p[i]]-mean_magnitude)/input_merr[input_array_index_p[i]];
    dj=sqrt_n_nm1*(input_m[input_array_index_p[i-1]]-mean_magnitude)/input_merr[input_array_index_p[i-1]];
    P[k]=di*dj;w[k]=1.0;k++;
    i--;//
    continue;
   }
  }
  // 1-point case
  di=sqrt_n_nm1*(input_m[input_array_index_p[i]]-mean_magnitude)/input_merr[input_array_index_p[i]];
  P[k]=di*di-1.0;w[k]=1.0;k++; // should we assign the same weight to singleton observations???
 }

 n=k;
 sum_w=sum=0.0;
 for(k=0;k<n;k++){
  sum+=w[k]*sgn(P[k])*sqrt(fabs(P[k]));
  sum_w+=w[k];
 }
 
 J_backward=sum/sum_w;
 
 J=(J_backward+J_forward)/2.0;
 
 //fprintf(stderr,"J_forward=%lf  J_backward=%lf  J=%lf  J_forward-J_backward=%lf n=%d\n",J_forward, J_backward,J,J_forward-J_backward,n);

 // K is a robust measure of the kurtosis of the magnitude histogram:
 sum_w=sum=0.0;
 for(i=0;i<input_Nobs;i++){
  di=sqrt_n_nm1*(input_m[input_array_index_p[i]]-mean_magnitude)/input_merr[input_array_index_p[i]];
  sum+=fabs(di);
  sum_w+=di*di;
 }
 K=1.0/(double)input_Nobs*sum/sqrt(1.0/(double)input_Nobs*sum_w);
 // For a Gaussian noise K->0.798
 
 L=J*K/0.798*(double)input_Nobs/(double)input_Nmax;
 
 (*output_J)=J;
 (*output_K)=K;
 (*output_L)=L;

 free(w);
 
 free(P);

 //exit(0);
 
 return;
}


double sign_only_welch_seteson_I_from_sorted_lightcurve(size_t * input_array_index_p, double * input_JD, double * input_m, double * input_merr, int input_Nobs){
 double I,I_forward;//,I_backward;
 int i,n;
 
 double mean_magnitude,sum;

 double * w;
 w=malloc(input_Nobs*sizeof(double));
 if( w==NULL ){fprintf(stderr,"ERROR in stetson_variability_indexes.c - cannot allocate memory\n");exit(1);}
 
 // Set weights
 for(i=0;i<input_Nobs;i++){w[i]=1.0/(input_merr[i]*input_merr[i]);}
 mean_magnitude=gsl_stats_wmean(w, 1, input_m, 1, input_Nobs);
 
 //for(i=0;i<input_Nobs-1;i++)fprintf(stderr,"%lf %lf %lf\n",input_JD[input_array_index_p[i]],input_m[input_array_index_p[i]],input_merr[input_array_index_p[i]]);
 //fprintf(stderr,"#######################################################################\n");
 // Forward
 sum=0.0;
 for(n=0,i=0;i<input_Nobs-1;i++){
  // do not form a pair if two points are too far apart
  if( input_JD[input_array_index_p[i+1]]-input_JD[input_array_index_p[i]]>WS_BINNING_DAYS )continue;
  // form a pair
  sum+=sgn( (input_m[input_array_index_p[i]]-mean_magnitude)/input_merr[input_array_index_p[i]]*(input_m[input_array_index_p[i+1]]-mean_magnitude)/input_merr[input_array_index_p[i+1]] );
  n++;
  //fprintf(stderr,"%lf %lf %lf\n%lf %lf %lf\n",input_JD[input_array_index_p[i]],input_m[input_array_index_p[i]],input_merr[input_array_index_p[i]],input_JD[input_array_index_p[i+1]],input_m[input_array_index_p[i+1]],input_merr[input_array_index_p[i+1]]);
  //i++;//
 }
 I_forward=sum*sqrt(1.0/( (double)n * ( (double)n - 1.0) ) );

 I=I_forward;

 free(w);

 return I;

}

double classic_welch_seteson_I_from_sorted_lightcurve(size_t * input_array_index_p, double * input_JD, double * input_m, double * input_merr, int input_Nobs){
 double I,I_forward;//,I_backward;
 int i,n;
 
 double mean_magnitude,sum;//,sum_w;

 double * w;
 w=malloc(input_Nobs*sizeof(double));
 if( w==NULL ){fprintf(stderr,"ERROR in stetson_variability_indexes.c - cannot allocate memory\n");exit(1);}
 
 // Set weights
 for(i=0;i<input_Nobs;i++){w[i]=1.0/(input_merr[i]*input_merr[i]);}
 mean_magnitude=gsl_stats_wmean(w, 1, input_m, 1, input_Nobs);
 //mean_magnitude=gsl_stats_mean(input_m, 1, input_Nobs);

/*
 sum_w=sum=0.0;
 for(n=0,i=0;i<input_Nobs-1;i++){
  // do not form a pair if two points are too far apart
  if( input_JD[input_array_index_p[i+1]]-input_JD[input_array_index_p[i]]>WS_BINNING_DAYS )continue;
  // form a pair
  w[i]=1.0; // /(input_merr[input_array_index_p[i]]*input_merr[input_array_index_p[i]]);
  w[i+1]=1.0; // /(input_merr[input_array_index_p[i+1]]*input_merr[input_array_index_p[i+1]]);
  sum+=w[i]*input_m[input_array_index_p[i]];
  sum+=w[i+1]*input_m[input_array_index_p[i+1]];
  sum_w+=w[i];
  sum_w+=w[i+1];
  n++;
  n++;
  //fprintf(stderr,"%lf %lf %lf\n%lf %lf %lf\n",input_JD[input_array_index_p[i]],input_m[input_array_index_p[i]],input_merr[input_array_index_p[i]],input_JD[input_array_index_p[i+1]],input_m[input_array_index_p[i+1]],input_merr[input_array_index_p[i+1]]);
  //i++;//
 }
 mean_magnitude=sum/sum_w;
*/
 
 //for(i=0;i<input_Nobs-1;i++)fprintf(stderr,"%lf %lf %lf\n",input_JD[input_array_index_p[i]],input_m[input_array_index_p[i]],input_merr[input_array_index_p[i]]);
 //fprintf(stderr,"#######################################################################\n");
 // Forward
 sum=0.0;
 for(n=0,i=0;i<input_Nobs-1;i++){
  // do not form a pair if two points are too far apart
  if( input_JD[input_array_index_p[i+1]]-input_JD[input_array_index_p[i]]>WS_BINNING_DAYS )continue;
  // form a pair
  sum+=(input_m[input_array_index_p[i]]-mean_magnitude)/input_merr[input_array_index_p[i]]*(input_m[input_array_index_p[i+1]]-mean_magnitude)/input_merr[input_array_index_p[i+1]];
  //fprintf(stderr,"%lf %lf  mean=%lf   %lf %lf\n",input_m[input_array_index_p[i]],input_m[input_array_index_p[i+1]], mean_magnitude, sum,  (input_m[input_array_index_p[i]]-mean_magnitude)/input_merr[input_array_index_p[i]]*(input_m[input_array_index_p[i+1]]-mean_magnitude)/input_merr[input_array_index_p[i+1]]  );
  n++;
  //fprintf(stderr,"%lf %lf %lf\n%lf %lf %lf\n",input_JD[input_array_index_p[i]],input_m[input_array_index_p[i]],input_merr[input_array_index_p[i]],input_JD[input_array_index_p[i+1]],input_m[input_array_index_p[i+1]],input_merr[input_array_index_p[i+1]]);
  //i++;//
 }
 I_forward=sum*sqrt( 1.0 /( (double)n * ( (double)n - 1.0) ) );

 //fprintf(stderr,"#######################################################################\n");
/*
 // Backward is the same as forward if we allow each point to be binned bothe with both its right and left neighbours 


 // Backward
 sum=0.0;
 for(n=0,i=input_Nobs-2;i>-1;i--){
  // do not form a pair if two points are too far apart
  if( input_JD[input_array_index_p[i+1]]-input_JD[input_array_index_p[i]]>WS_BINNING_DAYS )continue;
  // form a pair
  sum+=(input_m[input_array_index_p[i]]-mean_magnitude)/input_merr[input_array_index_p[i]]*(input_m[input_array_index_p[i+1]]-mean_magnitude)/input_merr[input_array_index_p[i+1]];
  n++;
  //fprintf(stderr,"%lf %lf %lf\n%lf %lf %lf\n",input_JD[input_array_index_p[i]],input_m[input_array_index_p[i]],input_merr[input_array_index_p[i]],input_JD[input_array_index_p[i+1]],input_m[input_array_index_p[i+1]],input_merr[input_array_index_p[i+1]]);
  //i--;//
 }
 I_backward=sum*sqrt(1.0/( (double)n * ( (double)n - 1.0) ) );

 I=(I_forward+I_backward)/2.0;

 fprintf(stderr,"#######################################################################\n");

 fprintf(stderr,"%lf %lf %lf\n",I_forward,I_backward,I);
*/
 I=I_forward;

 free(w);

 //exit(1);
 
 return I;
}


void stetson_variability_indexes(double * input_JD, double * input_m, double * input_merr, int input_Nobs, int input_Nmax, double * output_index_I, double * output_index_J, double * output_index_K, double * output_index_L, double * output_index_I_sign_only){
 size_t * p; // for index sorting
 int i;
 p=malloc(input_Nobs*sizeof(size_t));
 if( p==NULL ){fprintf(stderr,"ERROR in create_data.c - cannot allocate memory\n");exit(1);}
 // don't forget to initialize the index array "p"!
 for(i=0;i<input_Nobs-1;i++)p[i]=(size_t)i;
 gsl_sort_index( p, input_JD, 1, input_Nobs); // The elements of p give the index of the array element which would have been stored in that position if the array had been sorted in place. The array data is not changed. 
 (*output_index_I)=classic_welch_seteson_I_from_sorted_lightcurve( p, input_JD, input_m, input_merr, input_Nobs);
 (*output_index_I_sign_only)=sign_only_welch_seteson_I_from_sorted_lightcurve( p, input_JD, input_m, input_merr, input_Nobs);
 seteson_JKL_from_sorted_lightcurve( p, input_JD, input_m, input_merr, input_Nobs, input_Nmax, output_index_J, output_index_K, output_index_L);
 free(p);
 return;
}


// see http://en.wikipedia.org/wiki/Robust_measures_of_scale
// and http://en.wikipedia.org/wiki/Median_absolute_deviation#relation_to_standard_deviation
double esimate_sigma_from_MAD_of_sorted_data( double * sorted_data, int n){
 double median_data,MAD,sigma;
 double * AD;
 int i;   
 
 AD=malloc(n*sizeof(double));
 
 median_data=gsl_stats_median_from_sorted_data( sorted_data,1,n);
 for(i=0;i<n;i++){
  AD[i]=fabs(sorted_data[i]-median_data);
  //fprintf(stderr,"sorted_data[i]=%lf, median_data=%lf   AD=%lf\n",sorted_data[i],median_data,AD[i]);
 }
 gsl_sort( AD,1,n);
 MAD=gsl_stats_median_from_sorted_data( AD,1,n);
 
 free(AD);
 
 // 1.48260221850560 = 1/norminv(3/4)
 sigma=1.48260221850560*MAD;
 return sigma;
}
