#define _GNU_SOURCE         /* for memmem(). See feature_test_macros(7) */

// Standard header files
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h> /* for basename() */
#include <math.h>
// CFITSIO
#include "fitsio.h" // we use a local copy of this file because we use a local copy of cfitsio
// VaST's own header files
#include "limits.h"
#include "fitsfile_read_check.h"

#define MAX_FOV_ARCMIN 7200 // 120 deg.
#define MIN_FOV_ARCMIN 5

int try_to_recognize_telescop_keyword(char *fitsfilename, double * estimated_fov_arcmin){
 char telescop[1024];
 char telescop_comment[1024];
 char *pointer_to_the_key_start;
 // fitsio
 int status = 0;
 fitsfile *fptr; /* pointer to the FITS file; defined in fitsio.h */
 // Extract data from fits header
 fits_open_file(&fptr, fitsfilename, READONLY, &status);
 if ( 0!=status ){
  fits_report_error(stderr, status);  // print out any error messages
  fits_clear_errmsg(); // clear the CFITSIO error message stack
  return status;
 }
 fits_read_key(fptr, TSTRING, "TELESCOP", telescop, telescop_comment, &status);
 if ( 0!=status ){return 1;}
// TELESCOP= 'Lens  2.8/170'
 if( 0==strncasecmp(telescop,"Lens  2.8/170",1024-1) ){(*estimated_fov_arcmin)=180.0;return 0;}
// Lens
 pointer_to_the_key_start=(char *)memmem( telescop, strlen(telescop), "Lens", 4);
 if(pointer_to_the_key_start!=NULL){(*estimated_fov_arcmin)=180.0;return 0;}
// LENS
 pointer_to_the_key_start=(char *)memmem( telescop, strlen(telescop), "LENS", 4);
 if(pointer_to_the_key_start!=NULL){(*estimated_fov_arcmin)=180.0;return 0;}
// TELESCOP= 'HST'
// HST/WFC3 Field of View: 123 x 136 arcsec; Pixel Size: 0.13 arcsec
 pointer_to_the_key_start=(char *)memmem( telescop, strlen(telescop), "HST", 3);
 if(pointer_to_the_key_start!=NULL){(*estimated_fov_arcmin)=2.0;return 0;}

 return 1; // failed to recognize the telescope
}

int look_for_focallen_keyword(char *fitsfilename, double * estimated_fov_arcmin){
 // fitsio 
 int status = 0;
 double internal_estimated_fov_arcmin;
 fitsfile *fptr; /* pointer to the FITS file; defined in fitsio.h */
 double focallen;
 // Extract data from fits header
 fits_open_file(&fptr, fitsfilename, READONLY, &status);
 if ( 0!=status ){
  fits_report_error(stderr, status);  // print out any error messages
  fits_clear_errmsg(); // clear the CFITSIO error message stack
  return status;
 }
 // Assuming FOCALLEN keyword reports focal length in mm
 fits_read_key(fptr, TDOUBLE, "FOCALLEN", &focallen, NULL, &status);
 if ( 0!=status ){
  // Try another one
  status=0;
  fits_clear_errmsg(); // clear the CFITSIO error message stack
  fits_read_key(fptr, TDOUBLE, "FOC_LEN", &focallen, NULL, &status);
  if ( 0!=status ){
   //fits_report_error(stderr, status);  // print out any error messages
   fits_clear_errmsg(); // clear the CFITSIO error message stack
   return status;
  }
 } 
 
 // Check if the value of focallen is within reasonable limits
 if( focallen<20 || focallen>20000 )return 1;

 // 14mm is a typical minor axis size of a CCD chip 
 internal_estimated_fov_arcmin=atan2(14.0,focallen)*60.0*180.0/M_PI;

 internal_estimated_fov_arcmin=internal_estimated_fov_arcmin-0.1*internal_estimated_fov_arcmin;
 
 if( internal_estimated_fov_arcmin>5.0 && internal_estimated_fov_arcmin<2000.0 ){
  (*estimated_fov_arcmin)=internal_estimated_fov_arcmin;
  return 0;
 }
 else
  return 1;
}

int look_for_existing_wcs_header(char *fitsfilename, double * estimated_fov_arcmin){
 // fitsio 
 int status = 0;
 fitsfile *fptr; /* pointer to the FITS file; defined in fitsio.h */
 long naxes[2];
 
 int i; // stupid counter
 int No_of_wcs_keys; // number of keys in header
 int wcs_keys_left;
 char **wcs_key;
 
 //
 double scale_arcsec_pix=0.0;
 
 //
 double CDELT1;

 //
 double xrefval, yrefval, xrefpix, yrefpix, xinc, yinc, rot;
 char coordtype[512];

 //
 double internal_estimated_fov_arcmin;
  
 // Extract data from fits header
 fits_open_file(&fptr, fitsfilename, READONLY, &status);
 if ( 0!=status ){
  fits_report_error(stderr, status);  // print out any error messages
  fits_clear_errmsg(); // clear the CFITSIO error message stack
  return status;
 }
 fits_read_key(fptr, TLONG, "NAXIS1", &naxes[0], NULL, &status);
 if ( 0!=status ){
  fits_report_error(stderr, status);  // print out any error messages
  fits_clear_errmsg(); // clear the CFITSIO error message stack
  return status;
 } 
 fits_read_key(fptr, TLONG, "NAXIS2", &naxes[1], NULL, &status);
 if ( 0!=status ){
  fits_report_error(stderr, status);  // print out any error messages
  fits_clear_errmsg(); // clear the CFITSIO error message stack
  return status;
 }
 CDELT1=0.0;
 fits_read_key(fptr, TDOUBLE, "CDELT1", &CDELT1, NULL, &status);
 if ( 0!=status ){
  status=0;
  CDELT1=0.0; // OK, no such keyword, will indicate this by simple setting CDELT1=0.0
 }
 // Try to get WCS info using the built-in CFITSIO mechanism
 xinc=0.0;
 fits_read_img_coord(fptr, &xrefval, &yrefval, &xrefpix, &yrefpix, &xinc, &yinc, &rot, coordtype, &status);
 if( status==APPROX_WCS_KEY )status=0; // we are fine with approximate WCS key values
 if( status!=0 ){
  xinc=0.0;
  status=0; // reset status
 }
               
 
 fits_get_hdrspace(fptr,&No_of_wcs_keys,&wcs_keys_left,&status);
 wcs_key=malloc(No_of_wcs_keys*sizeof(char *));
 for(i=1;i<No_of_wcs_keys;i++){
  wcs_key[i]=(char *)malloc(80*sizeof(char));
  fits_read_record(fptr,i,wcs_key[i],&status);
 }
 fits_close_file(fptr, &status); // close file

 // Look for the COMMENT that is inseted by the Astrometry.net software
 for(i=1;i<No_of_wcs_keys;i++){
  if( wcs_key[i][0]=='C' && wcs_key[i][1]=='O' && wcs_key[i][2]=='M' && wcs_key[i][3]=='M' && wcs_key[i][4]=='E' && wcs_key[i][5]=='N' && wcs_key[i][6]=='T' ){
   //
   //fprintf(stderr,"%s\n",wcs_key[i]);
   //
   if( strlen(wcs_key[i])<23 )continue;
   if( wcs_key[i][7]==' ' && wcs_key[i][8]=='s' && wcs_key[i][9]=='c' && wcs_key[i][10]=='a' && wcs_key[i][11]=='l' && wcs_key[i][12]=='e' && wcs_key[i][13]==':')
   // COMMENT scale: 8.38108 arcsec/pix
   if( 1==sscanf(wcs_key[i],"COMMENT scale: %lf arcsec/pix",&scale_arcsec_pix) ){
    (*estimated_fov_arcmin)=scale_arcsec_pix*MIN(naxes[0],naxes[1])/60.0;
    (*estimated_fov_arcmin)=(*estimated_fov_arcmin)-0.1*(*estimated_fov_arcmin);
    return 0;
   }
  }
 }
 
 // If the above didn't help, try the new CD*_* convention
 // fits_read_img_coord() may reset xinc to 1.0 in case of failure ????
 if( xinc!=0.0 && xinc!=1.0 ){
  internal_estimated_fov_arcmin=fabs(60.0*xinc*(double)naxes[0]);
  internal_estimated_fov_arcmin=internal_estimated_fov_arcmin-0.1*internal_estimated_fov_arcmin;
  // Check the guessed FOV against the expected range
  if( MIN_FOV_ARCMIN<internal_estimated_fov_arcmin && internal_estimated_fov_arcmin<MAX_FOV_ARCMIN ){
   (*estimated_fov_arcmin)=internal_estimated_fov_arcmin;
   return 0;
  }
 }


 // If the above didn't help, try CDELT1*NAXIS1
 if( CDELT1!=0.0 ){
  internal_estimated_fov_arcmin=fabs(60.0*CDELT1*(double)naxes[0]);
  internal_estimated_fov_arcmin=internal_estimated_fov_arcmin-0.1*internal_estimated_fov_arcmin;
  // Check the guessed FOV against the expected range
  if( MIN_FOV_ARCMIN<internal_estimated_fov_arcmin && internal_estimated_fov_arcmin<MAX_FOV_ARCMIN ){
   (*estimated_fov_arcmin)=internal_estimated_fov_arcmin;
   return 0;
  }
 }

 
 //fprintf(stderr,"AAAA xinc=%lf CDELT1=%lf (*estimated_fov_arcmin)=%lf\n",xinc,CDELT1,(*estimated_fov_arcmin));
 
 return 1;
}

int main(int argc, char **argv){

 FILE * image_details_logfile;

 char fitsfile_name[FILENAME_LENGTH];
 
 double estimated_fov_arcmin=40.0; // default value
 
 // First test: reading vast_image_details.log
 char exp_start_date[MAX_LOG_STR_LENGTH];
 char exp_start_time[MAX_LOG_STR_LENGTH];
 char status[MAX_LOG_STR_LENGTH];
 char full_path_to_fits_image[MAX_LOG_STR_LENGTH];
 double exp,jd,ap,rotation;
 int detected,matched;
 //
 //char name_of_wcs_solved_image[MAX_LOG_STR_LENGTH];
 char name_of_wcs_solved_reference_image[MAX_LOG_STR_LENGTH];
 char name_of_fits_image[FILENAME_LENGTH];
 //

 if( argc != 2 ){
  fprintf(stderr,"This program will make a naive attempt to guess image field of view in arcminutes based on various circumstantial evidence.\n Usage: %s my_image.fits\n",argv[0]);
  return 1;
 }

 strncpy(fitsfile_name,argv[1],FILENAME_LENGTH);
 fitsfile_name[FILENAME_LENGTH-1]='\0';
 if(0!=fitsfile_read_check(fitsfile_name)){
  fprintf(stderr,"ERROR: %s is not a readbale FITS image!\n",fitsfile_name);
  return 1;
 }
 
 // First test see if there is a WCS header in the image itself?
 if( 0==look_for_existing_wcs_header( fitsfile_name, &estimated_fov_arcmin) ){
  fprintf(stdout,"%4.0lf\n",estimated_fov_arcmin);
  return 0;
 }
 
 // Second test: try to see if the reference image from this series has already been solved
 image_details_logfile=fopen("vast_image_details.log","r");
 if( NULL!=image_details_logfile ){
  fscanf(image_details_logfile,"exp_start= %s %s  exp= %lf  JD= %lf  ap= %lf  rotation= %lf  *detected= %d  *matched= %d  status=%s  %s",exp_start_date,exp_start_time,&exp,&jd,&ap,&rotation,&detected,&matched,status,full_path_to_fits_image);
  // If the current image is not the reference image
  if( 0!=strncmp(fitsfile_name,full_path_to_fits_image,FILENAME_LENGTH) ){
   strncpy(name_of_fits_image,basename(full_path_to_fits_image),FILENAME_LENGTH);
   fitsfile_name[FILENAME_LENGTH-1]='\0';
   if(strlen(name_of_fits_image)<5){fprintf(stderr,"ERROR: too short image name: %s\n",name_of_fits_image);return 1;}
   if( name_of_fits_image[0]=='w' && name_of_fits_image[1]=='c' && name_of_fits_image[2]=='s' && name_of_fits_image[3]=='_' )
    sprintf(name_of_wcs_solved_reference_image,"%s",name_of_fits_image);
   else
    sprintf(name_of_wcs_solved_reference_image,"wcs_%s",name_of_fits_image);
   name_of_wcs_solved_reference_image[FILENAME_LENGTH-1]='\0';
   // If the wcs-solved reference image is available
   if(0==fitsfile_read_check(name_of_wcs_solved_reference_image)){
    // If the current image is from the same image series as the reference image
    fseek(image_details_logfile,0,SEEK_SET); // go back to the beginning of the log file
    while(0<fscanf(image_details_logfile,"exp_start= %s %s  exp= %lf  JD= %lf  ap= %lf  rotation= %lf  *detected= %d  *matched= %d  status=%s  %s\n",exp_start_date,exp_start_time,&exp,&jd,&ap,&rotation,&detected,&matched,status,full_path_to_fits_image)){
     if( 0==strncmp(fitsfile_name,full_path_to_fits_image,FILENAME_LENGTH) ){
      //fprintf(stderr,"Oh yeah, %s is found in vast_image_details.log and this is not the reference image!\n",fitsfile_name);
      if( 0==look_for_existing_wcs_header(name_of_wcs_solved_reference_image, &estimated_fov_arcmin) ){
       fprintf(stdout,"%4.0lf\n",estimated_fov_arcmin);
       fclose(image_details_logfile);
       return 0;
      }
      break;
     }
    } // while(0<fscanf(image_details_logfile...
   } // if(0==fitsfile_read_check(name_of_wcs_solved_reference_image)){
  } // if( 0!=strncmp(fitsfile_name,full_path_to_fits_image,FILENAME_LENGTH) ){
  fclose(image_details_logfile);
 }

 if( 0==look_for_focallen_keyword(fitsfile_name, &estimated_fov_arcmin) ){
  fprintf(stdout,"%4.0lf\n",estimated_fov_arcmin);
  return 0;
 }

 if( 0==try_to_recognize_telescop_keyword(fitsfile_name, &estimated_fov_arcmin) ){
  fprintf(stdout,"%4.0lf\n",estimated_fov_arcmin);
  return 0;
 }
 
 // print out the default value anyhow
 fprintf(stdout,"%4.0lf\n",estimated_fov_arcmin);

 return 1;
}
