/*

 vast.c - photometric reduction of images with SExtractor.

 This file is part of VaST -
 a SExtractor front-end for search of variable objects in series of FITS images.

 Copyleft 2005-2015  Sokolovsky Kirill <kirx@scan.sai.msu.ru>,
                     Lebedev Alexandr  <lebastr@gmail.com>,
                     Nasonov Dmitry    
                     Nazarov Sergey 

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/****************** Include files ******************/

// Standard header files
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <math.h>
#include <time.h> /* to measure execution time */
#include <getopt.h>
#include <libgen.h> /* for basename() */
#include <unistd.h> /* for sleep() and getpid() */
#include <sys/types.h> /* for getpid() */
#include <sys/wait.h> /* defines WEXITED */
#include <sys/resource.h>
#include <sys/stat.h> /* for stat(), also requires #include <sys/types.h> and #include <unistd.h> */
#include <dirent.h> /* to work with directories */

// GSL header files
#include <gsl/gsl_statistics.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_errno.h>

// CFITSIO
#include "fitsio.h" // we use a local copy of this file because we use a local copy of cfitsio

// VaST's own header files
#include "ident.h"
#include "limits.h"
#include "vast_report_memory_error.h"
#include "photocurve.h"
#include "get_number_of_cpu_cores.h"
#include "fit_plane_lin.h"
#include "fitsfile_read_check.h"
#include "wpolyfit.h"
#include "replace_file_with_symlink_if_filename_contains_white_spaces.h"
#include "lightcurve_io.h"
/****************** Auxiliary functions ******************/

//#define debug_x 586.696
//#define debug_y 438.463


void version(char *version_string){
 strcpy(version_string,"VaST 1.0rc72");
 return;
}

void print_vast_version(void){
 char version_string[128];
 version(version_string);
 fprintf(stderr, "\n--==%s==--\n\n",version_string);
 return;
}

void hello_msg() {

        int n;

	fprintf(stderr,"\n");
        fprintf(stderr,"\n");

        fprintf(stderr,"              #########   \E[34;47mATTENTION!\E[33;00m   #########              \n");
        fprintf(stderr,"According to IAU recommendations (Resolution B1 XXIII IAU GA,\n");
        fprintf(stderr,"see http://www.iers.org/IERS/EN/Science/Recommendations/resolutionB1.html for details)  \n");
        fprintf(stderr,"Julian Dates computed by VaST will be expressed by default    \n");
        fprintf(stderr,"in \E[34;47mTerrestrial Time (TT)\E[33;00m! ");
        fprintf(stderr,"Starting from July 1, 2015:\n  TT = UTC + 68.184 sec  \n");
        fprintf(stderr,"If you want Julian Dates to be expressed in UTC,\nuse VaST with -u or --UTC key:   ./vast -u\n");
        fprintf(stderr,"Information which time system is used may be found in the log\n");
        fprintf(stderr,"file vast_summary.log\n");
        fprintf(stderr,"If you need precise timing, don't forget to apply \nHeliocentric Correction to the final lightcurve!\nHave fun! =)\n");

        fprintf(stderr,"\n");
        fprintf(stderr,"\n");
                

	fprintf(stderr, "This warning message will disappear in...   ");
	/* sleep for 6 seconds to make sure user saw the message */
	for( n=5; n>0 ; n--){
	 sleep(1);
	 fprintf(stderr,"%d ", n);
	}
	sleep(1);
	fprintf(stderr,"NOW!\n");


}


/* help_msg(const char* progname, int exit_code) - print help */
void help_msg(const char* progname, int exit_code) {
	fprintf(stdout, "Usage: %s [options] image1.fit image2.fit ... imageN.fit\n\n", progname);
	printf("Options:\n");
	printf("  -h or --help       print this message\n");
	printf("  -9 or --ds9        use DS9 instead of pgfv to view FITS files\n");
	printf("  -f or --nofind     run ./find_candidates manually\n");
	printf("  -d or --debug      run in debug (verbose) mode\n");
	printf("  -t or --time       use ANOVA & BLS period search algorithms\n");
//	printf("  -s or --small      use small comparison window (1 pix, useful for most CCD images)\n");
//	printf("  -m or --medium     use medium comparison window (6 pix)\n");
//	printf("  -w or --wide       use wide comparison window (10 pix, useful for scanned photographic plates)\n");
	printf("  -p or --poly       DO NOT use polynomial magnitude calibration (useful for good quality CCD images)\n");
	printf("  -o or --photocurve use formulas (1) and (3) from Bacher et al. (2005, MNRAS, 362, 542) for \n");
	printf("                     magnitude calibration. Useful for photographic data.\n");
	printf("  -P                 PSF photometry mode with SExtractor and PSFEx (experimental!)\n");
        printf("  -r or --norotation assume the input images are not rotated by more than 3 deg. w.r.t. the first (reference) one.\n");
	printf("  -l or --filter     use sigma-clip filter\n");
	printf("  -e or --failsafe   FAILSAFE mode. Only stars detected on the reference frame will be processed.\n");
	printf("  -u or --UTC        always assume UTC time system, don't perform conversion to TT.\n");
	printf("  -k or --nojdkeyword  ignore \"JD\" keyword in FITS image header. Time of observation will be taken from the usual keywords instead.\n");
        printf("  -a 5.0 or --aperture 5.0  use fixed aperture (e.g. 5 pixels) in diameter.\n");
        printf("  -b 200 or --matchstarnumber 200  use 200 (or whatever) reference stars for image matching.\n");
        printf("  -y 3 or --sysrem 3 conduct a few (e.g. 3) iterations of SysRem\n");
        printf("  -x 2 or --maxsextractorflag 2 accept stars with flag <=2 (2 means 'accept blended stars')\n");
        printf("  -j or --position_dependent_correction    use position-dependent magnitude correction (recommended for wide-field images).\n");
        printf("  -J or --no_position_dependent_correction DO NOT use position-dependent magnitude correction (recommended for narrow-field images with not too many stars on them).\n");
        printf("  -g or --guess_saturation_limit try to guess image saturation limit based on the brightest pixels found in the image.\n");
        printf("  -G or --no_guess_saturation_limit DO NOT try to guess image saturation limit based on the brightest pixels found in the image.\n");
	printf("\nExamples:\n");
	printf("  ./vast ../data/ccd_image-001.fit ../data/ccd_image-*.fit       # Typical CCD image reduction.\n");
	printf("  ./vast --UTC ../data/ccd_image-001.fit ../data/ccd_image-*.fit # CCD image reduction, UTC time will be used instead of TT.\n");
	printf("  ./vast -y 1 ../data/ccd_image-001.fit ../data/ccd_image-*.fit  # CCD image reduction with one SysRem iteration.\n");

	hello_msg();
	
	exit(exit_code);
}

// This function will check if the input SExtractor catalog is produced in the PSF-fitting mode
// and if it is, will filter-out N_DROP_WORST_PSF_FITS detections from the catalog
void filter_out_bad_psf_fits_from_catalog(char *sextractor_catalog_filename, int * number_of_filtered_out_detections){
 FILE *sextractor_catalog;

 int NUMBER,FLAGS,IMAFLAGS_ISO;
 double double_FLAGS,double_IMAFLAGS_ISO;
 double FLUX_PSF,FLUXERR_PSF,MAG_PSF,MAGERR_PSF,XPSF_IMAGE,YPSF_IMAGE,A_IMAGE,ERRA_IMAGE,B_IMAGE,ERRB_IMAGE,CHI2_PSF;
 
 size_t * p; // for index sorting
 
 char sextractor_catalog_string[MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT];
 char **sextractor_catalog_string_array;

 double *chi2array;
 double *m;
 double data[2*N_POINTS_PSF_FIT_QUALITY_FILTER+1];
 
 int i,j,k,N,N_good;
 double median;
 
 char sextractor_catalog_filename_tmp[512];

 int format_with_flags=1; // 1 - there is an external flag column, 0 - no external flag column
 
 double_FLAGS=double_IMAFLAGS_ISO=0.0;

 //fprintf(stderr,"DEBUG01\n");
 
 // open the catalog file
 sextractor_catalog=fopen(sextractor_catalog_filename,"r");
 if( NULL==sextractor_catalog ){fprintf(stderr,"ERROR: cannot open SExtractor catalog file %s\n",sextractor_catalog_filename);(*number_of_filtered_out_detections)=0;return;}
 //fprintf(stderr,"DEBUG02\n");
 // ### We assume Chi2 column is there ###
 // The question is if there is image flag column?
 // Determine if the catalog has Chi2 column produced during the PSF fitting stage, exit if not
 if( NULL==fgets(sextractor_catalog_string,MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT,sextractor_catalog) ){fclose(sextractor_catalog);(*number_of_filtered_out_detections)=0;return;}
 if(14!=sscanf(sextractor_catalog_string,"%d  %lf %lf %lf %lf  %lf %lf  %lf %lf %lf %lf %lf %lf %lf", &NUMBER, &FLUX_PSF, &FLUXERR_PSF, &MAG_PSF, &MAGERR_PSF, &XPSF_IMAGE, &YPSF_IMAGE, &A_IMAGE, &ERRA_IMAGE, &B_IMAGE, &ERRB_IMAGE, &double_FLAGS,&double_IMAFLAGS_ISO, &CHI2_PSF)){
  if(13!=sscanf(sextractor_catalog_string,"%d  %lf %lf %lf %lf  %lf %lf  %lf %lf %lf %lf %lf %lf", &NUMBER, &FLUX_PSF, &FLUXERR_PSF, &MAG_PSF, &MAGERR_PSF, &XPSF_IMAGE, &YPSF_IMAGE, &A_IMAGE, &ERRA_IMAGE, &B_IMAGE, &ERRB_IMAGE, &double_FLAGS, &CHI2_PSF)){
   fclose(sextractor_catalog);
   (*number_of_filtered_out_detections)=0;
   return;
  }
  else{
   format_with_flags=0;
  }
 }
 FLAGS=double_FLAGS;IMAFLAGS_ISO=double_IMAFLAGS_ISO; // This is to avoid "%d %lg" combination in sscanf() above 
 //fprintf(stderr,"\n\n\nDEBUG03  FLAGS=%d IMAFLAGS_ISO=%d  CHI2_PSF=%lg\n",FLAGS,IMAFLAGS_ISO,CHI2_PSF);
 //fprintf(stderr,"DEBUG03a  %d  %lf %lf %lf %lf  %lf %lf  %lf %lf %lf %lf %d %d %lg\n\n\n", NUMBER, FLUX_PSF, FLUXERR_PSF, MAG_PSF, MAGERR_PSF, XPSF_IMAGE, YPSF_IMAGE, A_IMAGE, ERRA_IMAGE, B_IMAGE, ERRB_IMAGE, FLAGS,IMAFLAGS_ISO, CHI2_PSF);
 // count lines in the catalog
 i=0;
 while( NULL!=fgets(sextractor_catalog_string,MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT,sextractor_catalog) )i++;
 i++; // account for the line we read with fscanf()
 N=i;
 //fprintf(stderr,"DEBUG04\n");
 fseek(sextractor_catalog,0,SEEK_SET); // go back to the beginning of the file
 //fprintf(stderr,"DEBUG05\n");
 p=malloc(N*sizeof(size_t));
 //fprintf(stderr,"DEBUG06\n");
 chi2array=malloc(N*sizeof(double));
 m=malloc(N*sizeof(double));
 //fprintf(stderr,"DEBUG07\n");
 sextractor_catalog_string_array=(char **)malloc(N*sizeof(char **));
 //fprintf(stderr,"DEBUG08\n");
 for(i=0;i<N;i++){
  //fprintf(stderr,"DEBUG08.5 i=%d N=%d\n",i,N);
  sextractor_catalog_string_array[i]=(char *)malloc(MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT*sizeof(char));
 }
 //fprintf(stderr,"DEBUG09\n");
 N_good=0;
 while( NULL!=fgets(sextractor_catalog_string,MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT,sextractor_catalog) ){
  sextractor_catalog_string[MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT-1]='\0'; // just in case
  if( format_with_flags==1 ){
   if(14!=sscanf(sextractor_catalog_string,"%d  %lf %lf %lf %lf  %lf %lf  %lf %lf %lf %lf %lf %lf %lf",&NUMBER, &FLUX_PSF,&FLUXERR_PSF,&MAG_PSF,&MAGERR_PSF,&XPSF_IMAGE,&YPSF_IMAGE,&A_IMAGE,&ERRA_IMAGE,&B_IMAGE,&ERRB_IMAGE, &double_FLAGS,&double_IMAFLAGS_ISO, &CHI2_PSF))continue;
  }
  else{
   double_IMAFLAGS_ISO=1.0;
   if(13!=sscanf(sextractor_catalog_string,"%d  %lf %lf %lf %lf  %lf %lf  %lf %lf %lf %lf %lf %lf", &NUMBER, &FLUX_PSF, &FLUXERR_PSF, &MAG_PSF, &MAGERR_PSF, &XPSF_IMAGE, &YPSF_IMAGE, &A_IMAGE, &ERRA_IMAGE, &B_IMAGE, &ERRB_IMAGE, &double_FLAGS, &CHI2_PSF))continue;
  }
  if( CHI2_PSF==0.0 )continue; // filter-out the very bad fits right away
  FLAGS=double_FLAGS;IMAFLAGS_ISO=double_IMAFLAGS_ISO; // This is to avoid "%d %lg" combination in sscanf() above
  chi2array[N_good]=CHI2_PSF;
  m[N_good]=MAG_PSF;
  strncpy(sextractor_catalog_string_array[N_good],sextractor_catalog_string,MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT);
  //fprintf(stderr,"%d %lg %s",N_good,chi2array,sextractor_catalog_string_array[N_good]);
  N_good++;
 }
 // close the catalog file
 fclose(sextractor_catalog);

 //gsl_sort_index( p, chi2array, 1, N_good); // The elements of p give the index of the array element which would have been stored in that position if the array had been sorted in place. The array data is not changed.
 gsl_sort_index( p, m, 1, N_good); // The elements of p give the index of the array element which would have been stored in that position if the array had been sorted in place. The array data is not changed.

 strncpy(sextractor_catalog_filename_tmp,sextractor_catalog_filename,512);
 sextractor_catalog_filename_tmp[511]='\0'; // just in case
 strncat(sextractor_catalog_filename_tmp,".tmp",512);
 sextractor_catalog_filename_tmp[511]='\0'; // just in case
 // open catalog for writing
 sextractor_catalog=fopen(sextractor_catalog_filename_tmp,"w");

 (*number_of_filtered_out_detections)=0;
 for(i=0;i<N_good;i++){
  for( k=0, j=MAX(0,i-N_POINTS_PSF_FIT_QUALITY_FILTER); j<MIN(i+N_POINTS_PSF_FIT_QUALITY_FILTER,N_good); j++ ){
   data[k]=chi2array[p[j]];
   k++;
  }
  gsl_sort(data,1,k);
  median=data[2]; // take the tird smallest point
  //median=gsl_stats_median_from_sorted_data(data,1,k);
  //fprintf(stderr,"%lf %lg %lg\n",m[p[i]],chi2array[p[i]],median);
  if( chi2array[p[i]]<5*median )
   fputs(sextractor_catalog_string_array[p[i]], sextractor_catalog);
  else
   (*number_of_filtered_out_detections)++;
 }

/*
 for(i=0;i<N_good-(int)(N_DROP_WORST_PSF_FITS*(double)N_good);i++){
//  fputs(sextractor_catalog_string_array[p[i]], stderr);
  fputs(sextractor_catalog_string_array[p[i]], sextractor_catalog);
 }
*/
 //fprintf(stderr,"%d %d  \n",N_good,(int)(N_DROP_WORST_PSF_FITS*(double)N_good));
 //fprintf(stderr,"%d %d  %s",i,p[i],sextractor_catalog_string_array[p[i]]);
 fclose(sextractor_catalog);

 //exit(1); /// !!!

 unlink(sextractor_catalog_filename);
 rename(sextractor_catalog_filename_tmp, sextractor_catalog_filename);

 // Free-up the memory 
 free(m);
 free(chi2array);
 for(i=0;i<N;i++)free(sextractor_catalog_string_array[i]);
 free(sextractor_catalog_string_array);
 free(p);

 //(*number_of_filtered_out_detections)=N_good-(N_good-(int)(N_DROP_WORST_PSF_FITS*(double)N_good)); 

 //exit(1); // !!!!

 return;
}


/*
 This function is useful for debugging. It will create a DS9 region file from an rray of structures (type struct Star)
 containing a list of stars.
 */
void write_Star_struct_to_ds9_region_file(struct Star *star, int N_start, int N_stop, char *filename, double aperture){
 int i;
 FILE *f;
 f=fopen(filename,"w");
 fprintf(f,"# Region file format: DS9 version 4.0\n");
 fprintf(f,"# Filename:\n");
 fprintf(f,"global color=green font=\"sans 10 normal\" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source\n");
 fprintf(f,"image\n");
 for(i=N_start;i<N_stop;i++){
  fprintf(f,"circle(%f,%f,%lf)\n",star[i].x_frame,star[i].y_frame,aperture*0.5);///2.0);
 }
 fclose(f);
 return;
}

void write_single_Star_from_struct_to_ds9_region_file(struct Star *star, int N_start, int N_stop, char *filename, double aperture){
 int i;
 FILE *f;
 // try to open the file
 f=fopen(filename,"r");
 if(f==NULL){
  // write header
  f=fopen(filename,"w");
  fprintf(f,"# Region file format: DS9 version 4.0\n");
  fprintf(f,"# Filename:\n");
  fprintf(f,"global color=green font=\"sans 10 normal\" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source\n");
  fprintf(f,"image\n");
 }
 fclose(f);
 f=fopen(filename,"a");
 for(i=N_start;i<N_stop;i++){
  fprintf(f,"circle(%f,%f,%lf)\n",star[i].x_frame,star[i].y_frame,aperture*0.5);///2.0);
 }
 fclose(f);
 return;
}


/*
 This function is useful for debugging.
 */
void write_Star_struct_to_ASCII_file(struct Star *star, int N_start, int N_stop, char *filename, double aperture){
 int i;
 FILE *f;
 f=fopen(filename,"w");
 for(i=N_start;i<N_stop;i++){
  fprintf(f,"%f  %f   %lf\n",star[i].x,star[i].y,aperture*0.5);///2.0);
 }
 fclose(f);
 return;
}


/*
 Get the compiler version, just for the sake of bookkeeping...
 */
void compiler_version(char *compiler_version_string){
 FILE *cc_version_file;
 cc_version_file=fopen(".cc.version","r");
 if( NULL==cc_version_file ){strcpy(compiler_version_string,"unknown compiler\n");return;}
 fgets(compiler_version_string, 256, cc_version_file);
 fclose(cc_version_file);
 return;
}


/* is_file() - a small function which checks is an input string is a name of a readable file */
int is_file(char *filename){
 FILE *f=NULL;
 f=fopen(filename,"r");
 if( f==NULL )return 0;
 else{
  fclose(f);
  return 1;
 }
}  

/*
   This function will write vast_images_catalogs.log file
*/
void write_images_catalogs_logfile(char **filelist, int n){
 FILE *f;
 int i;
 f=fopen("vast_images_catalogs.log","w");
 for(i=0;i<n;i++)
  fprintf(f,"image%05d.cat %s\n",i+1,filelist[i]);
 fclose(f);
 return;
}

/* Write data on magnitude calibration to the log file */
void write_magnitude_calibration_log(double *mag1, double *mag2, double *mag_err,int N, char *fitsimagename){
 char logfilename[FILENAME_LENGTH];
 FILE *logfile;
 int i;
 strncpy(logfilename,basename(fitsimagename),FILENAME_LENGTH);logfilename[FILENAME_LENGTH-1]='\0';
 for(i=(int)strlen(logfilename)-1;i--;){
  if( logfilename[i]=='.' ){
   logfilename[i]='\0';
   break;
  }
 }
 strcat(logfilename,".calib");
 logfile=fopen(logfilename,"w");
 if( NULL==logfile ){
  fprintf(stderr,"WARNING: can't open %s for writing!\n",logfilename);
  return;
 }
 for(i=0;i<N;i++){
  fprintf(logfile,"%8.4lf %8.4lf %.4lf\n",mag1[i],mag2[i],mag_err[i]);
 }
 fclose(logfile);
 return;
}

void write_magnitude_calibration_log2(double *mag1, double *mag2, double *mag_err,int N, char *fitsimagename){
 char logfilename[FILENAME_LENGTH];
 FILE *logfile;
 int i;
 strncpy(logfilename,basename(fitsimagename),FILENAME_LENGTH);logfilename[FILENAME_LENGTH-1]='\0';
 for(i=(int)strlen(logfilename)-1;i--;){
  if( logfilename[i]=='.' ){
   logfilename[i]='\0';
   break;
  }
 }
 strcat(logfilename,".calib2");
 logfile=fopen(logfilename,"w");
 if( NULL==logfile ){
  fprintf(stderr,"WARNING: can't open %s for writing!\n",logfilename);
  return;
 }
 for(i=0;i<N;i++){
  fprintf(logfile,"%8.4lf %8.4lf %.4lf\n",mag1[i],mag2[i],mag_err[i]);
 }
 fclose(logfile);
 return;
}

void write_magnitude_calibration_log_plane(double *mag1, double *mag2, double *mag_err,int N, char *fitsimagename, double A, double B, double C){
 char logfilename[FILENAME_LENGTH];
 FILE *logfile;
 int i;
 if( strlen(fitsimagename)<1 ){fprintf(stderr,"WARNING from write_magnitude_calibration_log_plane(): cannot get FITS image filename!\n");return;}
 strncpy(logfilename,basename(fitsimagename),FILENAME_LENGTH);logfilename[FILENAME_LENGTH-1]='\0';
 for(i=(int)strlen(logfilename)-1;i--;){
  if( logfilename[i]=='.' ){
   logfilename[i]='\0';
   break;
  }
 }
 strcat(logfilename,".calib_plane");
 logfile=fopen(logfilename,"w");
 if( NULL==logfile ){
  fprintf(stderr,"WARNING: can't open %s for writing!\n",logfilename);
  return;
 }
 for(i=0;i<N;i++){
  fprintf(logfile,"%8.4lf %8.4lf %.4lf\n",mag1[i],mag2[i],mag_err[i]);
 }
 fclose(logfile);
 strcat(logfilename,".calib_plane_param");
 logfile=fopen(logfilename,"w");
 if( NULL==logfile ){
  fprintf(stderr,"WARNING: can't open %s for writing!\n",logfilename);
  return;
 }
 fprintf(logfile,"%lf %lf %lf\n",A,B,C);
 fclose(logfile); 
 return;
}


/* Write parameters of magnitude calibration to another log file */
void write_magnitude_calibration_param_log(double *poly_coeff, char *fitsimagename){
 char logfilename[FILENAME_LENGTH];
 FILE *logfile;
 int i;
 strncpy(logfilename,basename(fitsimagename),FILENAME_LENGTH);logfilename[FILENAME_LENGTH-1]='\0';
 for(i=(int)strlen(logfilename)-1;i--;){
  if( logfilename[i]=='.' ){
   logfilename[i]='\0';
   break;
  }
 }
 strncat(logfilename,".calib_param",FILENAME_LENGTH);logfilename[FILENAME_LENGTH-1]='\0';
 logfile=fopen(logfilename,"w");
 if( NULL==logfile ){
  fprintf(stderr,"WARNING: can't open %s for writing!\n",logfilename);
  return;
 }
 fprintf(logfile,"%lf %lf %lf %lf %lf\n",poly_coeff[4],poly_coeff[3],poly_coeff[2],poly_coeff[1],poly_coeff[0]);
 fclose(logfile); 
 return;
}

/* New memory check - try to allocate lots of space */
int check_if_we_can_allocate_lots_of_memory(){
 char * big_chunk_of_memory;
 big_chunk_of_memory=malloc(134217728*sizeof(char)); // try to allocate 128MB
 if( NULL==big_chunk_of_memory ){fprintf(stderr,"WARNING: the system is low on memory!\n");return 1;}
 free(big_chunk_of_memory);
 return 0;
}

/* Memory check */
int check_and_print_memory_statistics(){
 FILE *meminfofile;
 char string1[256];
 char string2[256];
 double VmPeak=0.0;
 char VmPeak_units[256];
 double VmSize=0.0;
 char VmSize_units[256];
 double RAM_size=0.0;
 char RAM_size_units[256];
 double mem=0.0;
 pid_t pid;
 pid=getpid();
 
 /* Check if process status information is available in /proc */
 sprintf(string2,"/proc/%d/status",pid);
 if( 0==is_file(string2) ){fprintf(stderr,"can't read %s   no memory statistics available\n",string2);return 0;}
  
 /* Get ammount of used memory from /proc/PID/status */
 sprintf(string1,"grep -B1 VmSize %s |grep -v Groups > vast_memory_usage.log",string2);
 system(string1);

 /* Check if memory information is available in /proc */
 if( 0==is_file("/proc/meminfo") ){fprintf(stderr,"can't read /proc/meminfo   no memory statistics available\n");return 0;}
 
 /* Get RAM size */
 sprintf(string1,"grep MemTotal /proc/meminfo >> vast_memory_usage.log");
 system(string1);
 
 /* Load memory information from the log file */
 meminfofile=fopen("vast_memory_usage.log","r");
 if( meminfofile==NULL ){fprintf(stderr,"can't open vast_memory_usage.log, no memory statistics available\n");return 0;}
 if( 3!=fscanf(meminfofile,"%s %lf %s ",string1,&mem,string2) ){fprintf(stderr,"no memory statistics available\n");return 0;}
 if( 0==strcasecmp(string1,"VmPeak:") ){VmPeak=mem;strcpy(VmPeak_units,string2);}
 if( 0==strcasecmp(string1,"VmSize:") ){VmSize=mem;strcpy(VmSize_units,string2);}
 if( 0==strcasecmp(string1,"MemTotal:") ){RAM_size=mem;strcpy(RAM_size_units,string2);}
 if( 3==fscanf(meminfofile,"%s %lf %s ",string1,&mem,string2) ){
  if( 0==strcasecmp(string1,"VmPeak:") ){VmPeak=mem;strcpy(VmPeak_units,string2);}
  if( 0==strcasecmp(string1,"VmSize:") ){VmSize=mem;strcpy(VmSize_units,string2);}
  if( 0==strcasecmp(string1,"MemTotal:") ){RAM_size=mem;strcpy(RAM_size_units,string2);}
  if( 3==fscanf(meminfofile,"%s %lf %s ",string1,&mem,string2) ){
   if( 0==strcasecmp(string1,"VmPeak:") ){VmPeak=mem;strcpy(VmPeak_units,string2);}
   if( 0==strcasecmp(string1,"VmSize:") ){VmSize=mem;strcpy(VmSize_units,string2);}
   if( 0==strcasecmp(string1,"MemTotal:") ){RAM_size=mem;strcpy(RAM_size_units,string2);}
  }
 }
 fclose(meminfofile);
 
 /* Write information about memory usage */
 fprintf(stderr,"memory: ");
 if( 0.0!=VmSize )fprintf(stderr," %.0lf %s used",VmSize,VmSize_units);
 if( 0.0!=VmPeak )fprintf(stderr,", %.0lf %s peak",VmPeak,VmPeak_units);
 if( 0.0!=RAM_size )fprintf(stderr,", %.0lf %s available RAM",RAM_size,RAM_size_units);
 fprintf(stderr,"\n");

 /* If RAM and VmSize were correctly read and are in the same units... */
 if( 0==strcasecmp(VmSize_units,RAM_size_units) && 0!=VmSize && 0!=RAM_size ){
  /* Check aren't we using too much memory? */
  if( VmSize>MAX_RAM_USAGE*RAM_size ){
   fprintf(stderr,"\E[01;31mWARNING! VaST is using more than %d%% of RAM! Trying to free some memory...\E[33;00m\n",(int)(MAX_RAM_USAGE*100));
   return 1; // return value 1 means that we need to free some momory
  }
 }
 
 if( 0!=check_if_we_can_allocate_lots_of_memory() )return 1;

 return 0;
}
 
/* progress(int done, int all) - print out progress status, how many images were sextracted */
void progress(int done, int all) {
	fprintf(stderr, "processed %d of %d images (%5.1lf%%)\n", done, all, (double)done / (double)all*100.0);
	return;
}

/* save_command_line_to_log_file(int argc, char **argv) - save command line arguments to the log file vast_command_line.log */
void save_command_line_to_log_file(int argc, char **argv){
 int i;
 FILE *cmdlogfile;
 cmdlogfile=fopen("vast_command_line.log","w");
 for(i=0;i<argc;i++)
  fprintf(cmdlogfile,"%s ",argv[i]);
 fclose(cmdlogfile);
}

// a housekeeping function to exclude i'th element from three arrays
void exclude_from_3_double_arrays( double *array1, double *array2, double *array3, int i, int *N){
 int j;
 for(j=i;j<(*N)-1;j++){
  array1[j]=array1[j+1];
  array2[j]=array2[j+1];
  array3[j]=array3[j+1];
 }
 (*N)=(*N)-1;
 return;
}
// a housekeeping function to exclude i'th element from six arrays
void exclude_from_6_double_arrays( double *array1, double *array2, double *array3, double *array4, double *array5, double *array6, int i, int *N){
 int j;
 for(j=i;j<(*N)-1;j++){
  array1[j]=array1[j+1];
  array2[j]=array2[j+1];
  array3[j]=array3[j+1];
  array4[j]=array4[j+1];
  array5[j]=array5[j+1];
  array6[j]=array6[j+1];
 }
 (*N)=(*N)-1;
 return;
}
    

// This function writes a string to an image log file.
// vast_image_details.log will be created from image log files later...
void write_string_to_log_file(char *log_string, char *sextractor_catalog){
 FILE *vast_image_details;
 char vast_image_details_log_filename[256];
 int i;
 // Guess the log file name
 strcpy(vast_image_details_log_filename,sextractor_catalog);
 for(i=(int)strlen(vast_image_details_log_filename);i--;){
  if( vast_image_details_log_filename[i]=='.' ){
   vast_image_details_log_filename[i+1]='l';
   vast_image_details_log_filename[i+2]='o';
   vast_image_details_log_filename[i+3]='g';
   break;
  }
 }
 // Write the string
 vast_image_details=fopen(vast_image_details_log_filename,"a");
 if( vast_image_details==NULL ){fprintf(stderr,"ERROR appending line \"%s\" to log file \"%s\"\n",log_string,vast_image_details_log_filename);return;}
 fprintf(vast_image_details,"%s",log_string);
 fclose(vast_image_details);
 return;
}
     
int exclude_test(double X, double Y, double *exX, double *exY, int N) {
	int result = 0;
	int i;
	//for (i = 0; i < N; i++) {
	for (i = N; i--; ) {
		if (fabs(exX[i] - X) < 1.5 && fabs(exY[i] - Y) < 1.5) {
			result = 1;
			break;
		}
	}
	if (result != 0) {
		fprintf(stderr, "The star %lg %lg is listed in exclude.lst => excluded from magnitude calibration\n", X, Y);
	}
	return result;
}

/* Transients are objects which were not netected on the reference frame but have now suddenly appeared. */
void test_transient( double *search_area_boundaries, struct Star star, double reference_image_JD){
 FILE *transientfile;
 int n=star.n;
 double x=star.x;
 double y=star.y;
 double m=star.mag;
 double m_err=star.sigma_mag;
 
 // Test if the time difference between the reference and the current image is >TRANSIENT_MIN_TIMESCALE_DAYS
 if( fabs(star.JD-reference_image_JD)<TRANSIENT_MIN_TIMESCALE_DAYS )return;
 //if( star.n==4511 )fprintf(stderr,"##### %lf %lf\n",star.JD,reference_image_JD);
 
 if( x>search_area_boundaries[0] && x<search_area_boundaries[1] ){
  if( y>search_area_boundaries[2] && y<search_area_boundaries[3] ){
   if( m+1.0*m_err<search_area_boundaries[5] ){
    if( x<FRAME_EDGE_INDENT_PIXELS || y<FRAME_EDGE_INDENT_PIXELS )return;
    transientfile=fopen("candidates-transients.lst","a");
    if( NULL==transientfile ){fprintf(stderr,"ERROR writing to candidates-transients.lst\n");return;}
    fprintf(transientfile,"out%05d.dat  %8.3lf %8.3lf\n", n, x, y);
    fclose(transientfile);
   }
  }
 }
 return;
}

void set_transient_search_boundaries(double *search_area_boundaries, struct Star *star, int NUMBER){
 int i;
 search_area_boundaries[0]=search_area_boundaries[1]=search_area_boundaries[2]=search_area_boundaries[3]=FRAME_EDGE_INDENT_PIXELS;
 search_area_boundaries[4]=search_area_boundaries[5]=(double)star[0].mag;
 
 //for(i=0;i<NUMBER;i++){
 for(i=NUMBER;i--;){
  if( star[i].vast_flag!=0 )continue;
  if( star[i].mag!=0.0 ){ // just in case
   if( (double)star[i].x<search_area_boundaries[0] )search_area_boundaries[0]=(double)star[i].x;
   if( (double)star[i].x>search_area_boundaries[1] )search_area_boundaries[1]=(double)star[i].x;
   if( (double)star[i].y<search_area_boundaries[2] )search_area_boundaries[2]=(double)star[i].y;
   if( (double)star[i].y>search_area_boundaries[3] )search_area_boundaries[3]=(double)star[i].y;
   if( (double)star[i].mag<search_area_boundaries[4] )search_area_boundaries[4]=(double)star[i].mag;
   if( (double)star[i].mag>search_area_boundaries[5] )search_area_boundaries[5]=(double)star[i].mag;
  }
 }

 search_area_boundaries[5]=search_area_boundaries[5]-MAG_TRANSIENT_ABOVE_THE_REFERENCE_FRAME_LIMIT;
 
 fprintf(stderr,"\nParameter box for transient search: %lf<X<%lf %lf<Y<%lf %lf<m<%lf\n \n",search_area_boundaries[0],search_area_boundaries[1],search_area_boundaries[2],search_area_boundaries[3],search_area_boundaries[4],search_area_boundaries[5]);
 
 return;
}
/*
void replace_file_with_symlink_if_filename_contains_white_spaces(char * filename){
 int i,need_to_replace_with_symlink;
 char symlink[FILENAME_LENGTH];
 char command_string[2*FILENAME_LENGTH];
 struct stat sb; // structure returned by stat() system call
 for(need_to_replace_with_symlink=0,i=0;i<strlen(filename);i++){
  if( filename[i]==' ' ){
   need_to_replace_with_symlink=1;
   break;
  }
 }
 if( need_to_replace_with_symlink==0 )return;
 // create symlink name
 for(i=1;i<65535;i++){
  sprintf(symlink,"symlinks_to_images/symlink_to_image_%05d.fits",i);
  if(0!=stat(symlink, &sb))break; // continue is such a symlink already exists
 }
 if( i>=65535 ){fprintf(stderr,"ERROR: symlinks_to_images counter is out of range!\n");return;}
 //if( 0!=fitsfile_read_check(filename) ){
 // fprintf(stderr,"The input does not appear to be a FITS image: %s\n",filename);return;
 //}
 fprintf(stderr,"WARNING: image path \"%s\" contains white spaces - SExtractor will not be able to handle that!\nTrying to circumvent this problem by creating symbolic link %s\n",filename,symlink);
 
 sprintf(command_string,"if [ ! -d symlinks_to_images ];then mkdir symlinks_to_images ;fi ; TRUEPATH=`readlink -f '%s'` ; ln -s \"$TRUEPATH\" %s", filename, symlink);
 system(command_string);
 
 strncpy(filename,symlink,FILENAME_LENGTH-1);
 filename[FILENAME_LENGTH-1]='\0'; // just in case
 
 return;
}

*/
/****************** The main function ******************/

int main(int argc, char **argv){
/// Definitions begins  --==Плиз разложи вот это все по полочкам и напиши ко всем комменты==--
	/* Read_sex_cat2 */
	//int nnn;
	FILE *file;
	int star_number_in_sextractor_catalog, sextractor_flag;//, key;
	double flux_adu, flux_adu_err, position_x_pix, position_y_pix, mag, sigma_mag;
	/* end of Read_sex_cat2 */
	struct Preobr_Sk *preobr;
	//struct Frame frame;
	struct Star *STAR1 = NULL, *STAR2 = NULL, *STAR3 = NULL; // STAR1 - structure with all stars we can match
                                                                 //         new stars are added to STAR1
	                                                         // STAR2 - structure with stars on a current image
	                                                         // STAR3 - structure with stars on the reference image (to mutch frames)
	                                                         //         no new stars are added to STAR3
	char *starfile, **base_star;
	FILE *file_out;
	int n, Num = 0, i, j; // Counters. Num is number of files to operate
	int NUMBER1, NUMBER2, NUMBER3;
	int Number_of_ecv_star;
	int *Pos1, *Pos2;

	double JD = 0;
	char tmpNAME[OUTFILENAME_LENGTH]; /* Array to store generated lightcurve filenames */
	struct Observation * ptr_struct_Obs = NULL; // Structure to store all observations
	long TOTAL_OBS = 0;                         // Total number of measurements
	long obs_in_RAM = 0;                        // Number of observations which were not written to disk
	long Max_obs_in_RAM = MAX_MEASUREMENTS_IN_RAM; // maximum number of observations in RAM
	int N_good_stars = 0;
	int N_bad_stars = 0;
	double *bad_stars_X;
	double *bad_stars_Y;
	double X_im_size=0.0;
	double Y_im_size=0.0;
	double max_X_im_size=0.0; // new (We may have input images of different size and we need the largest size for image identifictaion) 
	double max_Y_im_size=0.0; // new
	double aperture;

	/* Variables to set special parameters */
	int fitsfile_read_error = 0; // returned by gettime
	int param_p = 1;	// do not calibrate mags by polynom
	int param_P = 0;	// PSF photometry mode (1 - do it; 2 - do usual aperture photometry)
	int param_w = 0;        // wide comparison window
	//int param_nocalib = 0;  // do not change magnitude scale
	int param_nofind = 0;   // do not run find_candidates
	int param_nofilter = 1; // do not run filtering
	int no_rotation = 0;    // count as error rotation larger than 3 degrees
	int debug = 0;          // be more verbose
	int period_search_switch = 0; // do not use period search algorithms 
	int use_ds9_instead_of_pgfv=0; // if 1 use ds9 instead of pgfv
	int param_failsafe = 0; // if 1 consider only stars that were detected on the reference frame,
	                        // as in older versions of VaST
        int cache_counter=0;    // is needed for the lightcurve cache in memory
        int number_of_sysrem_iteration=0; // if !=0 - use util/sysrem number_of_sysrem_iteration times
        int n_start=1; // sould be 0 if the increment mode is active

        int param_nojdkeyword=0; // 1 - ignore "JD" keyword in FITS image header, useful if sme junk is written in this keyword instead of the middle exposure time
        
        int maxsextractorflag=0; // Maximum star flag value set by sextractor acceptable for VaST
        
        int param_use_photocurve=0; // Use "photocurve" for magnitude calibration. See src/photocurve.c for details
        
        
	// poly_mag
	double *poly_x=NULL;
	double *poly_y=NULL;
	double *poly_err=NULL;
	double *poly_err_fake=NULL; // this is an array with fake error values that is used for unweighted fitting
	double poly_coeff[10];

	double faintest_stars = FAINTEST_STARS;
	
	
	int wpolyfit_exit_code=0;


        // Linear magnitude correction as a function of X and Y
	int apply_position_dependent_correction=0; // 1 - apply, 2 - do not apply
        int param_apply_position_dependent_correction=0; // determines if this parameter is specified in the command line
        double *lin_mag_cor_x=NULL;
        double *lin_mag_cor_y=NULL;
        double *lin_mag_cor_z=NULL;
        double lin_mag_A,lin_mag_B,lin_mag_C;
	
	int MATCH_SUCESS=0;
	
	
	/* LOG-file */
	FILE *vast_image_details; /* May refer to both vast_image_details.log and vast_summary.log when needed */
	
	char stderr_output[1024]; 
	char log_output[1024];
	
	
	int exclude_outlier_mags_counter;
	int the_baddest_outlier_number;
	double the_baddest_outlier;
	double abs_computed_predicted_mag_diff;
	double computed_mag;
	
	double fixed_aperture=0.0;  /* If fixed_aperture!=0.0 - use fixed aperture for all images */
	
	FILE *vast_list_of_all_stars_log;
	FILE *vast_list_of_all_stars_ds9;
	
	/* Hunt for transients */
	double search_area_boundaries[6]; // Xmin, Xmax, Ymin, Ymax, MAGmin, MAGmax
	
	double a_a; // semi-major axis lengths
	double a_a_err;
	double a_b; // semi-minor axis lengths
	double a_b_err;
	
	/* Execution time measurements */
	time_t start_time;
	time_t end_time;
	double elapsed_time;
	
	/* Time system */
	int timesys=0; // 0 - unknown
                       // 1 - UTC
                       // 2 - TT
	int convert_timesys_to_TT=1; // Convert JD from UTC to TT?
	                             // 0 - no
	                             // 1 - yes
	
	char tymesys_str[32];
	
	
	int match_try; // counter for multimple match trials
	int match_retry=0; // variable set by Ident
	int default_Number_of_main_star;
	int default_Number_of_ecv_triangle;
	int param_set_manually_Number_of_main_star=0;
	int success_match_on_increase=0;
        int best_number_of_matched_stars=0;
        int best_number_of_reference_stars=0;

	
	// Coordinate arrays
	int coordinate_array_counter;
	int *number_of_coordinate_measurements_for_star;
	int *star_numbers_for_coordinate_arrays;
	float **coordinate_array_x;
	float **coordinate_array_y;
	int i_update_coordinates_STAR3;
	

        int max_number; // Maximum star number, will be needed to make up numbers for new stars if we want to add them to the list
        
	//int xxx; // for test
	
	
        char sextractor_catalog[FILENAME_LENGTH];

        pid_t pid;
	
	int number_of_lightcurves_for_each_thread; // number of lightcurves to be written on disk by each parrallel thread
	
	int k; // a counter used to write down lightcurve files
	
	struct stat sb; // structure returned by stat() system call
        DIR *dp;
        struct dirent *ep;
        char dir_string[FILENAME_LENGTH];

        // The following variables are used to handle vast_list_of_input_images_with_time_corrections.txt        
        FILE *vast_list_of_input_images_with_time_corrections;
        char image_filename_from_input_list[FILENAME_LENGTH];
        double image_date_correction_from_input_list;
        // ------------------------------------------
        
        int j_poly_err; // a counter for checking that no element in the poly_err[] array is equal to 0.0,
                        // so no data point is super-weighted
	
	char sextractor_catalog_string[MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT];
	
	int guess_saturation_limit_operation_mode=2; // by default try to be smart and guess
	                                             // if we should guess the saturation limit 
	                                             // or use the one specified in default.sex
	int external_flag;
	double double_external_flag,psf_chi2;

        char external_flag_string[256];

        int counter_rejected_bad_flux,counter_rejected_low_snr,counter_rejected_bad_region,counter_rejected_frame_edge,counter_rejected_too_small,counter_rejected_too_large,counter_rejected_external_flag,counter_rejected_bad_psf_fit;

        int ii,jj; // for SExtractor catalog parsing

/// end of definitions
        lin_mag_A=lin_mag_B=lin_mag_C=0.0; // just in case

        //doesn't help
        //counter_rejected_bad_flux=counter_rejected_low_snr=counter_rejected_bad_region=counter_rejected_frame_edge=counter_rejected_too_small=counter_rejected_too_large=counter_rejected_external_flag=0; // reset bad star counters


        /* Protection against strange free() crashes */
        //setenv("MALLOC_CHECK_", "0", 1);

        print_vast_version();


        /* argv[] parsing begins */
	if(argc == 1){
	 // no command line arguments! Is this is a mistake or should we read the input list of images from a file
	 vast_list_of_input_images_with_time_corrections=fopen("vast_list_of_input_images_with_time_corrections.txt","r");
         // Check if we can open the file
	 if(NULL==vast_list_of_input_images_with_time_corrections){
          help_msg(argv[0], 0);
         }
         else{
          fclose(vast_list_of_input_images_with_time_corrections);
         }
         //
        }

        
        /* Options for getopt() */
        char *cvalue = NULL;
  
	const char* const shortopt = "vh9fdqmwpoPngGrlsteuijJka:b:x:y:";
	const struct option longopt[] = {
	        {"guess_saturation_limit", 0, NULL, 'g'}, {"no_guess_saturation_limit", 0, NULL, 'G'}, {"version", 0, NULL, 'v'},
		{"help", 0, NULL, 'h'}, {"ds9", 0, NULL, '9'}, {"small", 0, NULL, 's'}, {"time", 0, NULL, 't'}, {"medium", 0, NULL, 'm'}, {"wide", 0, NULL, 'w'}, 
		{"poly", 0, NULL, 'p'}, {"filter", 0, NULL, 'l'}, {"norotation", 0, NULL, 'r'}, {"nofind", 0, NULL, 'f'}, {"debug", 0, NULL, 'd'},
		{"position_dependent_correction", 0, NULL, 'j'}, {"no_position_dependent_correction", 0, NULL, 'J'},
		{"apeture", 1, NULL, 'a'}, {"matchstarnumber", 1, NULL, 'b'}, {"sysrem", 1, NULL, 'y'}, {"failsafe", 0, NULL, 'e'}, 
		{"UTC", 0, NULL, 'u'}, {"increment", 0, NULL, 'i'}, {"nojdkeyword", 0, NULL, 'k'}, {"maxsextractorflag", 1, NULL, 'x'}, {"photocurve", 0, NULL, 'o'}, {NULL, 0, NULL, 0}
	}; //NULL string must be in the end
	int nextopt;
	while (nextopt = getopt_long(argc, argv, shortopt, longopt, NULL), nextopt != -1) {
		switch (nextopt) {
			case 'v':
				version(stderr_output);
				fprintf(stdout,"%s\n",stderr_output);
				return 1;
			case 'h':
				help_msg(argv[0], 0);
			case 'f':
				param_nofind = 1;
				fprintf(stdout, "opt 'f': Run ./find_candidates manually!\n");
				break;
			case 'd':
				debug = 1;
				/* Protection against strange free() crushes */
				//setenv("MALLOC_CHECK_", "1", 1);
				break;
			case 'q':
				debug = 1;
				param_nofind = 1;
				break;
			case 't': //use period search
				period_search_switch = 1;
				fprintf(stdout, "opt 't': ANOVA & BLS period search algorithms will be used\n");
				break;
			case '9': //use ds9 FITS viewer
				use_ds9_instead_of_pgfv = 1;
				fprintf(stdout, "opt '9': DS9 FITS viewer will be used instead of pgfv\n");
				break;
			case 'g': // Auto-detect saturation limit
                                guess_saturation_limit_operation_mode=1;
				fprintf(stdout, "opt 'g': Will try to guess saturation limit for each image\n");
				break;
			case 'G': // Auto-detect saturation limit
			        guess_saturation_limit_operation_mode=0;
				fprintf(stdout, "opt 'G': Will NOT try to guess saturation limit for each image\n");
				break;
			case 's': //small comparison window - 1 pix.
				param_w = 3;
				preobr = New_Preobr_Sk();
				fprintf(stdout, "opt 's': Using small comparison window\n");
				break;
			case 'm': //medium comparison window
				param_w = 2;
				preobr = New_Preobr_Sk_M();
				fprintf(stdout, "opt 'm': Using medium comparison window\n");
				break;
			case 'w': //wide comparison window - 10 pix.
				param_w = 1;
				preobr = New_Preobr_Sk_W();
				faintest_stars = FAINTEST_STARS_PHOTO;
				fprintf(stdout, "opt 'w': Using wide comparison window\n");
				break;
			case 'p':
				param_p = 0;
				fprintf(stdout, "opt 'p': Polynomial magnitude calibration will *NOT* be used!\n");
				break;
			case 'o':
				param_use_photocurve = 1;
				param_p = 1; // force parabolic magnitude fit (it should be reasonably good). It is needed to remove outliers.
				fprintf(stdout, "opt 'o': \"photocurve\" will be used for magnitude calibration!\n");
				break;
			case 'P':
				param_P = 1;
				fprintf(stdout, "opt 'P': PSF photometry mode!\n");
                                /* Check if the PSFEx executable (named "psfex") is present in $PATH */
                                system("lib/look_for_psfex.sh");
                                // psfex.found will be created by lib/look_for_psfex.sh if PSFEx is found
				if( 0==is_file("psfex.found") ){
				 exit(1);
				}
                                break;
//			case 'n':
//				param_nocalib = 1;
//				param_nofind = 1; /* Do not start ./find_candidates it's useless without mag. calibration! */
//				fprintf(stdout, "opt 'n': No magnitude calibration will be done!!!\n");
//				break;
			case 'r':
				no_rotation=1;
				fprintf(stdout, "opt 'r': assuming no rotation larger than 3 degrees!\n");
				break;
			case 'l':
				param_nofilter=0;
				fprintf(stdout,"opt 'l': USING sigma filter!\n");
				break;
			case 'j':
				apply_position_dependent_correction=1;
				param_apply_position_dependent_correction=1;
				fprintf(stdout,"opt 'j': USING image-position-dependent correction!\n");
				break;
			case 'J':
				apply_position_dependent_correction=0;
				param_apply_position_dependent_correction=1;
				fprintf(stdout,"opt 'J': NOT USING image-position-dependent correction!\n");
				break;
			case 'e':
                                param_failsafe=1;
				fprintf(stdout,"opt 'e': FAILSAFE mode. Only stars detected on the reference frame will be processed!\n");
				break;
			case 'u':
				convert_timesys_to_TT=0;
				timesys=1;
				fprintf(stdout,"opt 'u': Stick with UTC time system, no conversion to TT will be done!\n");
				break;
			case 'k':
				param_nojdkeyword=1;
				fprintf(stdout,"opt 'k': \"JD\" keyword in FITS image header will be ignored!\n");
				break;
			case 'a':
				cvalue = optarg;
				if( 1==is_file(cvalue) ){
				 fprintf (stderr, "Option -%c requires an argument: fixed aperture size in pix.!\n", optopt);
				 exit(1);
				}
				fixed_aperture=atof(cvalue);
				fprintf(stdout, "opt 'a': Using fixed aperture %.1lf pix. in diameter!\n",fixed_aperture);
				break;
			case 'b':
				cvalue = optarg;
				if( 1==is_file(cvalue) ){
				 fprintf (stderr, "Option -%c requires an argument: number of reference stars!\n", optopt);
				 exit(1);
				}
				param_set_manually_Number_of_main_star=1;
				default_Number_of_main_star=atoi(cvalue);
				default_Number_of_ecv_triangle=MATCH_MAX_NUMBER_OF_TRIANGLES;
				fprintf(stdout, "opt 'b': Using %d reference stars for image matching!\n",default_Number_of_main_star);
				break;
			case 'y':
				cvalue = optarg;
				if( 1==is_file(cvalue) ){
				 fprintf (stderr, "Option -%c requires an argument: number of SysRem iterations!\n", optopt);
				 exit(1);
				}
				if( number_of_sysrem_iteration!=1 )
				 number_of_sysrem_iteration=atoi(cvalue);
				fprintf(stdout, "opt 'y': %d SysRem iterations will be conducted!\n",number_of_sysrem_iteration);
				break;
			case 'x':
				cvalue = optarg;
				if( 1==is_file(cvalue) ){
				 fprintf (stderr, "Option -%c requires an argument: Maximum acceptable SExtractor flag!\n", optopt);
				 exit(1);
				}
				maxsextractorflag=atoi(cvalue);
				fprintf(stdout, "opt 'x': %d is the maximum acceptable SExtractor flag!\n",maxsextractorflag);
				break;
			case '?':
				if (optopt == 'a'){
				 fprintf (stderr, "Option -%c requires an argument: fixed aperture size in pix.!\n", optopt);
				 exit(1);
				}
				break;
			case  -1:
				fprintf(stderr, "That's all\n");
				break;
		}
	}

	starfile = malloc(sizeof(char) * FILENAME_LENGTH);
	if( starfile==NULL ){
	 fprintf(stderr,"ERROR: can't allocate memory!\n starfile = malloc(sizeof(char) * FILENAME_LENGTH); - failed!\n");
	 vast_report_memory_error();
	 return 1;
	}
	base_star = NULL;
	Num=0;
	
	// Clean symlinks to images
	if( argc>1 ){
         system("if [ -d symlinks_to_images ];then rm -rf symlinks_to_images/ ;fi");
        }

        // Go through images and directories specified in the command line
        fprintf(stderr,"\nChecking the list of input images specified in the command line ...\n");
	for (n = optind; n < argc; ++n) {
		//sprintf(starfile, "%s", argv[n]);
		strncpy(starfile,argv[n],FILENAME_LENGTH);starfile[FILENAME_LENGTH-1]='\0';
		//fprintf(stderr,"Checking the input: %s\n",starfile);
		// Try to stat() the input
		if(0!=stat(starfile, &sb)){fprintf(stderr,"Cannot stat() %s - No such file or directory. Skipping the input.\n",starfile);continue;}
		// If this is some funny file
		if((sb.st_mode & S_IFMT) != S_IFDIR && (sb.st_mode & S_IFMT) != S_IFREG){fprintf(stderr,"The input is not a regular file, directory or symlink. Skipping the input %s\n",starfile);continue;}
		// If this is dir
		if((sb.st_mode & S_IFMT) == S_IFDIR){
		 //fprintf(stderr,"This is a directory: %s\n",starfile);
                 fprintf(stderr,"Listing %s\n",starfile);
                 dp = opendir(starfile);
                 if (dp == NULL)continue;
                 while( ep = readdir(dp) ){
                  // Cycle through all entries
                  if(ep->d_name[0]=='.')continue; // avoid hidden files and dirs
                  if((int)strlen(ep->d_name)<3)continue; // avoid short filenames
                  if( starfile[(int)strlen(starfile)-1]=='/' )
                   sprintf(dir_string,"%s%s",starfile,ep->d_name);
                  else
                   sprintf(dir_string,"%s/%s",starfile,ep->d_name);
                  // Try to stat() the input
                  if(0!=stat(dir_string, &sb)){fprintf(stderr,"Cannot stat() %s - No such file or directory. Skipping the input.\n",dir_string);continue;}
                  if((sb.st_mode & S_IFMT) != S_IFDIR && (sb.st_mode & S_IFMT) != S_IFREG){fprintf(stderr,"The input is not a regular file, directory or symlink. Skipping the input %s\n",dir_string);continue;}
                  if((sb.st_mode & S_IFMT) == S_IFDIR){
                   fprintf(stderr,"This is a directory: %s\n",dir_string);
                   continue;
                  } // if((sb.st_mode & S_IFMT) == S_IFDIR){
                  if((sb.st_mode & S_IFMT) == S_IFREG){
                   // Handle a regular file
                   if( 0!=fitsfile_read_check(dir_string) ){
                    fprintf(stderr,"The input does not appear to be a FITS image: %s\n",dir_string);continue;
                   }
                   else{
                    base_star = (char **)realloc(base_star, sizeof(char *) * (Num+1));
                    if( base_star==NULL ){fprintf(stderr,"ERROR: can't allocate memory!\n base_star = (char **)realloc(base_star, sizeof(char *) * n); - failed!\n");return 1;}
                    replace_file_with_symlink_if_filename_contains_white_spaces(dir_string);
                    base_star[Num] = malloc(sizeof(char) * (strlen(dir_string) + 1));
                    if( base_star[Num]==NULL ){fprintf(stderr,"ERROR: can't allocate memory!\n base_star[Num] = malloc(sizeof(char) * (strlen(starfile) + 1)); - failed!\n");vast_report_memory_error();return 1;}
                    strcpy(base_star[Num], dir_string);
                    Num++;
                   }
                  } // if((sb.st_mode & S_IFMT) == S_IFREG){
                 }
                 (void) closedir (dp);
		 continue;
                } // if((sb.st_mode & S_IFMT) == S_IFDIR){
                // If this is a regular file
		if((sb.st_mode & S_IFMT) == S_IFREG){
		 // Handle a regular file
		 if( 0!=fitsfile_read_check(starfile) ){fprintf(stderr,"The input does not appear to be a FITS image: %s\n",starfile);continue;}
		 //base_star = (char **)realloc(base_star, sizeof(char *) * n);
		 base_star = (char **)realloc(base_star, sizeof(char *) * (Num+1));
		 if( base_star==NULL ){fprintf(stderr,"ERROR: can't allocate memory!\n base_star = (char **)realloc(base_star, sizeof(char *) * n); - failed!\n");return 1;}
		 replace_file_with_symlink_if_filename_contains_white_spaces(starfile);
		 base_star[Num] = malloc(sizeof(char) * (strlen(starfile) + 1));
		 if( base_star[Num]==NULL ){fprintf(stderr,"ERROR: can't allocate memory!\n base_star[Num] = malloc(sizeof(char) * (strlen(starfile) + 1)); - failed!\n");vast_report_memory_error();return 1;}
                 strcpy(base_star[Num], starfile);
		 Num++;
		} // if((sb.st_mode & S_IFMT) == S_IFREG){
	}
	
	// Go through images specified in vast_list_of_input_images_with_time_corrections.txt
	fprintf(stderr,"\nLooking for the list of input images with time corrections in vast_list_of_input_images_with_time_corrections.txt ... ");
	vast_list_of_input_images_with_time_corrections=fopen("vast_list_of_input_images_with_time_corrections.txt","r");
        // Check if we can open the file
	if(NULL!=vast_list_of_input_images_with_time_corrections){
	 // If the file exists
	 fprintf(stderr,"list found!\n");
	 // Possibe buffer overflow here beacuse of fscanf(..., "%s", ...), but I feel lucky
	 while(2==fscanf(vast_list_of_input_images_with_time_corrections,"%s %lf",image_filename_from_input_list,&image_date_correction_from_input_list)){
	  // We will run a few simple checks on image_date_correction_from_input_list and forget image_date_correction_from_input_list for now, it will be looked for in gettime() later
	  if( fabs(image_date_correction_from_input_list)>EXPECTED_MAX_JD )continue;
	  // Try to stat image_filename_from_input_list
	  if(0!=stat(image_filename_from_input_list, &sb)){fprintf(stderr,"Cannot stat() %s - No such file or directory. Skipping the input.\n",image_filename_from_input_list);continue;}
          // If this is some funny file
          if((sb.st_mode & S_IFMT) != S_IFDIR && (sb.st_mode & S_IFMT) != S_IFREG){fprintf(stderr,"The input is not a regular file, directory or symlink. Skipping the input %s\n",image_filename_from_input_list);continue;}
          // If this is dir
          if((sb.st_mode & S_IFMT) == S_IFDIR){fprintf(stderr,"Sorry, you cannot specify a directory in vast_list_of_input_images_with_time_corrections.txt\nPlease specify only individual images in vast_list_of_input_images_with_time_corrections.txt\nSkipping the input %s\n",image_filename_from_input_list);continue;}
	  if((sb.st_mode & S_IFMT) == S_IFREG){
	   // Handle a regular file
	   if( 0!=fitsfile_read_check(image_filename_from_input_list) ){fprintf(stderr,"The input does not appear to be a FITS image: %s\n",image_filename_from_input_list);continue;}
           base_star = (char **)realloc(base_star, sizeof(char *) * (Num+1));
           if( base_star==NULL ){fprintf(stderr,"ERROR: can't allocate memory!\n base_star = (char **)realloc(base_star, sizeof(char *) * n); - failed!\n");return 1;}
           base_star[Num] = malloc(sizeof(char) * (strlen(image_filename_from_input_list) + 1));
           if( base_star[Num]==NULL ){fprintf(stderr,"ERROR: can't allocate memory!\n base_star[Num] = malloc(sizeof(char) * (strlen(image_filename_from_input_list) + 1)); - failed!\n");vast_report_memory_error();return 1;}
           strcpy(base_star[Num], image_filename_from_input_list);
           Num++;
	  } // if((sb.st_mode & S_IFMT) == S_IFREG){
	 }
	 fclose(vast_list_of_input_images_with_time_corrections);
	}
	else{
	 // If the file does not exist - never mind
	 fprintf(stderr,"list not found.\nThis is OK if you specify the input images in the command line, not through this text file.\n");
	}

	if( Num==1 ){ 
	 fprintf(stderr,"Only one image was supplied: %s",base_star[0]);
	 fprintf(stderr,"Entering single image mode...\n\n");
	 if( 0==is_file(base_star[0]) ){fprintf(stderr,"ERROR: can't open file %s \nIt does not exist or is not readable! :(\n",base_star[0]);return 0;}
	 sprintf(stderr_output,"./sextract_single_image %s\n",base_star[0]);
	 fprintf(stderr,"%s",stderr_output);
	 system(stderr_output);
	 return 0;
	}
	
	fprintf(stderr,"\nPreparing to process %d input FITS images...\n",Num);

	// Num==3, 4, or 5 - is likely the triplet mode for transient detection, we'll skip the usual stupid warnings
	if ( Num>HARD_MIN_NUMBER_OF_POINTS && Num<TESTOR_MIN_NUMBER_OF_POINTS && Num!=3 && Num!=4 && Num!=5 ){
		fprintf(stderr, "WARNING: It is recommended to use VaST with more than %d images (much more is much better)!\nYou have supplied only %d images. :(\n",TESTOR_MIN_NUMBER_OF_POINTS,Num);
		fprintf(stderr, "\n");
		fprintf(stderr, "This warning message will disappear in...   ");
		/* sleep for 6 seconds to make sure user saw the above message */
		for( n=5; n>0 ; n--){
		 sleep(1);
		 fprintf(stderr,"%d ", n);
		}
		sleep(1);
		fprintf(stderr,"NOW!\n");	
	}
	
	if ( Num<HARD_MIN_NUMBER_OF_POINTS && Num!=3 ){ 
		fprintf(stderr, "ERROR: At least %d images are needed for correct processing (much more is much better)!\nYou have supplied only %d images. :(\n",HARD_MIN_NUMBER_OF_POINTS,Num);
		fprintf(stderr, "\nThis error message often appears if there is a typo in the command line argument(s) specifying path to the images.\nPlease check the input carefully.\n\n");
		return 1; // disable the cheating mode
	//	fprintf(stderr, "#############################################################################\n");
	//	fprintf(stderr, "##                                WARNING!!!                               ##\n");
	//	fprintf(stderr, "##  Entering cheating mode. Each image will be processed more than once!   ##\n");
	//	fprintf(stderr, "##  In the lightcurve files each data point will be repeated many times!   ##\n");
	//	fprintf(stderr, "#############################################################################\n");
	//	fprintf(stderr, "\n");
	//	fprintf(stderr, "If you don't want it, press Ctrl+C to abort now...   ");
	//	/* sleep for 6 seconds to make sure user saw the above message */
	//	for( n=5; n>0 ; n--){
	//	 sleep(1);
	//	 fprintf(stderr,"%d ", n);
	//	}
	//	sleep(1);
	//	fprintf(stderr,"GO!\n");
	//	while( Num<TESTOR_MIN_NUMBER_OF_POINTS + (int)(0.3 * TESTOR_MIN_NUMBER_OF_POINTS) ){
	//	 for (n = optind + 1; n < argc; ++n) { // we do not want to repeat reference image: it will not be processed more than once anyway 
	//	  sprintf(starfile, "%s", argv[n]);
	//	  base_star = (char **)realloc(base_star, sizeof(char *) * (n + Num) );
	//	  base_star[Num] = malloc(sizeof(char) * (strlen(starfile) + 1));
	//	  if( base_star[Num]==NULL ){fprintf(stderr,"ERROR: can't allocate memory for base_star[Num]\n");vast_report_memory_error();return 1;}
	//	  strcpy(base_star[Num], starfile);
	//	  Num++;
	//	 }
	//	}
	}

 
        /* Print the TT warning only if UTC was not explicitly requested by user. */
        if( convert_timesys_to_TT!=0 && Num!=3 && Num!=4 && Num!=5 )
	 hello_msg();

        /* The end of the beginning =) */


        /* Start timer */
        start_time=time(NULL);

        /* Update PATH variable to make sure the local copy of SExtractor is there */
        char pathstring[8192];
        strncpy(pathstring,getenv("PATH"),8192);pathstring[8191]='\0';
        strncat(pathstring,":lib/bin",8192);pathstring[8191]='\0';
        setenv("PATH", pathstring, 1);
        
        /* Check if the SExtractor executable (named "sex") is present in $PATH */
        if(0!=system("lib/look_for_sextractor.sh")){fprintf(stderr,"Error looking for SExtractor. Aborting further computations...\n");return 1;}

        /* Destroy files created by previous session */
	fprintf(stderr,"Cleaning old outNNNNN.dat files.\n");
	if(0!=system("util/clean_data.sh all >/dev/null")){fprintf(stderr,"Error while cleaning old files. Aborting further computations...\n");return 1;}
	fprintf(stderr,"Done with cleaning!\n");
	
        /* Save command line arguments to the log file vast_command_line.log */
	save_command_line_to_log_file(argc, argv);

        /* 
           Generate bright_star_blend_check.sex from default.sex for false transient removal
        */
        system("lib/generate_bright_star_blend_check_sex.sh default.sex");

	/* Reading file which defines rectangular regions we want to exclude */
	double X1[MAX_NUMBER_OF_BAD_REGIONS_ON_CCD], Y1[MAX_NUMBER_OF_BAD_REGIONS_ON_CCD], X2[MAX_NUMBER_OF_BAD_REGIONS_ON_CCD], Y2[MAX_NUMBER_OF_BAD_REGIONS_ON_CCD];
	int N_bad_regions = 0;
	read_bad_lst(X1, Y1, X2, Y2, &N_bad_regions);

 	/* Reading file with stars which should not be used for magnitude calibration */
	FILE *excludefile;
	bad_stars_X = malloc(sizeof(double));
	if( bad_stars_X==NULL ){fprintf(stderr,"ERROR: can't allocate memory for bad_stars_X\n");vast_report_memory_error();return 1;}
	bad_stars_Y = malloc(sizeof(double));
	if( bad_stars_Y==NULL ){fprintf(stderr,"ERROR: can't allocate memory for bad_stars_Y\n");vast_report_memory_error();return 1;}
	excludefile = fopen("exclude.lst", "r");
	if (excludefile == NULL) {
		fprintf(stderr, "ERROR: I can't find file exclude.lst! :(\nI die.\n");
		exit(1);
	}
	while (-1 < fscanf(excludefile, "%lf %lf", &bad_stars_X[N_bad_stars], &bad_stars_Y[N_bad_stars])) {
		bad_stars_X = realloc(bad_stars_X, sizeof(double) * (N_bad_stars + 2));
		bad_stars_Y = realloc(bad_stars_Y, sizeof(double) * (N_bad_stars + 2));
		N_bad_stars += 1;
	}
	fclose(excludefile);
	fprintf(stderr, "Exclusion file was read\n\n\n");

          
	/* From now on, ignore SIGHUP! */
	signal(SIGHUP, SIG_IGN);

        /* Starting real work... */
	if (param_w == 0)
		preobr = New_Preobr_Sk();
		
        if( param_set_manually_Number_of_main_star==0 ){
         // Remamber default preobr parameters
         default_Number_of_ecv_triangle=preobr->Number_of_ecv_triangle;
         default_Number_of_main_star=preobr->Number_of_main_star;
        }
        else{
         // Or set parameters supplied by user
         preobr->Number_of_ecv_triangle=default_Number_of_ecv_triangle;
         preobr->Number_of_main_star=default_Number_of_main_star;
        }
        
        fprintf(stderr,"We have %d images to process.\n",Num);
        
        // Debug the input image list
        //for(i=0;i<Num;i++)
        // fprintf(stderr,"%s\n",base_star[i]);
        
        /* Create vast_images_catalogs.log */
        write_images_catalogs_logfile( base_star, Num);
        
        int n_fork; //=get_number_of_cpu_cores(); // number of parallel threads
        int i_fork=0; // counter for fork
        int pid_status;
        int *child_pids;
        
        // Set the desired number of threads
        //if( param_P==1 )
        // n_fork=1;
        //else
         n_fork=get_number_of_cpu_cores();
        
        
        child_pids=malloc(n_fork*sizeof(int));
        if( child_pids==NULL ){fprintf(stderr,"ERROR: can't allocate memory!\n child_pids=malloc(n_fork*sizeof(int)); - failed!\n");vast_report_memory_error();return 1;}
        
        fprintf(stderr,"Running SExtractor in %d parallel threads...\n",MIN(n_fork,Num));
        for(i=0;i<Num;i++){
      	 fitsfile_read_error = gettime(base_star[i], &JD, &timesys, convert_timesys_to_TT, &X_im_size, &Y_im_size, stderr_output, log_output, param_nojdkeyword, 0);
	 if (fitsfile_read_error != 0 && i==0 ) {
	  fprintf(stderr, "Error reading reference file: code %d\nI'll die :(\n", fitsfile_read_error);
	  fits_report_error(stderr, fitsfile_read_error);
	  exit(fitsfile_read_error);
	 }
	 else{
	  i_fork++;
	  pid=fork();
	  if( pid==0 || pid==-1 ){
	   if( pid==-1 )fprintf(stderr,"WARNING: cannot fork()! Continuing in the streamline mode...\n");
	   //autodetect_aperture(base_star[i], sextractor_catalog, 0, param_P, fixed_aperture, 0, X_im_size, Y_im_size);
	   autodetect_aperture(base_star[i], sextractor_catalog, 0, param_P, fixed_aperture, X_im_size, Y_im_size, guess_saturation_limit_operation_mode);
           if( pid==0 )exit(0); // exit only if this is actually a child
          }
          else{
           child_pids[i_fork-1]=pid;
           if( i_fork==MIN(n_fork,Num) ){
            for(;i_fork--;){
             //fprintf(stderr,"WAITING FOR %d\n",i_fork);
             pid=child_pids[i_fork];
             waitpid(pid,&pid_status,0);
            }
            // i_fork=-1 after the for
            i_fork=0;
           }
          }
         }
        }
        
        // Here is a wild assumption: if we have more than 100 images,
        // the remaining few will be processed while we are starting
        // to match star lists corresponding to the first few images  
        if( Num<100 ){
         for(;i_fork--;){
          fprintf(stderr,"Waiting for thread %d to finish...\n",i_fork+1);
          if( i_fork<0 )break;
          pid=child_pids[i_fork];
          waitpid(pid,&pid_status,0);
         }
        }
        
        //free(child_pids); we'll need it later!
        fprintf(stderr,"\n\nDone with SExtractor!\n\n");

        /*  Allocate memory for arrays with coordinates. */
        number_of_coordinate_measurements_for_star=malloc(MAX_NUMBER_OF_STARS*sizeof(int));
        if( number_of_coordinate_measurements_for_star==NULL ){fprintf(stderr,"ERROR: can't allocate memory for number_of_coordinate_measurements_for_star\n");vast_report_memory_error();return 1;}
        star_numbers_for_coordinate_arrays=malloc(MAX_NUMBER_OF_STARS*sizeof(int));
        if( star_numbers_for_coordinate_arrays==NULL ){fprintf(stderr,"ERROR: can't allocate memory for star_numbers_for_coordinate_arrays\n");vast_report_memory_error();return 1;}
        coordinate_array_x=malloc(MAX_NUMBER_OF_STARS*sizeof(float *));
        if( coordinate_array_x==NULL ){fprintf(stderr,"ERROR: can't allocate memory for coordinate_array_x\n");vast_report_memory_error();return 1;}
        coordinate_array_y=malloc(MAX_NUMBER_OF_STARS*sizeof(float *));
        if( coordinate_array_y==NULL ){fprintf(stderr,"ERROR: can't allocate memory for coordinate_array_y\n");vast_report_memory_error();return 1;}
	coordinate_array_counter=0;	
        /* Inititalize the coordinate arrays just to make debugger happy */
        //for(i=0;i<MAX_NUMBER_OF_STARS;i++){
        for(i=MAX_NUMBER_OF_STARS;i--;){
         coordinate_array_x[i]=NULL;
         coordinate_array_y[i]=NULL;
         star_numbers_for_coordinate_arrays[i]=0;
        }


	/* Process the reference image */

	if (debug != 0) fprintf(stderr, "DEBUG MSG: (ref) gettime(base_star[0])\n"); // Debug message!
	fitsfile_read_error = gettime(base_star[0], &JD, &timesys, convert_timesys_to_TT, &X_im_size, &Y_im_size, stderr_output, log_output, param_nojdkeyword, 1);
	if (fitsfile_read_error != 0) {
	 fprintf(stderr, "Error reading reference file: code %d\nI'll die :(\n", fitsfile_read_error);
	 fits_report_error(stderr, fitsfile_read_error);
	 exit(fitsfile_read_error);
	}
	//frame.X_centre = X_im_size / 2;
	//frame.Y_centre = Y_im_size / 2;
	max_X_im_size=X_im_size;
	max_Y_im_size=Y_im_size;
	
	if (debug != 0) fprintf(stderr, "DEBUG MSG: (ref) autodetect_aperture(base_star[0])\n");
	//aperture = autodetect_aperture(base_star[0], sextractor_catalog, 0, param_P, fixed_aperture, 1, X_im_size, Y_im_size);
	aperture = autodetect_aperture(base_star[0], sextractor_catalog, 0, param_P, fixed_aperture, X_im_size, Y_im_size, guess_saturation_limit_operation_mode);
	if (aperture > 50 || aperture < 1.0 ) {
	 fprintf(stderr, "APERTURE = %.1lf is > 50.0 or < 1.0\nBad reference image...\n", aperture);
	 exit(1);
	}
	if( param_w==0 ){
	 preobr->sigma_popadaniya=AUTO_SIGMA_POPADANIYA_COEF*aperture;
	}

	fprintf(stderr,"%s",stderr_output);
        //fprintf(stderr,"----------------%s----------------\n",sextractor_catalog);
	write_string_to_log_file(log_output,sextractor_catalog);

	
	 if (debug != 0) fprintf(stderr, "DEBUG MSG: (ref) Read_sex_cat(%s and other stuff)\n",sextractor_catalog);

	 // NUMBER1 is the number of stars on the reference frame

         /* Read_sex_cat2 */
         counter_rejected_bad_psf_fit=0;// just in case
         if( param_P==1 )filter_out_bad_psf_fits_from_catalog(sextractor_catalog,&counter_rejected_bad_psf_fit);
         file=fopen(sextractor_catalog, "r");
	 if (file == NULL) {
	 	fprintf(stderr, "Can't open file %s\n", sextractor_catalog);
	 	exit(1);
	 }
	 else
	  if (debug != 0) fprintf(stderr, "DEBUG MSG: Opened file %s\n", sextractor_catalog);
	 STAR1=malloc(MAX_NUMBER_OF_STARS*sizeof(struct Star));
	 if( STAR1==NULL ){fprintf(stderr,"ERROR: can't allocate memory!\n STAR1=malloc(MAX_NUMBER_OF_STARS*sizeof(struct Star)); - failed!\n");vast_report_memory_error();return 1;}
	 STAR3=malloc(MAX_NUMBER_OF_STARS*sizeof(struct Star));
	 if( STAR3==NULL ){fprintf(stderr,"ERROR: can't allocate memory!\n STAR3=malloc(MAX_NUMBER_OF_STARS*sizeof(struct Star)); - failed!\n");vast_report_memory_error();return 1;}
	 //---------------------------------------
	 // Determine SExtractor catalog format //
	 //if( NULL==fgets(sextractor_catalog_string,MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT,file) ){fprintf(stderr,"ERROR determining SExtractor catalog format!\n");}
	 counter_rejected_bad_flux=counter_rejected_low_snr=counter_rejected_bad_region=counter_rejected_frame_edge=counter_rejected_too_small=counter_rejected_too_large=counter_rejected_external_flag=0; // reset bad star counters
	 //---------------------------------------
	 for (NUMBER1 = 0, NUMBER3 = 0;;) {
                if (debug != 0) fprintf(stderr, "Reading a catalog line NUMBER1=%d NUMBER3=%d \n",NUMBER1,NUMBER3);
		//key = fscanf(file, "%d %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %d\n", &star_number_in_sextractor_catalog, &flux_adu, &flux_adu_err, &mag, &sigma_mag, &position_x_pix, &position_y_pix, &a_a, &a_a_err, &a_b, &a_b_err, &sextractor_flag);
		//if (key == EOF)
		//	break;
                if( NULL==fgets(sextractor_catalog_string,MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT,file) ){break;}
                sextractor_catalog_string[MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT-1]='\0'; // just in case
                external_flag=0;
                external_flag_string[0]='\0';
                if( 12>sscanf(sextractor_catalog_string, "%d %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %d %[^\t\n]", &star_number_in_sextractor_catalog, &flux_adu, &flux_adu_err, &mag, &sigma_mag, &position_x_pix, &position_y_pix, &a_a, &a_a_err, &a_b, &a_b_err, &sextractor_flag, external_flag_string) ){
                 fprintf(stderr,"WARNING: problem occurred while parsing SExtractor catalog %s\nThe offending line is:\n%s\n",sextractor_catalog,sextractor_catalog_string);
                 continue;
                }
                // Now this is some crazy stuff:
                // The last columns of the SExtractor catalog file might be:
                // ... flags
                // ... flags external_flags
                // ... flags external_flags psf_fitting_chi2
                // ... flags psf_fitting_chi2
                // Below we try to handle each of the four possibilities
                //
                // if these are not just flags
                // (but make sure we perform the tese only on a line wit ha good measurement)
                if( strlen(external_flag_string)>0 && flux_adu>0.0 && mag!=99.0000 ){
                 // if these are not "flags external_flags psf_fitting_chi2"
                 if( 2!=sscanf(external_flag_string,"%lf %lf",&double_external_flag,&psf_chi2) ){
                  // Decide between "flags external_flags" and "flags psf_fitting_chi2"
                  for(ii=0,jj=0;ii<(int)strlen(external_flag_string);ii++){
                   if( external_flag_string[ii]=='.' || external_flag_string[ii]=='e' ){jj=1;break;} // assume that a decimal point indicates psf_chi2 rather than external_flag that is expected to be 0 or 1 only
                  }
                  if( jj==0 ){
                   // "flags external_flags" case
                   psf_chi2=1.0; // no PSF fitting results
                   if( 1!=sscanf(external_flag_string,"%lf",&double_external_flag) ){
                    double_external_flag=0.0; // no external flag image used
                   }
                  }
                  else{
                   // "flags psf_fitting_chi2" case
                   double_external_flag=0.0; // no external flag image used
                   if( 1!=sscanf(external_flag_string,"%lf",&psf_chi2) ){
                    psf_chi2=1.0; // no PSF fitting results
                   }
                  }
                 } // if( 2!=sscanf(external_flag_string,"%lf %lf",&double_external_flag,&psf_chi2) ){
                 external_flag=(int)double_external_flag;
                }
                else{
                 psf_chi2=1.0; // no PSF fitting results
                 external_flag=0; // no external flag image used
                }

		// Check if the catalog line is a really band one
		if( flux_adu==0 ){counter_rejected_bad_flux++;continue;}
		if( flux_adu_err==999999 ){counter_rejected_bad_flux++;continue;}
		if( mag==99.0000 ){counter_rejected_bad_flux++;continue;}
		if( sigma_mag==99.0000 ){counter_rejected_bad_flux++;continue;}
		if( flux_adu<MIN_SNR*flux_adu_err ){counter_rejected_low_snr++;continue;}
		if( 0 != exclude_region(X1, Y1, X2, Y2, N_bad_regions, position_x_pix, position_y_pix, aperture) ){counter_rejected_bad_region++;continue;}
		if( position_x_pix < FRAME_EDGE_INDENT_PIXELS || position_y_pix < FRAME_EDGE_INDENT_PIXELS || fabs(position_x_pix - X_im_size) < FRAME_EDGE_INDENT_PIXELS || fabs(position_y_pix - Y_im_size) < FRAME_EDGE_INDENT_PIXELS ){counter_rejected_frame_edge++;continue;}
		//
		if( a_a+a_a_err<FWHM_MIN ){counter_rejected_too_small++;continue;}
		if( a_b+a_b_err<FWHM_MIN ){counter_rejected_too_small++;continue;}
		//
		if( a_a>aperture ){counter_rejected_too_large++;continue;}
		if( external_flag!=0 ){counter_rejected_external_flag++;continue;}
		//
		NUMBER1++;
		NUMBER3++;
		STAR1[NUMBER1-1].vast_flag=0;
		//if (position_x_pix < FRAME_EDGE_INDENT_PIXELS || position_y_pix < FRAME_EDGE_INDENT_PIXELS || fabs(position_x_pix - X_im_size) < FRAME_EDGE_INDENT_PIXELS || fabs(position_y_pix - Y_im_size) < FRAME_EDGE_INDENT_PIXELS )
		// STAR1[NUMBER1-1].vast_flag+=5;
		//if( 0 != exclude_region(X1, Y1, X2, Y2, N_bad_regions, position_x_pix, position_y_pix, aperture) ) 
		// STAR1[NUMBER1-1].vast_flag+=40;
                //if( flux_adu<MIN_SNR*flux_adu_err )
                // STAR1[NUMBER1-1].vast_flag+=300;
                //if( mag==99.0000 )
                // STAR1[NUMBER1-1].vast_flag+=2000;

                STAR1[NUMBER1-1].n = star_number_in_sextractor_catalog;
                STAR1[NUMBER1-1].x_frame = STAR1[NUMBER1-1].x = (float)position_x_pix;
                STAR1[NUMBER1-1].y_frame = STAR1[NUMBER1-1].y = (float)position_y_pix;
                STAR1[NUMBER1-1].flux = flux_adu;
                STAR1[NUMBER1-1].mag = (float)mag;
                STAR1[NUMBER1-1].sigma_mag = (float)sigma_mag; 
                STAR1[NUMBER1-1].JD = JD;
                STAR1[NUMBER1-1].detected_on_ref_frame=1; // Mark the star that it was detected on the reference frame and not added later
                STAR1[NUMBER1-1].sextractor_flag=(char)sextractor_flag; // SEXtractor flag ;)
                STAR1[NUMBER1-1].star_size=a_a;
                // Use only good stars for coordinate transformation
                //if( STAR1[NUMBER1-1].sextractor_flag==0 && STAR1[NUMBER1-1].vast_flag==0 ){
                if( STAR1[NUMBER1-1].sextractor_flag<=6 && STAR1[NUMBER1-1].vast_flag==0 ){
	         Star_Copy(STAR3 + NUMBER3 - 1, STAR1 + NUMBER1 - 1);
                }
                else{
                 NUMBER3--;
                }
        
	 }
	 fclose(file);
	 if (debug != 0) fprintf(stderr, "DEBUG MSG: image00001.cat was closed...\n");
	 /* end of Read_sex_cat2 */
	
	 // Print out input catalog filtering stats
	 fprintf(stderr,"SExtractor output filtering results:\n * passed selection: %d, (%d good for image matching)\n * rejected as having no flux measurement: %d\n * rejected as having SNR<%.1lf: %d\n * rejected inside rectangles defined in bad_region.lst: %d\n * rejected close to frame edge: %d\n * rejected as being too small (<%.1lf pix): %d\n * rejected as being too large (>%.1lf pix): %d\n * rejected due to an external image flag: %d\n * rejected as having bad PSF fit: %d\n---\n",
	 NUMBER1,
	 NUMBER3,
	 counter_rejected_bad_flux, 
	 MIN_SNR,
	 counter_rejected_low_snr,
	 counter_rejected_bad_region,
	 counter_rejected_frame_edge, 
	 FWHM_MIN,
	 counter_rejected_too_small, 
	 aperture,counter_rejected_too_large, 
	 counter_rejected_external_flag, 
	 counter_rejected_bad_psf_fit);
         fprintf(stderr,"You may change some of the filtering parameters by editing src/limits.h and re-running 'make'.\n");
         // Check the stats and issue warnings if needed
         if( (double)counter_rejected_too_small/(double)NUMBER1>0.3 )fprintf(stderr,"\E[01;31m WARNING: \E[33;00m suspiciously many stars are rejected as being too small. Please check FWHM_MIN in src/limits.h and re-run 'make' if you change it.\n");
         
	 /* Check if enough stars were detected on the reference frame */
	 if( NUMBER1<MIN_NUMBER_OF_STARS_ON_FRAME || NUMBER3<MIN_NUMBER_OF_STARS_ON_FRAME ){
	  fprintf(stderr,"ERROR! Too few stars detected on the reference frame: %d<%d\n",NUMBER1,MIN_NUMBER_OF_STARS_ON_FRAME);
	  fprintf(stderr,"\nPlease check that the reference image file is readable and looks reasonably good.\n");
	  fprintf(stderr,"\nThe SExtractor catalog for the reference image may be found in image00001.cat \n");
	  if(param_P==1)fprintf(stderr,"\nCheck that GAIN parameter in the default.sex file is set to a value >0.0 \n");
	  return 1;
	 }
	 
	 /* Set distance to the closest star in the structure: will be useful for star matching later... */
	 //set_distance_to_neighbor_in_struct_Star(STAR1, NUMBER1, aperture, X_im_size, Y_im_size);
	 //set_distance_to_neighbor_in_struct_Star(STAR3, NUMBER3, aperture, X_im_size, Y_im_size);
	 /* Set maximum star number */
	 max_number=STAR1[0].n;
	 //for(i=0;i<NUMBER1;i++){
	 for(i=NUMBER1;i--;){
	  if( max_number<STAR1[i].n )max_number=STAR1[i].n;
	 }
	 // make a gap in numbering so stars detected on the reference frame can be easily distinguished from those added later by their number
	 //
	 // a sily attempt to preserve star numbers over data reduction runs with different settings (-x0 vs. -x2)
	 // I'm afraid that will not change much...
	 if( max_number<50000){
	  if( max_number<30000){
	   if( max_number<20000){
	    max_number=20000;
           }
           else{
	    max_number=30000;
           }
	  }
	  else{
	   max_number=50000;
          }
	 }
	 else{
	  // fall back to the default behaviour 
	  max_number+=10000;
	 }
	 //max_number+=10000; // make a gap in numbering so stars detected on the reference frame can be easily distinguished from those added later by their number
	 
	 // Turn on the CCD position dependent magnitude correction if needed
	 if( param_apply_position_dependent_correction==0 ){
	  // If this was not specified in the command line, make our own choice
	  if( NUMBER1>MIN_NUMBER_OF_STARS_FOR_CCD_POSITION_DEPENDENT_MAGNITUDE_CORRECTION )
	   apply_position_dependent_correction=1; // apply the correction
          else
           apply_position_dependent_correction=0; // do not apply the correction
	 }
	 
	 // Report if the CCD position dependent magnitude correction will be used
	 if( apply_position_dependent_correction==1 )
	  fprintf(stderr,"\nPlease note: the CCD position dependent magnitude correction will be applied.\n");
         else
          fprintf(stderr,"\nPlease note: the CCD position dependent magnitude correction will not be applied.\n");
	  

         /* Sort arrays for Ident */
         if (debug != 0) fprintf(stderr, "DEBUG MSG: NUMBER1=%d  NUMBER3=%d...\n",NUMBER1,NUMBER3);
         if (debug != 0) fprintf(stderr, "DEBUG MSG: Sort arrays for Ident STAR1, NUMBER1...\n");
         Sort_in_mag_of_stars(STAR1, NUMBER1);
         if (debug != 0) fprintf(stderr, "DEBUG MSG: Sort arrays for Ident STAR3, NUMBER3...\n");
         Sort_in_mag_of_stars(STAR3, NUMBER3);
         if (debug != 0) fprintf(stderr, "DEBUG MSG: Done with sorting arrays...\n");
         
         Pos1 = malloc(sizeof(int) * NUMBER1);
         if( Pos1==NULL ){fprintf(stderr,"ERROR: can't allocate memory!\n Pos1 = malloc(sizeof(int) * NUMBER1); - failed!\n");vast_report_memory_error();return 1;}


         if (debug != 0) fprintf(stderr, "DEBUG MSG: set_transient_search_boundaries()\n");
         set_transient_search_boundaries( search_area_boundaries, STAR3, NUMBER3);

	 if (debug != 0) fprintf(stderr, "DEBUG MSG: I am NUMBER1: %d, I am JD: %lf, I am Mister X: %lf\n", NUMBER1, JD, X_im_size);
	 if (debug != 0) fprintf(stderr, "DEBUG MSG: malloc(sizeof(int) * NUMBER1)\n");


	 /* Read catalog of the reference image */
	 // Try to allocate a large chunk of memory
	 ptr_struct_Obs = malloc(sizeof(struct Observation) * Max_obs_in_RAM);
	 // i is just a counter
	 for(i=0; i<4; i++){
	  // Check if the allocation failed
	  if( ptr_struct_Obs==NULL ){
	   fprintf(stderr,"WARNING: can't allocate memory for observations!\n ptr_struct_Obs = malloc(sizeof(struct Observation) * Max_obs_in_RAM);\nTrying to allocate less memory...\n");
          }
          else{
           // If the allocation is a success, get out of this cycle
           break;
          }
          // If the allocation failed, try to pre-allocate less memory
          Max_obs_in_RAM=Max_obs_in_RAM/2;
          ptr_struct_Obs = malloc(sizeof(struct Observation) * Max_obs_in_RAM);
         }
         // If we still cannot allocate memory - exit with an error message
         if( ptr_struct_Obs==NULL ){fprintf(stderr,"ERROR: can't allocate memory for observations!\n ptr_struct_Obs = malloc(sizeof(struct Observation) * Max_obs_in_RAM);\n");return 1;}

	 for (i = 0; i < NUMBER1; i++) {
		if (STAR1[i].n >= MAX_NUMBER_OF_STARS) {
			fprintf(stderr, "Oops!!! Too many stars!\nChange string \"#define MAX_NUMBER_OF_STARS %d\" in src/limits.h file and recompile the program by running \"make\".\n\nOr you may choose a higher star detection limit (get less stars per frame) by changing DETECT_MINAREA and DETECT_THRESH/ANALYSIS_THRESH parameters in default.sex file\n",
			        MAX_NUMBER_OF_STARS);
			exit(1);
		}
		// Check if the star is good enough to be written to the output
		if ( X_im_size-STAR1[i].x_frame>FRAME_EDGE_INDENT_PIXELS && Y_im_size-STAR1[i].y_frame>FRAME_EDGE_INDENT_PIXELS && STAR1[i].x_frame>FRAME_EDGE_INDENT_PIXELS && STAR1[i].y_frame>FRAME_EDGE_INDENT_PIXELS && STAR1[i].sextractor_flag<=maxsextractorflag ){
		 TOTAL_OBS++;
		 obs_in_RAM++;
		
		 // Save coordinates to the array. (the arrays are used to compute mean position of a star across all images)
		 star_numbers_for_coordinate_arrays[coordinate_array_counter]=STAR1[i].n;
		 number_of_coordinate_measurements_for_star[coordinate_array_counter]=1; // first measurement
		 coordinate_array_x[coordinate_array_counter]=malloc(sizeof(float));
		 if( coordinate_array_x[coordinate_array_counter]==NULL ){fprintf(stderr,"ERROR: can't allocate memory for coordinate_array_x[coordinate_array_counter]\n");vast_report_memory_error();return 1;}
		 coordinate_array_x[coordinate_array_counter][0]=STAR1[i].x;
		 coordinate_array_y[coordinate_array_counter]=malloc(sizeof(float));
		 if( coordinate_array_y[coordinate_array_counter]==NULL ){fprintf(stderr,"ERROR: can't allocate memory for coordinate_array_y[coordinate_array_counter]\n");vast_report_memory_error();return 1;}
		 coordinate_array_y[coordinate_array_counter][0]=STAR1[i].y;
		 coordinate_array_counter++;
		 
		 if( obs_in_RAM>Max_obs_in_RAM ){
		  ptr_struct_Obs = realloc(ptr_struct_Obs, sizeof(struct Observation) * obs_in_RAM);
		  if( ptr_struct_Obs==NULL ){fprintf(stderr,"ERROR: cannot re-allocate memory for a new observation (from reference image)!\n ptr_struct_Obs = realloc(ptr_struct_Obs, sizeof(struct Observation) * obs_in_RAM); - failed!\n");return 1;}
                 }
		 ptr_struct_Obs[obs_in_RAM-1].star_num = STAR1[i].n;
		 ptr_struct_Obs[obs_in_RAM-1].JD = STAR1[i].JD;
		 ptr_struct_Obs[obs_in_RAM-1].mag = (double)STAR1[i].mag;
		 ptr_struct_Obs[obs_in_RAM-1].mag_err = (double)STAR1[i].sigma_mag;
		 ptr_struct_Obs[obs_in_RAM-1].X = (double)STAR1[i].x_frame;
		 ptr_struct_Obs[obs_in_RAM-1].Y = (double)STAR1[i].y_frame;
		 ptr_struct_Obs[obs_in_RAM-1].APER = aperture;
		 
		 strcpy(ptr_struct_Obs[obs_in_RAM-1].filename, base_star[0]);
		 ptr_struct_Obs[obs_in_RAM-1].is_used=0;
                } // if the star is good enough...
	 }
	 fprintf(stderr,"    %4d stars detected\n",NUMBER1);

	 check_and_print_memory_statistics();
	 progress(1, Num);

	
	 /* log first observation */
	 sprintf(log_output,"JD= %13.5lf  ap= %4.1lf  rotation= %7.3lf  *detected= %5d  *matched= %5d  status=OK     %s\n", JD, aperture, 0.0, NUMBER1, NUMBER1, base_star[0]);
	 write_string_to_log_file(log_output,sextractor_catalog);
	
	/* Process other images */
	for (n = n_start; n < Num; n++) {
		fprintf(stderr,"\n");
		if (debug != 0) fprintf(stderr, "DEBUG MSG: gettime() - ");
		fitsfile_read_error = gettime(base_star[n], &JD, &timesys, convert_timesys_to_TT, &X_im_size, &Y_im_size, stderr_output, log_output, param_nojdkeyword, 1);//Вот этот код дублирован выше! Зачем?
		if (fitsfile_read_error == 0) {
			if (debug != 0) fprintf(stderr, "Ok\n");
			if (debug != 0) fprintf(stderr, "DEBUG MSG: autodetect_aperture() - ");
			//aperture = autodetect_aperture(base_star[n], sextractor_catalog, 0, param_P, fixed_aperture, 1, X_im_size, Y_im_size);
			aperture = autodetect_aperture(base_star[n], sextractor_catalog, 0, param_P, fixed_aperture, X_im_size, Y_im_size, guess_saturation_limit_operation_mode);
			if (debug != 0) fprintf(stderr, "Ok\n");
			/* Write the logfile */
			if( aperture<0.0 )aperture=0.0; // Do not corrupt the log file with bad apertures
			if( aperture>99.9 )aperture=99.9; // Do not corrupt the log file with bad apertures
			write_string_to_log_file(log_output,sextractor_catalog);
			sprintf(log_output,"JD= %13.5lf  ap= %4.1lf  ", JD, aperture);
			write_string_to_log_file(log_output,sextractor_catalog);
			
			if (aperture < 50.0 && aperture>1.0 ) {
			        //
			        // Make sure we record the largest image size
			        //
			        if( max_X_im_size<X_im_size )max_X_im_size=X_im_size;
			        if( max_Y_im_size<Y_im_size )max_Y_im_size=Y_im_size;
			        //
				if( param_w==0 ){
				 preobr->sigma_popadaniya=AUTO_SIGMA_POPADANIYA_COEF*aperture;
				}
				if (debug != 0) fprintf(stderr, "DEBUG MSG: Read_sex_cat() - ");
				/* Read_sex_cat2 */
				STAR2=malloc( MAX_NUMBER_OF_STARS*sizeof(struct Star));
				if (STAR2 == NULL) {
				 fprintf(stderr, "ERROR: No memory (STAR2)\n");
				 vast_report_memory_error();
				 exit(1);
				}
				if( param_P==1 )filter_out_bad_psf_fits_from_catalog(sextractor_catalog,&counter_rejected_bad_psf_fit);
				file = fopen(sextractor_catalog, "r");
				if (file == NULL) {
					fprintf(stderr, "Can't open file %s\n", sextractor_catalog);
					exit(1);
				}
				NUMBER2=0;
				counter_rejected_bad_flux=counter_rejected_low_snr=counter_rejected_bad_region=counter_rejected_frame_edge=counter_rejected_too_small=counter_rejected_too_large=counter_rejected_external_flag=0; // reset bad star counters
				//while( -1<fscanf(file, "%d %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %d\n", &star_number_in_sextractor_catalog, &flux_adu, &flux_adu_err, &mag, &sigma_mag, &position_x_pix, &position_y_pix, &a_a, &a_a_err, &a_b, &a_b_err, &sextractor_flag) ){
				while( NULL!=fgets(sextractor_catalog_string,MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT,file) ){
				        sextractor_catalog_string[MAX_STRING_LENGTH_IN_SEXTARCTOR_CAT-1]='\0'; // just in case
				        external_flag=0;
				        external_flag_string[0]='\0';
				        if( 12>sscanf(sextractor_catalog_string, "%d %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %d %[^\t\n]", &star_number_in_sextractor_catalog, &flux_adu, &flux_adu_err, &mag, &sigma_mag, &position_x_pix, &position_y_pix, &a_a, &a_a_err, &a_b, &a_b_err, &sextractor_flag, external_flag_string) ){
				         fprintf(stderr,"WARNING: problem occurred while parsing SExtractor catalog %s\nThe offending line is:\n%s\n",sextractor_catalog,sextractor_catalog_string);
				         continue;
				        }
                                        // Now this is some crazy stuff:
                                        // The last columns of the SExtractor catalog file might be:
                                        // ... flags
                                        // ... flags external_flags
                                        // ... flags external_flags psf_fitting_chi2
                                        // ... flags psf_fitting_chi2
                                        // Below we try to handle each of the four possibilities
                                        //
                                        // if these are not just flags
                                        // (but make sure we perform the tese only on a line wit ha good measurement)
                                        if( strlen(external_flag_string)>0 && flux_adu>0.0 && mag!=99.0000 ){
                                         // if these are not "flags external_flags psf_fitting_chi2"
                                         if( 2!=sscanf(external_flag_string,"%lf %lf",&double_external_flag,&psf_chi2) ){
                                          // Decide between "flags external_flags" and "flags psf_fitting_chi2"
                                          for(ii=0,jj=0;ii<(int)strlen(external_flag_string);ii++){
                                           if( external_flag_string[ii]=='.' || external_flag_string[ii]=='e' ){jj=1;break;} // assume that a decimal point indicates psf_chi2 rather than external_flag that is expected to be 0 or 1 only
                                          }
                                          if( jj==0 ){
                                           // "flags external_flags" case
                                           psf_chi2=1.0; // no PSF fitting results
                                           if( 1!=sscanf(external_flag_string,"%lf",&double_external_flag) ){
                                            double_external_flag=0.0; // no external flag image used
                                           }
                                          }
                                          else{
                                           // "flags psf_fitting_chi2" case
                                           double_external_flag=0.0; // no external flag image used
                                           if( 1!=sscanf(external_flag_string,"%lf",&psf_chi2) ){
                                            psf_chi2=1.0; // no PSF fitting results
                                           }
                                          }
                                         } // if( 2!=sscanf(external_flag_string,"%lf %lf",&double_external_flag,&psf_chi2) ){
                                         external_flag=(int)double_external_flag;
                                        }
                                        else{
                                         psf_chi2=1.0; // no PSF fitting results
                                         external_flag=0; // no external flag image used
                                        }

                                        //if( strlen(external_flag_string)>0 ){
                                        // if( 2!=sscanf(external_flag_string,"%lf %lf",&double_external_flag,&psf_chi2) ){
                                        //  psf_chi2=1.0; // no PSF fitting results
                                        //  if( 1!=sscanf(external_flag_string,"%lf",&double_external_flag) ){
                                        //   double_external_flag=0.0; // no external flag image used
                                        //  }
                                        // }
                                        // external_flag=(int)double_external_flag;
                                        //}
                                        //else{
                                        // psf_chi2=1.0; // no PSF fitting results
                                        // external_flag=0; // no external flag image used
                                        //}

				        // Check if the catalog line is a really band one
				        if( flux_adu==0 ){counter_rejected_bad_flux++;continue;}
				        if( flux_adu_err==999999 ){counter_rejected_bad_flux++;continue;}
				        if( mag==99.0000 ){counter_rejected_bad_flux++;continue;}
				        if( sigma_mag==99.0000 ){counter_rejected_bad_flux++;continue;}
				        if( flux_adu<MIN_SNR*flux_adu_err ){counter_rejected_low_snr++;continue;}
				        if( 0 != exclude_region(X1, Y1, X2, Y2, N_bad_regions, position_x_pix, position_y_pix, aperture) ){counter_rejected_bad_region++;continue;}
				        if( position_x_pix < FRAME_EDGE_INDENT_PIXELS || position_y_pix < FRAME_EDGE_INDENT_PIXELS || fabs(position_x_pix - X_im_size) < FRAME_EDGE_INDENT_PIXELS || fabs(position_y_pix - Y_im_size) < FRAME_EDGE_INDENT_PIXELS ){counter_rejected_frame_edge++;continue;}
				        //
				        if( a_a+a_a_err<FWHM_MIN ){counter_rejected_too_small++;continue;}
				        if( a_b+a_b_err<FWHM_MIN ){counter_rejected_too_small++;continue;}
				        //
				        if( a_a>aperture ){counter_rejected_too_large++;continue;}
				        if( external_flag!=0 ){counter_rejected_external_flag++;continue;}
				        //
					NUMBER2++;
					STAR2[NUMBER2-1].vast_flag=0;
					//if (position_x_pix < FRAME_EDGE_INDENT_PIXELS || position_y_pix < FRAME_EDGE_INDENT_PIXELS || fabs(position_x_pix - X_im_size) < FRAME_EDGE_INDENT_PIXELS || fabs(position_y_pix - Y_im_size) < FRAME_EDGE_INDENT_PIXELS )
		                        // STAR2[NUMBER2-1].vast_flag+=5;
                                        //if( 0 != exclude_region(X1, Y1, X2, Y2, N_bad_regions, position_x_pix, position_y_pix, aperture) ) 
                                        // STAR2[NUMBER2-1].vast_flag+=40;
                                        //if( flux_adu<MIN_SNR*flux_adu_err )
                                        // STAR2[NUMBER2-1].vast_flag+=300;
                                        //if( mag==99.0000 )
                                        // STAR2[NUMBER2-1].vast_flag+=2000;

					STAR2[NUMBER2-1].n = star_number_in_sextractor_catalog;
					STAR2[NUMBER2-1].x = (float)position_x_pix;
					STAR2[NUMBER2-1].y = (float)position_y_pix;
					STAR2[NUMBER2-1].flux = flux_adu;
					//Теперь считаем ошибки правильно, а не как трактор ;)
					STAR2[NUMBER2-1].mag = (float)mag;
					STAR2[NUMBER2-1].sigma_mag = (float)sigma_mag; 
					STAR2[NUMBER2-1].JD = JD;
					STAR2[NUMBER2-1].x_frame=STAR2[NUMBER2-1].x;
					STAR2[NUMBER2-1].y_frame=STAR2[NUMBER2-1].y;
					STAR2[NUMBER2-1].detected_on_ref_frame=0; // Mark the star that it is not on the reference frame
					STAR2[NUMBER2-1].sextractor_flag=(char)sextractor_flag; // SEXtractor flag ;)
					//STAR2[NUMBER2-1].distance_to_neighbor_squared=4*a_a*a_a; // EXPERIMENTAL !!!
					STAR2[NUMBER2-1].star_size=a_a;
					
                                        /// !!! Debug !!!
                                        //float debug_x=586.696;
                                        //float debug_y=438.463;
                                        ///float debug_d=sqrt((STAR2[NUMBER2-1].x_frame-debug_x)*(STAR2[NUMBER2-1].x_frame-debug_x)+(STAR2[NUMBER2-1].y_frame-debug_y)*(STAR2[NUMBER2-1].y_frame-debug_y));
                                        ///if( debug_d<aperture ){
                                        // fprintf(stderr,"READING SEXTRACTOR CAT: %d  %.3f %.3f  (%f)  s=%d v=%6d\n",STAR2[NUMBER2-1].n,STAR2[NUMBER2-1].x_frame,STAR2[NUMBER2-1].y_frame,STAR2[NUMBER2-1].star_size,STAR2[NUMBER2-1].sextractor_flag,STAR2[NUMBER2-1].vast_flag);
                                        //}

					if (debug != 0) fprintf(stderr, "DEBUG MSG: Reading test.cat NUMBER2=%d Ok\n",NUMBER2);
				}
				fclose(file);
	                        if (debug != 0) fprintf(stderr, "DEBUG MSG: Finished reading cat file for %s\n",base_star[n]);
				/* end of Read_sex_cat2 */

                                // Print out input catalog filtering stats
                                fprintf(stderr,"SExtractor output filtering results:\n * passed selection: %d\n * rejected as having no flux measurement: %d\n * rejected as having SNR<%.1lf: %d\n * rejected inside rectangles defined in bad_region.lst: %d\n * rejected close to frame edge: %d\n * rejected as being too small (<%.1lf pix): %d\n * rejected as being too large (>%.1lf pix): %d\n * rejected due to an external image flag: %d\n * rejected as having bad PSF fit: %d\n---\n",NUMBER2,counter_rejected_bad_flux, MIN_SNR,counter_rejected_low_snr,counter_rejected_bad_region,counter_rejected_frame_edge, FWHM_MIN,counter_rejected_too_small, aperture,counter_rejected_too_large, counter_rejected_external_flag, counter_rejected_bad_psf_fit);
                                fprintf(stderr,"You may change some of the filtering parameters by editing src/limits.h and re-running 'make'.\n");
                                // Check the stats and issue warnings if needed
                                if( (double)counter_rejected_too_small/(double)NUMBER1>0.3 )fprintf(stderr,"\E[01;31m WARNING: \E[33;00m suspiciously many stars are rejected as being too small. Please check FWHM_MIN in src/limits.h and re-run 'make' if you change it.\n");

				
				if (debug != 0) fprintf(stderr, "DEBUG MSG: NUMBER2=%d Ok\n",NUMBER2);
				/* If we see enough stars... */
				if( NUMBER2>MIN_NUMBER_OF_STARS_ON_FRAME ){
				 Pos2 = malloc(sizeof(int) * NUMBER2);
				 if( Pos2==NULL ){fprintf(stderr,"ERROR: can't allocate memory for Pos2\n");vast_report_memory_error();return 1;}
				 if (debug != 0) fprintf(stderr, "DEBUG MSG: Ident(preobr etc) - ");
				 
				 //set_distance_to_neighbor_in_struct_Star(STAR2, NUMBER2, aperture, X_im_size, Y_im_size); // set distance to the closest neighbor for each star.

                                 Sort_in_mag_of_stars(STAR2, NUMBER2);
                                 best_number_of_matched_stars=0;
                                 best_number_of_matched_stars=0;
                                 preobr->Number_of_ecv_triangle=default_Number_of_ecv_triangle;
                                 preobr->Number_of_main_star=default_Number_of_main_star;
                                 for(match_try=0;match_try<MAX_MATCH_TRIALS;match_try++){
                                  match_retry=0;
                                  
                                  /* Identify stars */
                                  if( param_set_manually_Number_of_main_star==0 ){
                                   //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(2*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), X_im_size, Y_im_size);
                                   Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(2*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), max_X_im_size, max_Y_im_size);
                                  }
                                  else{
                                   //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), X_im_size, Y_im_size);
                                   Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), max_X_im_size, max_Y_im_size);
                                   if( match_retry==1 )match_retry=0;
                                  }
                                  
				  /* Test if match attempt was a success */
				  if( match_retry==0 )break;
				  if( Number_of_ecv_star>best_number_of_matched_stars ){fprintf(stderr,"the best number of matched * so far: %d\n",Number_of_ecv_star);best_number_of_matched_stars=Number_of_ecv_star;best_number_of_reference_stars=preobr->Number_of_main_star;}
				  if( preobr->Number_of_main_star>=MIN(MATCH_MAX_NUMBER_OF_REFERENCE_STARS,MIN(NUMBER2,NUMBER3)) ||preobr->Number_of_ecv_triangle>=MATCH_MAX_NUMBER_OF_TRIANGLES )
				   break;
				  /* Try to play with parameters */
				  preobr->Number_of_main_star=preobr->Number_of_main_star*2;
				  preobr->Number_of_ecv_triangle=preobr->Number_of_ecv_triangle*2;
				  if( preobr->Number_of_main_star==2*MIN(MATCH_MAX_NUMBER_OF_REFERENCE_STARS,MIN(NUMBER2,NUMBER3)) )break; // So we do not repeat many times attempt with the maximum number of reference stars
				  if( preobr->Number_of_main_star>MIN(MATCH_MAX_NUMBER_OF_REFERENCE_STARS,MIN(NUMBER2,NUMBER3)) ||preobr->Number_of_ecv_triangle>MATCH_MAX_NUMBER_OF_TRIANGLES ){
                                   preobr->Number_of_main_star=MIN(MATCH_MAX_NUMBER_OF_REFERENCE_STARS,MIN(NUMBER2,NUMBER3));
                                   preobr->Number_of_ecv_triangle=MATCH_MAX_NUMBER_OF_TRIANGLES;
                                  }
				 }

                                 /* !!!!! */
                                 /*
                                 if( match_retry==1 ){
                                  fprintf(stderr,"TESTTTTTT!\n");
                                  preobr->Number_of_ecv_triangle=default_Number_of_ecv_triangle;
                                  preobr->Number_of_main_star=default_Number_of_main_star;
                                  for(match_try=0;match_try<MAX_MATCH_TRIALS;match_try++){
                                   match_retry=0;
                                   
                                   // Identify stars 
                                   if( param_set_manually_Number_of_main_star==0 ){
                                    Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR1, NUMBER1, 0, &match_retry, (int)(2*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER1,NUMBER2)), X_im_size, Y_im_size);
                                   }
                                   else{
                                    Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR1, NUMBER1, 0, &match_retry, (int)(MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER1,NUMBER2)), X_im_size, Y_im_size);
                                    if( match_retry==1 )match_retry=0;
                                   }
                                   
 				   // Test if match attempt was a success 
 				   if( match_retry==0 )break;
 				   if( Number_of_ecv_star>best_number_of_matched_stars ){fprintf(stderr,"the best number of matched * so far: %d\n",Number_of_ecv_star);best_number_of_matched_stars=Number_of_ecv_star;best_number_of_reference_stars=preobr->Number_of_main_star;}
 				   if( preobr->Number_of_main_star>=MIN(MATCH_MAX_NUMBER_OF_REFERENCE_STARS,MIN(NUMBER2,NUMBER1)) ||preobr->Number_of_ecv_triangle>=MATCH_MAX_NUMBER_OF_TRIANGLES )
				    break;
				   // Try to play with parameters 
				   preobr->Number_of_main_star=preobr->Number_of_main_star*2;
				   preobr->Number_of_ecv_triangle=preobr->Number_of_ecv_triangle*2;
				   if( preobr->Number_of_main_star==2*MIN(MATCH_MAX_NUMBER_OF_REFERENCE_STARS,MIN(NUMBER2,NUMBER1)) )break; // So we do not repeat many times attempt with the maximum number of reference stars
				   if( preobr->Number_of_main_star>MIN(MATCH_MAX_NUMBER_OF_REFERENCE_STARS,MIN(NUMBER2,NUMBER1)) ||preobr->Number_of_ecv_triangle>MATCH_MAX_NUMBER_OF_TRIANGLES ){
                                    preobr->Number_of_main_star=MIN(MATCH_MAX_NUMBER_OF_REFERENCE_STARS,MIN(NUMBER2,NUMBER1));
                                    preobr->Number_of_ecv_triangle=MATCH_MAX_NUMBER_OF_TRIANGLES;
                                   }
                                  }
                                 }
                                 */
                                 /* !!!!! */
				 
				 /* Special test for the case if one image is much better focused than the other and
				    all the reference stars got saturated on it... */
                                 if( match_retry==1 )fprintf(stderr,"Performing special test for the case of a sudden focus change...\n");
                                 // current image
                                 if( match_retry==1 && NUMBER2>2*preobr->Number_of_main_star ){
                                  match_retry=0;
                                  //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, preobr->Number_of_main_star, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(1.5*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), X_im_size, Y_im_size);
                                  Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, preobr->Number_of_main_star, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(1.5*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), max_X_im_size, max_Y_im_size);
                                 }
                                 else{
                                  if( match_retry==1 )fprintf(stderr,"Ups, not enough stars for the test! Whatever...\n");
                                 }
                                 // reference image
                                 if( match_retry==1 && NUMBER3>2*preobr->Number_of_main_star ){
                                  match_retry=0;
                                  //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, preobr->Number_of_main_star, &match_retry, (int)(1.5*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), X_im_size, Y_im_size);
                                  Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, Pos1, Pos2, no_rotation, STAR3, NUMBER3, preobr->Number_of_main_star, &match_retry, (int)(1.5*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), max_X_im_size, max_Y_im_size);
                                 }
                                 else{
                                  if( match_retry==1 )fprintf(stderr,"Ups, not enough stars for the test! Whatever...\n");
                                 }
				 if( match_retry==1 )fprintf(stderr,"Done with the test...\n");
				 
				 /* If increasing number of reference stars did not help, try to decrease the number */
				 if( match_retry==1 ){
				  // start with 1 so test match_try!=0 will work later
                                  for(match_try=1;match_try<MAX_MATCH_TRIALS+1;match_try++){

				   /* Try to play with parameters */
				   preobr->Number_of_main_star=preobr->Number_of_main_star/2;
				   preobr->Number_of_ecv_triangle=preobr->Number_of_ecv_triangle/2;
				   
				   if( preobr->Number_of_main_star<MATCH_MIN_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle<MATCH_MIN_NUMBER_OF_TRIANGLES ){
                                    preobr->Number_of_main_star=MATCH_MIN_NUMBER_OF_REFERENCE_STARS;
                                    preobr->Number_of_ecv_triangle=MATCH_MIN_NUMBER_OF_TRIANGLES;
                                   }

                                   match_retry=0;
                                   //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(2*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), X_im_size, Y_im_size);
                                   Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(2*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), max_X_im_size, max_Y_im_size);
				   /* Test if match attemt was a success */
				   if( match_retry==0 )break;
				   if( Number_of_ecv_star>best_number_of_matched_stars ){fprintf(stderr,"the best number of matched * so far: %d\n",Number_of_ecv_star);best_number_of_matched_stars=Number_of_ecv_star;best_number_of_reference_stars=preobr->Number_of_main_star;}
				   if( preobr->Number_of_main_star<=MATCH_MIN_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle<=MATCH_MIN_NUMBER_OF_TRIANGLES )
				    break;

				  }				 
				 }
				 
				 /* If that did not help, try many different numbers of reference stars */
				 /*
				 if( match_retry==1 ){
				  // start with 1 so test match_try!=0 will work later
				  // MIN !!!
				  // is the following line correct?
				  preobr->Number_of_ecv_triangle=MATCH_MAX_NUMBER_OF_TRIANGLES;
                                  for(preobr->Number_of_main_star=MATCH_MIN_NUMBER_OF_REFERENCE_STARS,preobr->Number_of_ecv_triangle=MATCH_MIN_NUMBER_OF_TRIANGLES;preobr->Number_of_main_star<MIN(MATCH_MAX_NUMBER_OF_REFERENCE_STARS,MIN(NUMBER2,NUMBER3));preobr->Number_of_main_star+=MATCH_REFERENCE_STARS_NUMBER_STEP){ //,preobr->Number_of_ecv_triangle+=MATCH_REFERENCE_STARS_NUMBER_STEP){
                                   // Make sure we are not trying to set too much reference stars, just to speed things up
                                   if( preobr->Number_of_main_star>=MIN(MATCH_MAX_NUMBER_OF_REFERENCE_STARS,MIN(NUMBER2,NUMBER3)) ||preobr->Number_of_ecv_triangle>=MATCH_MAX_NUMBER_OF_TRIANGLES )
				    break;

                                   match_retry=0;
                                   Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry,(int)(2*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), X_im_size, Y_im_size);
				   // Test if match attemt was a success 
				   if( match_retry==0 )break;
				   if( Number_of_ecv_star>best_number_of_matched_stars ){fprintf(stderr,"the best number of matched * so far: %d\n",Number_of_ecv_star);best_number_of_matched_stars=Number_of_ecv_star;best_number_of_reference_stars=preobr->Number_of_main_star;}
				   if( preobr->Number_of_main_star<MATCH_MIN_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle<MATCH_MIN_NUMBER_OF_TRIANGLES ){
                                    preobr->Number_of_main_star=MATCH_MIN_NUMBER_OF_REFERENCE_STARS;
                                    preobr->Number_of_ecv_triangle=MATCH_MIN_NUMBER_OF_TRIANGLES;
                                   }
				  }				 
				 }
				 */
				 
				 /* If it still doesn't work - try to decrease an accpetable number of matched stars */
				 if( match_retry==1 ){
				  preobr->Number_of_main_star=best_number_of_reference_stars;
                                                                                               // is this correct?
				  preobr->Number_of_ecv_triangle=MATCH_MAX_NUMBER_OF_TRIANGLES;// best_number_of_reference_stars;
				  fprintf(stderr,"Trying again the best match with %d reference stars\n",best_number_of_reference_stars);
                                  //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(1.5*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), X_im_size, Y_im_size);
                                  Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(1.5*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), max_X_im_size, max_Y_im_size);
                                 }
				 /*----------------------------------------------------------------------------------*/
				 if( match_retry==1 ){
				  preobr->Number_of_ecv_triangle=default_Number_of_ecv_triangle;
                                  preobr->Number_of_main_star=default_Number_of_main_star;
                                  for(match_try=0;match_try<MAX_MATCH_TRIALS;match_try++){
                                   match_retry=0;
                                   //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(1.5*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER2,NUMBER3)), X_im_size, Y_im_size);
                                   Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(1.5*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER2,NUMBER3)), max_X_im_size, max_Y_im_size);
				   /* Test if match attemt was a success */
				   if( match_retry==0 )break;
				   if( Number_of_ecv_star>best_number_of_matched_stars ){fprintf(stderr,"the best number of matched * so far: %d\n",Number_of_ecv_star);best_number_of_matched_stars=Number_of_ecv_star;best_number_of_reference_stars=preobr->Number_of_main_star;}
				   if( preobr->Number_of_main_star>=MATCH_MAX_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle>=MATCH_MAX_NUMBER_OF_TRIANGLES )
				    break;
				   /* If not, try to play with parameters */
				   preobr->Number_of_main_star=preobr->Number_of_main_star*2;
				   preobr->Number_of_ecv_triangle=preobr->Number_of_ecv_triangle*2;
				   if( preobr->Number_of_main_star==2*MIN(MATCH_MAX_NUMBER_OF_REFERENCE_STARS,MIN(NUMBER2,NUMBER3)) )break; // So we do not repeat many times attempt with the maximum number of reference stars
				   if( preobr->Number_of_main_star>MATCH_MAX_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle>MATCH_MAX_NUMBER_OF_TRIANGLES ){
                                    preobr->Number_of_main_star=MATCH_MAX_NUMBER_OF_REFERENCE_STARS;
                                    preobr->Number_of_ecv_triangle=MATCH_MAX_NUMBER_OF_TRIANGLES;
                                   }
				  }
				 }
				 
				 /* If increasing number of reference stars did not help, try to decrease the number */
				 if( match_retry==1 ){
				  // start with 1 so test match_try!=0 will work later
                                  for(match_try=1;match_try<MAX_MATCH_TRIALS+1;match_try++){
                                   match_retry=0;
                                   //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(1.5*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), X_im_size, Y_im_size);
                                   Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(1.5*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), max_X_im_size, max_Y_im_size);
				   /* Test if match attemt was a success */
				   if( match_retry==0 )break;
				   if( Number_of_ecv_star>best_number_of_matched_stars ){fprintf(stderr,"the best number of matched * so far: %d\n",Number_of_ecv_star);best_number_of_matched_stars=Number_of_ecv_star;best_number_of_reference_stars=preobr->Number_of_main_star;}
				   if( preobr->Number_of_main_star<=MATCH_MIN_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle<=MATCH_MIN_NUMBER_OF_TRIANGLES )
				    break;
				   /* If not, try to play with parameters */
				   preobr->Number_of_main_star=preobr->Number_of_main_star/2;
				   preobr->Number_of_ecv_triangle=preobr->Number_of_ecv_triangle/2;
				   if( preobr->Number_of_main_star<MATCH_MIN_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle<MATCH_MIN_NUMBER_OF_TRIANGLES ){
                                    preobr->Number_of_main_star=MATCH_MIN_NUMBER_OF_REFERENCE_STARS;
                                    preobr->Number_of_ecv_triangle=MATCH_MIN_NUMBER_OF_TRIANGLES;
                                   }
				  }				 
				 }
				 
				 /* If that did not help, try many different numbers of reference stars */
				 if( match_retry==1 ){
				  // start with 1 so test match_try!=0 will work later
				  // is the follwing line correct?
				  preobr->Number_of_ecv_triangle=MATCH_MAX_NUMBER_OF_TRIANGLES;
                                  for(preobr->Number_of_main_star=MATCH_MIN_NUMBER_OF_REFERENCE_STARS,preobr->Number_of_ecv_triangle=MATCH_MIN_NUMBER_OF_TRIANGLES;preobr->Number_of_main_star<MATCH_MAX_NUMBER_OF_REFERENCE_STARS;preobr->Number_of_main_star+=MATCH_REFERENCE_STARS_NUMBER_STEP){
                                   match_retry=0;
                                   //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry,(int)(1.5*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), X_im_size, Y_im_size);
                                   Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry,(int)(1.5*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), max_X_im_size, max_Y_im_size);
				   /* Test if match attemt was a success */
				   if( match_retry==0 )break;
				   if( Number_of_ecv_star>best_number_of_matched_stars ){fprintf(stderr,"the best number of matched * so far: %d\n",Number_of_ecv_star);best_number_of_matched_stars=Number_of_ecv_star;best_number_of_reference_stars=preobr->Number_of_main_star;}
				   if( preobr->Number_of_main_star<MATCH_MIN_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle<MATCH_MIN_NUMBER_OF_TRIANGLES ){
                                    preobr->Number_of_main_star=MATCH_MIN_NUMBER_OF_REFERENCE_STARS;
                                    preobr->Number_of_ecv_triangle=MATCH_MIN_NUMBER_OF_TRIANGLES;
                                   }
				  }				 
				 }

				 
				 /*----------------------------------------------------------------------------------*/
				 if( match_retry==1 ){
				  preobr->Number_of_main_star=best_number_of_reference_stars;
				                                 // is this correct??
				  preobr->Number_of_ecv_triangle=MATCH_MAX_NUMBER_OF_TRIANGLES;//best_number_of_reference_stars;
				  fprintf(stderr,"Trying again the best match with %d reference stars\n",best_number_of_reference_stars);
                                  //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(1.0*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), X_im_size, Y_im_size);
                                  Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(1.0*MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), max_X_im_size, max_Y_im_size);
                                 }
				 /*----------------------------------------------------------------------------------*/
				 
				 /* Test if there is any hope to match this image */
				 if( match_retry==1 ){
				  if( Number_of_ecv_star<(int)(MIN_FRACTION_OF_MATCHED_STARS_STOP_ATTEMPTS*MIN(NUMBER3,NUMBER2)) ){
				   fprintf(stderr,"ERROR! Too few stars matched (%d<%d) after a few iterations! Something may be wrong with this image, skipping it!\n",Number_of_ecv_star,(int)(MIN_FRACTION_OF_MATCHED_STARS_STOP_ATTEMPTS*MIN(NUMBER3,NUMBER2)));
				   Number_of_ecv_star=0;
				   match_retry=0;
				  }
				 }
				 
				 if( match_retry==1 ){
				  preobr->Number_of_ecv_triangle=default_Number_of_ecv_triangle;
                                  preobr->Number_of_main_star=default_Number_of_main_star;
                                  for(match_try=0;match_try<MAX_MATCH_TRIALS;match_try++){
                                   match_retry=0;
                                   //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER2,NUMBER3)), X_im_size, Y_im_size);
                                   Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER2,NUMBER3)), max_X_im_size, max_Y_im_size);
				   /* Test if match attemt was a success */
				   if( match_retry==0 )break;
				   if( preobr->Number_of_main_star>=MATCH_MAX_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle>=MATCH_MAX_NUMBER_OF_TRIANGLES )
				    break;
				   /* If not, try to play with parameters */
				   preobr->Number_of_main_star=preobr->Number_of_main_star*2;
				   preobr->Number_of_ecv_triangle=preobr->Number_of_ecv_triangle*2;
				   if( preobr->Number_of_main_star>MATCH_MAX_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle>MATCH_MAX_NUMBER_OF_TRIANGLES ){
                                    preobr->Number_of_main_star=MATCH_MAX_NUMBER_OF_REFERENCE_STARS;
                                    preobr->Number_of_ecv_triangle=MATCH_MAX_NUMBER_OF_TRIANGLES;
                                   }
				  }
				 }
				 
				 /* If increasing number of reference stars did not help, try to decrease the number */
				 if( match_retry==1 ){
				  // start with 1 so test match_try!=0 will work later
                                  for(match_try=1;match_try<MAX_MATCH_TRIALS+1;match_try++){
                                   match_retry=0;
                                   //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), X_im_size, Y_im_size);
                                   Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry, (int)(MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), max_X_im_size, max_Y_im_size);
				   /* Test if match attemt was a success */
				   if( match_retry==0 )break;
				   if( preobr->Number_of_main_star<=MATCH_MIN_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle<=MATCH_MIN_NUMBER_OF_TRIANGLES )
				    break;
				   /* If not, try to play with parameters */
				   preobr->Number_of_main_star=preobr->Number_of_main_star/2;
				   preobr->Number_of_ecv_triangle=preobr->Number_of_ecv_triangle/2;
				   if( preobr->Number_of_main_star<MATCH_MIN_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle<MATCH_MIN_NUMBER_OF_TRIANGLES ){
                                    preobr->Number_of_main_star=MATCH_MIN_NUMBER_OF_REFERENCE_STARS;
                                    preobr->Number_of_ecv_triangle=MATCH_MIN_NUMBER_OF_TRIANGLES;
                                   }
				  }				 
				 }
				 
				 /* If that did not help, try many different numbers of reference stars */
				 if( match_retry==1 ){
				  // start with 1 so test match_try!=0 will work later
				  // is the follwing line correct ?
				  preobr->Number_of_ecv_triangle=MATCH_MAX_NUMBER_OF_TRIANGLES;
                                  for(preobr->Number_of_main_star=MATCH_MIN_NUMBER_OF_REFERENCE_STARS,preobr->Number_of_ecv_triangle=MATCH_MIN_NUMBER_OF_TRIANGLES;preobr->Number_of_main_star<MATCH_MAX_NUMBER_OF_REFERENCE_STARS;preobr->Number_of_main_star+=MATCH_REFERENCE_STARS_NUMBER_STEP){
                                   match_retry=0;
                                   //Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, frame, frame, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry,(int)(MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), X_im_size, Y_im_size);
                                   Number_of_ecv_star = Ident(preobr, STAR1, NUMBER1, STAR2, NUMBER2, 0, Pos1, Pos2, no_rotation, STAR3, NUMBER3, 0, &match_retry,(int)(MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)), max_X_im_size, max_Y_im_size);
				   /* Test if match attemt was a success */
				   if( match_retry==0 )break;
				   if( preobr->Number_of_main_star<MATCH_MIN_NUMBER_OF_REFERENCE_STARS ||preobr->Number_of_ecv_triangle<MATCH_MIN_NUMBER_OF_TRIANGLES ){
                                    preobr->Number_of_main_star=MATCH_MIN_NUMBER_OF_REFERENCE_STARS;
                                    preobr->Number_of_ecv_triangle=MATCH_MIN_NUMBER_OF_TRIANGLES;
                                   }
				  }				 
				 }

				 
				 /*----------------------------------------------------------------------------------*/				 
				 if (debug != 0) fprintf(stderr, "Ok\n");
				 
				 
                                 // Check if this is the reference image again? We don't want to measure it for the second time and write
                                 // duplicate points in all lightcurves!
                                 // Compare file names with the reference image! Old criteria - exectly 180 deg. rotation may not work now
                                 // since avaraged computed star coordinates are now used for image matching, not coordinates obtained 
                                 // directly from the reference image!  
                                 if( 0==strcmp(base_star[0],base_star[n]) ){ //&& increment_mode!=1 ){
                                  if (debug != 0) fprintf(stderr, "It is the reference frame again! File name match!\n");
                                  preobr->fi=M_PI;
                                 }
                                 if ( fabs(180*preobr->fi/M_PI-180.0)<0.0001 ){
                                  preobr->fi=M_PI; // set 180 even if it is not (but file name matches)!
                                  fprintf(stderr, " rotation is exactly 180 degrees! Is this the reference image again? Dropping image!  %lf\n", 180*preobr->fi / M_PI);
                                  Number_of_ecv_star=0;
                                 }
                                 
                                 // Write DS9 region files for debug
                                 //write_Star_struct_to_ds9_region_file(STAR1, 0, NUMBER1, "STAR1_ds9.reg", aperture);
                                 //write_Star_struct_to_ds9_region_file(STAR2, 0, NUMBER2, "STAR2_ds9.reg", aperture);
                                 //write_Star_struct_to_ds9_region_file(STAR3, 0, NUMBER3, "STAR3_ds9.reg", aperture);
                                 //write_Star_struct_to_ds9_region_file(STAR2, Number_of_ecv_star, NUMBER2, "STAR_UNID_ds9.reg", aperture);
                                 //exit(0); // !!! 
                                 //write_Star_struct_to_ASCII_file(STAR1, 0, NUMBER1, "image_referece.txt", aperture);
                                 //sprintf(log_output,"image_%03d.txt",n);
                                 //write_Star_struct_to_ASCII_file(STAR2, 0, NUMBER2, log_output, aperture);
                                 
                                 // Try to eliminate hot pixels from the star list
                                 //flag_false_star_detections_caused_by_hot_pixels( STAR1, Pos1, &NUMBER1, STAR2, Pos2, &NUMBER2, Number_of_ecv_star);
                                 // ...
                                 
				 /* log */
				 sprintf(log_output,"rotation= %7.3lf  ", 180*preobr->fi / M_PI);
				 write_string_to_log_file(log_output,sextractor_catalog);
				 fprintf(stderr, "  rotation [degrees] = %7.3lf\n", 180*preobr->fi / M_PI);
				 if(no_rotation==1 && fabs(preobr->fi) > 0.052353 && fabs(preobr->fi-M_PI) > 0.052353 ){
				  fprintf(stderr, " rotation is large! Dropping image!  %lf\n", 180*preobr->fi / M_PI);
				  Number_of_ecv_star=0;
                                 }
                                 
                                 fprintf(stderr,"    %5d stars detected, %5d stars matched\n",NUMBER2, Number_of_ecv_star);
				}
				else{
				 /* Else write log and go to next image */
				 Number_of_ecv_star=0;
				 fprintf(stderr, "  rotation [degrees] = %7.3lf\n", 180.000);
				 write_string_to_log_file(log_output,sextractor_catalog);
				}
				
				sprintf(log_output,"*detected= %5d  *matched= %5d  ", NUMBER2, Number_of_ecv_star);
				write_string_to_log_file(log_output,sextractor_catalog);
				
				
				/* If not enough stars were matched...*/
				if ( Number_of_ecv_star<(int)(MIN_FRACTION_OF_MATCHED_STARS*MIN(NUMBER3,NUMBER2)) ) { // We should compare with the number of stars on the reference frame, not with the total number of stars!
				 fprintf(stderr,"ERROR! Too few stars matched. Wrong match? Skipping file...\n");
				 Number_of_ecv_star=0;
				}
				
				if (Number_of_ecv_star == 0) {
				        if (debug != 0) fprintf(stderr, "DEBUG MSG: Writing status=ERROR to log file - ");
					//fprintf(vast_image_details,"status=ERROR  %s\n",base_star[n]);
					sprintf(log_output,"status=ERROR  %s\n",base_star[n]);
					write_string_to_log_file(log_output,sextractor_catalog);
					if (debug != 0) fprintf(stderr, "Ok\n");
					if (debug != 0) fprintf(stderr, "DEBUG MSG: Closing log file - ");
					//fclose(vast_image_details);
					if (debug != 0) fprintf(stderr, "Ok\n");
					if( NUMBER2>MIN_NUMBER_OF_STARS_ON_FRAME )free(Pos2);
					if (debug != 0) fprintf(stderr, "DEBUG MSG: free(STAR2) - ");
					free(STAR2);
					if (debug != 0) fprintf(stderr, "Ok\n");
					continue; // Go to next image
				}
				//MATCH_SUCESS++;

/*				
				//// **** Add new stars (which were not detected on the reference frame) to STAR1 - the structure with reference stars  ****
				//if( param_failsafe==0 ){
				if( param_failsafe==0 && Number_of_ecv_star > 0 ){

				 Pos1 = realloc(Pos1, sizeof(int) * ( MAX(NUMBER1,NUMBER2) + 1) );
				 
				 for (i = Number_of_ecv_star; i < NUMBER2; i++){
				  //fprintf(stderr,"DEBUG: NUMBER1=%d i=%d Number_of_ecv_star=%d NUMBER2=%d\n",NUMBER1,i,Number_of_ecv_star,NUMBER2);
				  // If this is a good star
				  if (fabs(STAR2[Pos2[i]].mag) > BRIGHTEST_STARS && STAR2[Pos2[i]].mag < faintest_stars  && X_im_size-STAR2[Pos2[i]].x_frame>FRAME_EDGE_INDENT_PIXELS && Y_im_size-STAR2[Pos2[i]].y_frame>FRAME_EDGE_INDENT_PIXELS && STAR2[Pos2[i]].x_frame>FRAME_EDGE_INDENT_PIXELS && STAR2[Pos2[i]].y_frame>FRAME_EDGE_INDENT_PIXELS && STAR2[Pos2[i]].sextractor_flag<=maxsextractorflag && STAR2[Pos2[i]].vast_flag==0 ){
				   // Add it to STAR1
				   if( STAR1==NULL ){fprintf(stderr,"ERROR: can't allocate memory for a new star!\n STAR1 = realloc(STAR1, sizeof(struct Star) * (NUMBER1+1) ); - failed!\n");return 1;}
				   //Pos1 = realloc(Pos1, sizeof(int) * (NUMBER1+1) ); // that was working for us all the time
				   //Pos1 = realloc(Pos1, sizeof(int) * ( MAX(i,NUMBER1) +1) ); // does not work
				   if( Pos1==NULL ){fprintf(stderr,"ERROR: can't allocate memory for a new star!\n Pos1 = realloc(Pos1, sizeof(int) * (NUMBER1+1) ); - failed!\n");return 1;}
				   Pos1[i] = NUMBER1;
				   max_number++;
				   STAR2[Pos2[i]].n=max_number;
				   Star_Copy(STAR1 + Pos1[i], STAR2 + Pos2[i]);
				   STAR1[Pos1[i]].mag=0.0;
				   NUMBER1++;
				  
				   /// !!! Debug !!!
				   //if(n==8)write_single_Star_from_struct_to_ds9_region_file(STAR2, Pos2[i], Pos2[i]+1, "STAR_UNID_ds9.reg", aperture);
                                   //float debug_x=586.696;
                                   //float debug_y=438.463;
                                   //float debug_d=sqrt((STAR2[Pos2[i]].x_frame-debug_x)*(STAR2[Pos2[i]].x_frame-debug_x)+(STAR2[Pos2[i]].y_frame-debug_y)*(STAR2[Pos2[i]].y_frame-debug_y));
                                   //if( debug_d<aperture ){
                                   // fprintf(stderr,"ADDING NEW STARS: %d  %.3f %.3f  (%f)  s=%d v=%6d\n",STAR2[Pos2[i]].n,STAR2[Pos2[i]].x_frame,STAR2[Pos2[i]].y_frame,STAR2[Pos2[i]].star_size,STAR2[Pos2[i]].sextractor_flag,STAR2[Pos2[i]].vast_flag);
                                   //}

				  
				   if ( NUMBER1 >= MAX_NUMBER_OF_STARS-1 ) {
				    fprintf(stderr, "ERROR while adding new stars: Too many stars!\nChange string \"#define MAX_NUMBER_OF_STARS %d\" in src/limits.h file and recompile the program by running \"make\".\n", MAX_NUMBER_OF_STARS);
				    exit(1);
                                   }
                                  } // // If this is a good star
                                  //else
                                  // fprintf(stderr,"Bad star!\n");
				 }
				 
				} // if( param_failsafe==0 )
				
				fprintf(stderr,"    %5d objects stored in memory (%5d max.)\n",NUMBER1,MAX_NUMBER_OF_STARS);
				//if(n==8)exit(0);// !!
*/				
				/* Magnitude calibration. */
				//if (param_nocalib != 1) {
					
					if (debug != 0) fprintf(stderr, "DEBUG MSG: Mag. calibr start here.   Number_of_ecv_star=%d\n",Number_of_ecv_star); // Debug message!
					
					// allocate memory for the arrays
					N_good_stars = 0;
                                        poly_x = (double *)malloc(MAX_NUMBER_OF_STARS * sizeof(double));
                                        poly_y = (double *)malloc(MAX_NUMBER_OF_STARS * sizeof(double));
                                        poly_err = (double *)malloc(MAX_NUMBER_OF_STARS * sizeof(double));
                                        poly_err_fake = (double *)malloc(MAX_NUMBER_OF_STARS * sizeof(double));
                                        if( poly_x==NULL || poly_y==NULL || poly_err==NULL || poly_err_fake==NULL ){fprintf(stderr,"ERROR: can't allocate memory for magnitude calibration!\n");vast_report_memory_error();return 1;}
                                        
                                        if( apply_position_dependent_correction==1 ){
                                         lin_mag_cor_x = (double *)malloc(MAX_NUMBER_OF_STARS * sizeof(double));
                                         lin_mag_cor_y = (double *)malloc(MAX_NUMBER_OF_STARS * sizeof(double));
                                         lin_mag_cor_z = (double *)malloc(MAX_NUMBER_OF_STARS * sizeof(double));
                                         if( lin_mag_cor_x==NULL || lin_mag_cor_y==NULL || lin_mag_cor_z==NULL ){fprintf(stderr,"ERROR: can't allocate memory for magnitude correction!\n");vast_report_memory_error();return 1;}
                                        }
					
					// poppulate the arrays
					for (i = 0; i < MIN( Number_of_ecv_star, NUMBER3) ; i++) {                                                                                 
					        // STAR1[Pos1[i]].magmag==0.0 if the star was not detected on the reference frame
						if (0 == exclude_test(STAR1[Pos1[i]].x, STAR1[Pos1[i]].y, bad_stars_X, bad_stars_Y, N_bad_stars) && fabs(STAR1[Pos1[i]].mag - STAR2[Pos2[i]].mag) < MAX_INSTR_MAG_DIFF && STAR1[Pos1[i]].mag!=0.0  && STAR1[Pos1[i]].sextractor_flag==0 && STAR1[Pos1[i]].vast_flag==0 && STAR2[Pos2[i]].sextractor_flag==0 && STAR2[Pos2[i]].vast_flag==0 ) {
                                                        poly_x[N_good_stars] = (double)STAR2[Pos2[i]].mag;
                                                        poly_y[N_good_stars] = (double)STAR1[Pos1[i]].mag;
                                                        poly_err[N_good_stars] = (double)STAR2[Pos2[i]].sigma_mag;
                                                        poly_err_fake[N_good_stars] = 0.01; // fake error for unweighted fitting
                                                        
                                                        if( apply_position_dependent_correction==1 ){
                                                         lin_mag_cor_x[N_good_stars] = (double)STAR2[Pos2[i]].x_frame;
                                                         lin_mag_cor_y[N_good_stars] = (double)STAR2[Pos2[i]].y_frame;
                                                        }
                                                        
							N_good_stars += 1;
						}
					}
					// make sure we don't have an estimated error == 0.0
					for(i=0;i<N_good_stars;i++){
					 // Make sure poly_err[i] is not 0
					 if( poly_err[i]==0.0 ){
					  fprintf(stderr,"WARNING: zero error encountered while calibrating magnitudes!\n");
					  poly_err[i]=0.1; // assume some large error
					  //  Try to refine this wild guess by setting the error to the smallest significant value we have seen so far
					  for(j_poly_err=0;j_poly_err<N_good_stars;j_poly_err++){
					   if( poly_err[j_poly_err]<poly_err[i] && poly_err[j_poly_err]>0.0 ){poly_err[i]=poly_err[j_poly_err];}
					  }
					  fprintf(stderr,"Trying to handle it by setting the magnitude error to %.5lf\n",poly_err[i]);
					 }
					}
					// -------------------------------------------------
					if (debug != 0) fprintf(stderr, "%d stars selected for mag. calibr. (N_good_stars)\n",N_good_stars);
						if (debug != 0) fprintf(stderr, "DEBUG MSG: flush poly_coeff - ");
						poly_coeff[0] = 0;
						poly_coeff[1] = 0;
						poly_coeff[2] = 0;
						poly_coeff[3] = 0;
						poly_coeff[4] = 0;
						if (debug != 0) fprintf(stderr, "Ok\n");
						// If we don't have enough stars to perform a reliable magnitude calibration
						if( N_good_stars<MIN_NUMBER_STARS_POLY_MAG_CALIBR ){
							wpolyfit_exit_code=1;
						} else{
						        // Perform magnitude calibration iteratively removing outliers on the mag-mag plot
						        
						        /* Write data to log */
                                                        write_magnitude_calibration_log( poly_x, poly_y, poly_err, N_good_stars, base_star[n]);
							
							if (debug != 0) fprintf(stderr, "DEBUG MSG: GSL: wpolyfit - ");
							
							// First pass - remove really bad outliers usin unweighted linear approximation
							
                                                        // Use linear function as the very first approximation
                                                        wpolyfit_exit_code=wlinearfit(poly_x, poly_y, poly_err_fake, N_good_stars, poly_coeff);
							do{
							 the_baddest_outlier=0.0;
							 the_baddest_outlier_number=-1;
							 for(exclude_outlier_mags_counter=N_good_stars;exclude_outlier_mags_counter--;){
							  computed_mag=poly_coeff[1]*poly_x[exclude_outlier_mags_counter]+poly_coeff[0];
							  // find the baddest outlier
							  abs_computed_predicted_mag_diff=fabs(computed_mag-poly_y[exclude_outlier_mags_counter]);
							  if( abs_computed_predicted_mag_diff>3*MAX_DIFF_POLY_MAG_CALIBRATION && abs_computed_predicted_mag_diff>the_baddest_outlier ){
							   the_baddest_outlier=abs_computed_predicted_mag_diff;
                                                           the_baddest_outlier_number=exclude_outlier_mags_counter;
							  }
							 }
							 // And now remove the baddest outlier
							 if(the_baddest_outlier_number>=0){
							  // Exclude bad point from calibration
							  if( apply_position_dependent_correction==0 )
							   exclude_from_3_double_arrays(poly_x, poly_y, poly_err, the_baddest_outlier_number, &N_good_stars);
                                                          else
							   exclude_from_6_double_arrays(poly_x, poly_y, poly_err, lin_mag_cor_x, lin_mag_cor_y, lin_mag_cor_z, the_baddest_outlier_number, &N_good_stars);
							  // Recompute fit using unweighted linear fit
                                                          wpolyfit_exit_code=wlinearfit(poly_x, poly_y, poly_err_fake, N_good_stars, poly_coeff);
							 }
                                                        }while(the_baddest_outlier_number>=0);

							/* Check that we haven't dropped too many stars so the parabolic fit still make sence */
							if( N_good_stars<MIN_NUMBER_STARS_POLY_MAG_CALIBR )
							 wpolyfit_exit_code=1;
							
							
							// Linear CCD position dependent corrections
							if( apply_position_dependent_correction==1 && wpolyfit_exit_code==0 ){
							 // Populate the array
							 for(i=0;i<N_good_stars;i++){lin_mag_cor_z[i]=poly_coeff[1]*poly_x[i]+poly_coeff[0]-poly_y[i];}
							 // Fit a plane in the (X, Y, delta_mag) space

                                                         fit_plane_lin( lin_mag_cor_x, lin_mag_cor_y, lin_mag_cor_z, (unsigned int)N_good_stars, &lin_mag_A, &lin_mag_B, &lin_mag_C);
							 write_magnitude_calibration_log_plane( lin_mag_cor_x, lin_mag_cor_y, lin_mag_cor_z, N_good_stars, base_star[n], lin_mag_A, lin_mag_B, lin_mag_C);
							 
							 // Iteratively remove outliers
							 do{
							  the_baddest_outlier=0.0;
							  the_baddest_outlier_number=-1;
							  for(exclude_outlier_mags_counter=N_good_stars;exclude_outlier_mags_counter--;){
							   computed_mag=poly_coeff[1]*poly_x[exclude_outlier_mags_counter]+poly_coeff[0];
							   // find the baddest outlier
							   abs_computed_predicted_mag_diff=fabs(computed_mag-poly_y[exclude_outlier_mags_counter]);
							   if( abs_computed_predicted_mag_diff>3*MAX_DIFF_POLY_MAG_CALIBRATION && abs_computed_predicted_mag_diff>the_baddest_outlier ){
							    the_baddest_outlier=abs_computed_predicted_mag_diff;
                                                            the_baddest_outlier_number=exclude_outlier_mags_counter;
							   }
							  }
							  // And now remove the baddest outlier
							  if(the_baddest_outlier_number>=0){
							   // Exclude bad point from calibration
							   //exclude_from_3_double_arrays(poly_x, poly_y, poly_err, the_baddest_outlier_number, &N_good_stars);
							   exclude_from_6_double_arrays(poly_x, poly_y, poly_err, lin_mag_cor_x, lin_mag_cor_y, lin_mag_cor_z, the_baddest_outlier_number, &N_good_stars);
							   // Recompute fit using unweighted linear fit
                                                           wpolyfit_exit_code=wlinearfit(poly_x, poly_y, poly_err_fake, N_good_stars, poly_coeff);
                                                           // Populate the array
                                                           for(i=0;i<N_good_stars;i++){lin_mag_cor_z[i]=poly_coeff[1]*poly_x[i]+poly_coeff[0]-poly_y[i];}
                                                           // Fit a plane in the (X, Y, delta_mag) space
                                                           fit_plane_lin( lin_mag_cor_x, lin_mag_cor_y, lin_mag_cor_z, (unsigned int)N_good_stars, &lin_mag_A, &lin_mag_B, &lin_mag_C);
							  }
                                                         }while(the_baddest_outlier_number>=0);
                                                         
                                                         // Apply the linear CCD position dependent corrections
                                                         for(i=0;i<N_good_stars;i++){poly_y[i]=poly_y[i]+(lin_mag_A*lin_mag_cor_x[i]+lin_mag_B*lin_mag_cor_y[i]+lin_mag_C);
                                                         }//fprintf(stderr,"DEBUG: %lf\n",lin_mag_A*lin_mag_cor_x[i]+lin_mag_B*lin_mag_cor_y[i]+lin_mag_C);}
                                                         write_magnitude_calibration_log2( poly_x, poly_y, poly_err, N_good_stars, base_star[n]);
                                                         fprintf(stderr,"Using the linear CCD position dependent magnitude correction:\n delta_m = %7.5lf*X_pix %+7.5lf*Y_pix %+7.5lf\n",lin_mag_A,lin_mag_B,lin_mag_C);
                                                         write_magnitude_calibration_log_plane( lin_mag_cor_x, lin_mag_cor_y, lin_mag_cor_z, N_good_stars, base_star[n], lin_mag_A, lin_mag_B, lin_mag_C);
                                                         
                                                         // Check if the magnitude correction is not too large
                                                         if( fabs(lin_mag_C)>MAX_LIN_CORR_MAG )wpolyfit_exit_code=1;
                                                         if( fabs(lin_mag_A*X_im_size+lin_mag_C)>MAX_LIN_CORR_MAG )wpolyfit_exit_code=1;
                                                         if( fabs(lin_mag_B*Y_im_size+lin_mag_C)>MAX_LIN_CORR_MAG )wpolyfit_exit_code=1;
                                                         if( fabs(lin_mag_A*X_im_size+lin_mag_B*Y_im_size+lin_mag_C)>MAX_LIN_CORR_MAG )wpolyfit_exit_code=1;

                                                         // Use linear function as an approximation
                                                         if( wpolyfit_exit_code==0 ){
                                                          wpolyfit_exit_code=wlinearfit(poly_x, poly_y, poly_err_fake, N_good_stars, poly_coeff);
							  do{
							   the_baddest_outlier=0.0;
							   the_baddest_outlier_number=-1;
							   for(exclude_outlier_mags_counter=N_good_stars;exclude_outlier_mags_counter--;){
							    computed_mag=poly_coeff[1]*poly_x[exclude_outlier_mags_counter]+poly_coeff[0];
							    // find the baddest outlier
							    abs_computed_predicted_mag_diff=fabs(computed_mag-poly_y[exclude_outlier_mags_counter]);
							    if( abs_computed_predicted_mag_diff>3*MAX_DIFF_POLY_MAG_CALIBRATION && abs_computed_predicted_mag_diff>the_baddest_outlier ){
							     the_baddest_outlier=abs_computed_predicted_mag_diff;
                                                             the_baddest_outlier_number=exclude_outlier_mags_counter;
							    }
							   }
							   // And now remove the baddest outlier
							   if(the_baddest_outlier_number>=0){
							    // Exclude bad point from calibration
                                                            //exclude_from_3_double_arrays(poly_x, poly_y, poly_err, the_baddest_outlier_number, &N_good_stars);
                                                            exclude_from_6_double_arrays(poly_x, poly_y, poly_err, lin_mag_cor_x, lin_mag_cor_y, lin_mag_cor_z, the_baddest_outlier_number, &N_good_stars);
							    // Recompute fit using unweighted linear fit
                                                            wpolyfit_exit_code=wlinearfit(poly_x, poly_y, poly_err_fake, N_good_stars, poly_coeff);
							   }
                                                          }while(the_baddest_outlier_number>=0);
  
							  /* Check that we haven't dropped too many stars so the parabolic fit still make sence */
							  if( N_good_stars<MIN_NUMBER_STARS_POLY_MAG_CALIBR )
							   wpolyfit_exit_code=1;
                                                         } // if( wpolyfit_exit_code==0 ){
                                                        } // if( apply_position_dependent_correction==1 && wpolyfit_exit_code==0 ){

							// Second pass - remove the remaining outlier using weighted approximation
							if( wpolyfit_exit_code==0 ){
                                                        // Use linear function as the very first approximation
                                                        wpolyfit_exit_code=wlinearfit(poly_x, poly_y, poly_err, N_good_stars, poly_coeff);
							do{
							 the_baddest_outlier=0.0;
							 the_baddest_outlier_number=-1;
							 for(exclude_outlier_mags_counter=N_good_stars;exclude_outlier_mags_counter--;){
							  computed_mag=poly_coeff[2]*poly_x[exclude_outlier_mags_counter]*poly_x[exclude_outlier_mags_counter]+poly_coeff[1]*poly_x[exclude_outlier_mags_counter]+poly_coeff[0];
							  // find the baddest outlier
							  abs_computed_predicted_mag_diff=fabs(computed_mag-poly_y[exclude_outlier_mags_counter]);
							  if( abs_computed_predicted_mag_diff>MAX_DIFF_POLY_MAG_CALIBRATION && abs_computed_predicted_mag_diff>the_baddest_outlier ){
							   the_baddest_outlier=abs_computed_predicted_mag_diff;
                                                           the_baddest_outlier_number=exclude_outlier_mags_counter;
							  }
							 }
							 // And now remove the baddest outlier
							 if(the_baddest_outlier_number>=0){
							  // Exclude bad point from calibration
							  if( apply_position_dependent_correction==0 )
							   exclude_from_3_double_arrays(poly_x, poly_y, poly_err, the_baddest_outlier_number, &N_good_stars);
                                                          else
							   exclude_from_6_double_arrays(poly_x, poly_y, poly_err, lin_mag_cor_x, lin_mag_cor_y, lin_mag_cor_z, the_baddest_outlier_number, &N_good_stars);
							  // Recompute fit using linear or parabolic function depending on settings
							  if(param_p==0) 
							   wpolyfit_exit_code=wlinearfit(poly_x, poly_y, poly_err, N_good_stars, poly_coeff);
							  else
							   wpolyfit_exit_code=wpolyfit(poly_x, poly_y, poly_err, N_good_stars, poly_coeff);
							 }
                                                        }while(the_baddest_outlier_number>=0);
							
							/* Check that we haven't dropped too many stars so the parabolic fit still make sence */
							if( N_good_stars<MIN_NUMBER_STARS_POLY_MAG_CALIBR )
							 wpolyfit_exit_code=1;
							else
							 if (debug != 0) fprintf(stderr, "Ok %d\n", wpolyfit_exit_code);
                                                        
                                                        /* Above we assume that parabola was a reasonable approximation.
                                                           Now we use photocurve for magnitude calibration if param_use_photocurve is set. */
                                                        if( param_use_photocurve!=0 && wpolyfit_exit_code==0 ){
                                                         wpolyfit_exit_code=fit_photocurve(poly_x, poly_y, poly_err, N_good_stars, poly_coeff, &param_use_photocurve);
                                                         poly_coeff[4]=(double)param_use_photocurve;
                                                        }
                                                        
                                                        /* Write calibration parameters to log files */
                                                        write_magnitude_calibration_param_log( poly_coeff, base_star[n]);
                                                        
                                                        } // if( wpolyfit_exit_code==0 ){
						}
						if (debug != 0) fprintf(stderr, "DEBUG MSG: free(poly_x) - ");
						free(poly_x);
						if (debug != 0) fprintf(stderr, "Ok\n");
						if (debug != 0) fprintf(stderr, "DEBUG MSG: free(poly_y) - "); 
						free(poly_y);
						if (debug != 0) fprintf(stderr, "Ok\n");
						if (debug != 0) fprintf(stderr, "DEBUG MSG: free(poly_err) - "); 
						free(poly_err);
						if (debug != 0) fprintf(stderr, "Ok\n");
						free(poly_err_fake);
						
						if( apply_position_dependent_correction==1 ){
						 free(lin_mag_cor_x);
						 free(lin_mag_cor_y);
						 free(lin_mag_cor_z);
						}
						
					if (debug != 0) fprintf(stderr, "DEBUG MSG: It was a lot of work, Ok now\n");
					if ( wpolyfit_exit_code==0 ){
						/* Transform all magnitudes to the ref-frame system. */
						if (debug != 0) fprintf(stderr, "DEBUG MSG: Mag. transform - ");
						//for (i = 0; i < NUMBER2; i++) {
						for (i = NUMBER2; i--; ) {
                                                                if( param_use_photocurve!=0 ){
                                                                 // If we use the "photocurve" for magnitude calibration
                                                                 if( apply_position_dependent_correction==1 ){
                                                                  // Aplly CCD position dependent correction
                                                                  STAR2[Pos2[i]].mag = (float)eval_photocurve( (double)STAR2[Pos2[i]].mag, poly_coeff, param_use_photocurve) - (lin_mag_A*STAR2[Pos2[i]].x_frame + lin_mag_B*STAR2[Pos2[i]].y_frame + lin_mag_C);
                                                                 }
                                                                 else{
                                                                  // Do not Aplly CCD position dependent correction
                                                                  STAR2[Pos2[i]].mag = (float)eval_photocurve( (double)STAR2[Pos2[i]].mag, poly_coeff, param_use_photocurve);
                                                                 }
                                                                }
                                                                else{
                                                                 // If we use a linear or parabolic calibration curve
                                                                 if( apply_position_dependent_correction==1 ){
                                                                  // Aplly CCD position dependent correction
                                                                  STAR2[Pos2[i]].mag = STAR2[Pos2[i]].mag * STAR2[Pos2[i]].mag * STAR2[Pos2[i]].mag * STAR2[Pos2[i]].mag * poly_coeff[4] + STAR2[Pos2[i]].mag * STAR2[Pos2[i]].mag * STAR2[Pos2[i]].mag * poly_coeff[3] + STAR2[Pos2[i]].mag * STAR2[Pos2[i]].mag * poly_coeff[2] + STAR2[Pos2[i]].mag * poly_coeff[1] + poly_coeff[0] - (lin_mag_A*STAR2[Pos2[i]].x_frame + lin_mag_B*STAR2[Pos2[i]].y_frame + lin_mag_C);
                                                                 }
                                                                 else{
                                                                  // Do not Aplly CCD position dependent correction
								  STAR2[Pos2[i]].mag = STAR2[Pos2[i]].mag * STAR2[Pos2[i]].mag * STAR2[Pos2[i]].mag * STAR2[Pos2[i]].mag * poly_coeff[4] + STAR2[Pos2[i]].mag * STAR2[Pos2[i]].mag * STAR2[Pos2[i]].mag * poly_coeff[3] + STAR2[Pos2[i]].mag * STAR2[Pos2[i]].mag * poly_coeff[2] + STAR2[Pos2[i]].mag * poly_coeff[1] + poly_coeff[0]; //Что это за монстр?
								 }
                                                                }
						}
						if (debug != 0) fprintf(stderr, "Ok\n");
					}
				//} // nocalib ??

				/* Write to log file if everything whent well or not */
				if( wpolyfit_exit_code==0 ){
				 sprintf(log_output,"status=OK     %s\n",base_star[n]);
				 write_string_to_log_file(log_output,sextractor_catalog);
				 MATCH_SUCESS++;
                                }
                                else{
                                 sprintf(log_output,"status=ERROR  %s\n",base_star[n]);
                                 write_string_to_log_file(log_output,sextractor_catalog);
                                 fprintf(stderr,"ERROR!!! Magnitude calibration failure. Dropping image.\n"); // and print to the terminal that there was a problem
                                }

                                
                                // ADD NEW STARS ONLY IF THE MAGNITUDE SCALE COULD BE SUCCESSFULY CALIBRATED
                                //
                                // If we can't calibrate magnitudes, something is not right with this image, better skip it altogether
                                //
                                
				//// **** Add new stars (which were not detected on the reference frame) to STAR1 - the structure with reference stars  ****
				//if( param_failsafe==0 ){
				if( param_failsafe==0 && Number_of_ecv_star > 0 && wpolyfit_exit_code==0 ){

				 Pos1 = realloc(Pos1, sizeof(int) * ( MAX(NUMBER1,NUMBER2) + 1) );
				 
				 for (i = Number_of_ecv_star; i < NUMBER2; i++){
				  //fprintf(stderr,"DEBUG: NUMBER1=%d i=%d Number_of_ecv_star=%d NUMBER2=%d\n",NUMBER1,i,Number_of_ecv_star,NUMBER2);
				  // If this is a good star
				  if (fabs(STAR2[Pos2[i]].mag) > BRIGHTEST_STARS && STAR2[Pos2[i]].mag < faintest_stars  && X_im_size-STAR2[Pos2[i]].x_frame>FRAME_EDGE_INDENT_PIXELS && Y_im_size-STAR2[Pos2[i]].y_frame>FRAME_EDGE_INDENT_PIXELS && STAR2[Pos2[i]].x_frame>FRAME_EDGE_INDENT_PIXELS && STAR2[Pos2[i]].y_frame>FRAME_EDGE_INDENT_PIXELS && STAR2[Pos2[i]].sextractor_flag<=maxsextractorflag && STAR2[Pos2[i]].vast_flag==0 ){
				   // Add it to STAR1
				   if( STAR1==NULL ){fprintf(stderr,"ERROR: can't allocate memory for a new star!\n STAR1 = realloc(STAR1, sizeof(struct Star) * (NUMBER1+1) ); - failed!\n");return 1;}
				   //Pos1 = realloc(Pos1, sizeof(int) * (NUMBER1+1) ); // that was working for us all the time
				   //Pos1 = realloc(Pos1, sizeof(int) * ( MAX(i,NUMBER1) +1) ); // does not work
				   if( Pos1==NULL ){fprintf(stderr,"ERROR: can't allocate memory for a new star!\n Pos1 = realloc(Pos1, sizeof(int) * (NUMBER1+1) ); - failed!\n");return 1;}
				   Pos1[i] = NUMBER1;
				   max_number++;
				   STAR2[Pos2[i]].n=max_number;
				   Star_Copy(STAR1 + Pos1[i], STAR2 + Pos2[i]);
				   STAR1[Pos1[i]].mag=0.0;
				   NUMBER1++;
				  
				   /// !!! Debug !!!
				   //if(n==8)write_single_Star_from_struct_to_ds9_region_file(STAR2, Pos2[i], Pos2[i]+1, "STAR_UNID_ds9.reg", aperture);
                                   //float debug_x=586.696;
                                   //float debug_y=438.463;
                                   //float debug_d=sqrt((STAR2[Pos2[i]].x_frame-debug_x)*(STAR2[Pos2[i]].x_frame-debug_x)+(STAR2[Pos2[i]].y_frame-debug_y)*(STAR2[Pos2[i]].y_frame-debug_y));
                                   //if( debug_d<aperture ){
                                   // fprintf(stderr,"ADDING NEW STARS: %d  %.3f %.3f  (%f)  s=%d v=%6d\n",STAR2[Pos2[i]].n,STAR2[Pos2[i]].x_frame,STAR2[Pos2[i]].y_frame,STAR2[Pos2[i]].star_size,STAR2[Pos2[i]].sextractor_flag,STAR2[Pos2[i]].vast_flag);
                                   //}

				  
				   if ( NUMBER1 >= MAX_NUMBER_OF_STARS-1 ) {
				    fprintf(stderr, "ERROR while adding new stars: Too many stars!\nChange string \"#define MAX_NUMBER_OF_STARS %d\" in src/limits.h file and recompile the program by running \"make\".\n", MAX_NUMBER_OF_STARS);
				    exit(1);
                                   }
                                  } // // If this is a good star
                                  //else
                                  // fprintf(stderr,"Bad star!\n");
				 }
				 
				} // if( param_failsafe==0 )
				
				fprintf(stderr,"    %5d objects stored in memory (%5d max.)\n",NUMBER1,MAX_NUMBER_OF_STARS);


				if ( wpolyfit_exit_code==0 ){
					/* Write observation to light curve cache. */
					if (debug != 0) fprintf(stderr, "DEBUG MSG: write cache - ");
                                        
					if( param_failsafe==0 )
					 cache_counter=NUMBER2;
					else
					 cache_counter=Number_of_ecv_star;
					if (debug != 0) fprintf(stderr, "NUMBER2= %d  Number_of_ecv_star= %d ",NUMBER2,Number_of_ecv_star);
					
					coordinate_array_counter=0; 
                                        
					for (i = 0; i < cache_counter; i++) {
						// !!! ONE LAST CHECK IF THIS IS A GOOD STAR !!!
						if (fabs(STAR2[Pos2[i]].mag) > BRIGHTEST_STARS && STAR2[Pos2[i]].mag < faintest_stars  && X_im_size-STAR2[Pos2[i]].x_frame>FRAME_EDGE_INDENT_PIXELS && Y_im_size-STAR2[Pos2[i]].y_frame>FRAME_EDGE_INDENT_PIXELS && STAR2[Pos2[i]].x_frame>FRAME_EDGE_INDENT_PIXELS && STAR2[Pos2[i]].y_frame>FRAME_EDGE_INDENT_PIXELS && STAR2[Pos2[i]].sextractor_flag<=maxsextractorflag && STAR2[Pos2[i]].vast_flag==0 ) {
							TOTAL_OBS++;
							obs_in_RAM++;
							
							// Save coordinates to the array and compute average (median) coordinates
							for(coordinate_array_counter=0;coordinate_array_counter<NUMBER1;coordinate_array_counter++){
							 if( STAR1[Pos1[i]].n==star_numbers_for_coordinate_arrays[coordinate_array_counter] ){
							  // may be we don't want to do it if number_of_coordinate_measurements_for_star[coordinate_array_counter] > something ?
							  if( number_of_coordinate_measurements_for_star[coordinate_array_counter]<MAX_N_IMAGES_USED_TO_DETERMINE_STAR_COORDINATES ){
							   number_of_coordinate_measurements_for_star[coordinate_array_counter]++;
							   coordinate_array_x[coordinate_array_counter]=realloc(coordinate_array_x[coordinate_array_counter], number_of_coordinate_measurements_for_star[coordinate_array_counter]*sizeof(float));
							   coordinate_array_y[coordinate_array_counter]=realloc(coordinate_array_y[coordinate_array_counter], number_of_coordinate_measurements_for_star[coordinate_array_counter]*sizeof(float));
							   coordinate_array_x[coordinate_array_counter][number_of_coordinate_measurements_for_star[coordinate_array_counter]-1]=STAR2[Pos2[i]].x;
							   coordinate_array_y[coordinate_array_counter][number_of_coordinate_measurements_for_star[coordinate_array_counter]-1]=STAR2[Pos2[i]].y;
							   // update coordinates ONLY if we already have many measurements
							   if( number_of_coordinate_measurements_for_star[coordinate_array_counter]>MIN_N_IMAGES_USED_TO_DETERMINE_STAR_COORDINATES ){
							    gsl_sort_float(coordinate_array_x[coordinate_array_counter],1,number_of_coordinate_measurements_for_star[coordinate_array_counter]);
							    gsl_sort_float(coordinate_array_y[coordinate_array_counter],1,number_of_coordinate_measurements_for_star[coordinate_array_counter]);
							    STAR1[Pos1[i]].x=gsl_stats_float_median_from_sorted_data(coordinate_array_x[coordinate_array_counter],1,number_of_coordinate_measurements_for_star[coordinate_array_counter]);
							    STAR1[Pos1[i]].y=gsl_stats_float_median_from_sorted_data(coordinate_array_y[coordinate_array_counter],1,number_of_coordinate_measurements_for_star[coordinate_array_counter]);
							   } // update coordinates ONLY if we already have many measurements
							  } // if( number_of_coordinate_measurements_for_star[coordinate_array_counter]<MAX_N_IMAGES_USED_TO_DETERMINE_STAR_COORDINATES ){
							 } // if( STAR1[Pos1[i]].n==star_numbers_for_coordinate_arrays[coordinate_array_counter] ){
							} // for(coordinate_array_counter=0;coordinate_array_counter<NUMBER1;coordinate_array_counter++){
							// Update coordinates in STAR3 (reference structure for image metching)
							if( n>MIN_N_IMAGES_USED_TO_DETERMINE_STAR_COORDINATES ){ // this step make sence only if coordinates in STAR1 have (cor could have) been updated, and that if checks it!
							 //for( i_update_coordinates_STAR3=0; i_update_coordinates_STAR3<NUMBER3; i_update_coordinates_STAR3++ ){
							 for( i_update_coordinates_STAR3=NUMBER3; i_update_coordinates_STAR3--; ){
							  if( STAR1[Pos1[i]].n==STAR3[i_update_coordinates_STAR3].n ){
							   STAR3[i_update_coordinates_STAR3].x=STAR1[Pos1[i]].x;
							   STAR3[i_update_coordinates_STAR3].y=STAR1[Pos1[i]].y;
							  } // if( STAR1[Pos1[i]].n==STAR3[i_update_coordinates_STAR3].n ){
							 } // for( i_update_coordinates_STAR3=0; i_update_coordinates_STAR3<NUMBER3; i_update_coordinates_STAR3++ ){
							} // if( n>MIN_N_IMAGES_USED_TO_DETERMINE_STAR_COORDINATES ){

							if( obs_in_RAM>Max_obs_in_RAM ){
							 ptr_struct_Obs = realloc(ptr_struct_Obs, sizeof(struct Observation) * obs_in_RAM);
							 if( ptr_struct_Obs==NULL ){fprintf(stderr,"ERROR: can't allocate memory for a new observation!\n ptr_struct_Obs = realloc(ptr_struct_Obs, sizeof(struct Observation) * obs_in_RAM); - failed!\n");return 1;}
                                                        }
							ptr_struct_Obs[obs_in_RAM-1].star_num = STAR1[Pos1[i]].n;
							ptr_struct_Obs[obs_in_RAM-1].JD = STAR2[Pos2[i]].JD;
							ptr_struct_Obs[obs_in_RAM-1].mag = (double)STAR2[Pos2[i]].mag;
							ptr_struct_Obs[obs_in_RAM-1].mag_err = (double)STAR2[Pos2[i]].sigma_mag;
							ptr_struct_Obs[obs_in_RAM-1].X = (double)STAR2[Pos2[i]].x_frame;
							ptr_struct_Obs[obs_in_RAM-1].Y = (double)STAR2[Pos2[i]].y_frame;
							ptr_struct_Obs[obs_in_RAM-1].APER = aperture;
							if( strlen(base_star[n])>FILENAME_LENGTH ){fprintf(stderr,"ERROR: strlen(base_star[n])>FILENAME_LENGTH \nFILENAME_LENGTH=%d\nFilename: %s\nPlease, increase FILENAME_LENGTH in src/limits.h and recompile the program using make!\n",FILENAME_LENGTH,base_star[n]);exit(1);}
							strcpy(ptr_struct_Obs[obs_in_RAM-1].filename, base_star[n]);
							ptr_struct_Obs[obs_in_RAM-1].is_used=0;
							
							/// !!! Debug !!!
							//float debug_x=586.696;
							//float debug_y=438.463;
							//float debug_d=sqrt((STAR2[Pos2[i]].x_frame-debug_x)*(STAR2[Pos2[i]].x_frame-debug_x)+(STAR2[Pos2[i]].y_frame-debug_y)*(STAR2[Pos2[i]].y_frame-debug_y));
							//if( debug_d<aperture ){
							// fprintf(stderr,"SAVING OBS TO MEMORY: %d  %.3f %.3f  (%f)  s=%d v=%6d\n",STAR2[Pos2[i]].n,STAR2[Pos2[i]].x_frame,STAR2[Pos2[i]].y_frame,STAR2[Pos2[i]].star_size,STAR2[Pos2[i]].sextractor_flag,STAR2[Pos2[i]].vast_flag);
                                                        //}

							
						}
					
					}
					if (debug != 0) fprintf(stderr, "Ok\n");
				} 
				//else{
                                // // Already reported
				// fprintf(stderr,"ERROR!!! Magnitude calibration failure. Dropping image.\n");
				//}
				
				
				if (debug != 0) fprintf(stderr, "DEBUG MSG: 00001 ");
				// at this point it is assumed that images were successfully matched
				if( match_try!=0 )success_match_on_increase++;
				// test should we use new number of reference stars as default?
				if( param_set_manually_Number_of_main_star==0 ){
				 if( success_match_on_increase>MIN_SUCCESS_MATCH_ON_RETRY ){
				  success_match_on_increase=0;
				  default_Number_of_ecv_triangle=preobr->Number_of_ecv_triangle;
                                  default_Number_of_main_star=preobr->Number_of_main_star;
                                  fprintf(stderr,"WARNING: changing the number of reference stars for image matching to %d\n",default_Number_of_main_star);
                                 }
                                }
			        if (debug != 0) fprintf(stderr, "Ok\n");
			        
			        if (debug != 0) fprintf(stderr, "DEBUG MSG: 00002 ");
			        // We do not care about transient candidates in failsafe mode OR if all stars on the frame were matched
				if( param_failsafe==0 && Number_of_ecv_star < NUMBER2 ){
				 // Make sure the potential transients are not suspiciously fast
				 if( fabs(STAR3[0].JD-STAR2[Pos2[Number_of_ecv_star]].JD)>TRANSIENT_MIN_TIMESCALE_DAYS ){
				  // Search for transients among new stars
				  for (i = Number_of_ecv_star; i < NUMBER2; i++){
				   if( STAR2[Pos2[i]].vast_flag!=0 )continue;
				   //fprintf(stderr,"*** DEBUG %d\n",STAR2[Pos2[i]].n);
				   test_transient( search_area_boundaries, STAR2[Pos2[i]], STAR3[0].JD);
                                  }
                                 }
                                }
                                if (debug != 0) fprintf(stderr, "Ok\n");
				
				/* Write observations to disk to save RAM */
				if( 0!=check_and_print_memory_statistics() && Max_obs_in_RAM>1000 )Max_obs_in_RAM=Max_obs_in_RAM/2;
				if( obs_in_RAM>Max_obs_in_RAM ){
                                   	fprintf(stderr,"Total number of measurements %ld (%ld measurements stored in RAM)\n",TOTAL_OBS,obs_in_RAM);
					fprintf(stderr,"writing lightcurve (outNNNNN.dat) files...\n");
					/* Write observation to disk */

					i_fork=0; // counter for fork 
					number_of_lightcurves_for_each_thread=(int)(NUMBER1/n_fork)+1;
					for(i = 0; i < NUMBER1; i+=number_of_lightcurves_for_each_thread) {
        				 i_fork++;
					 pid=fork();
        				 if( pid==0 || pid==-1 ){
					  // if child or parent cannot fork
					  if( pid==-1 )fprintf(stderr,"WARNING: cannot fork()! Continuing in the streamline mode...\n");

					  for(k=i; k < MIN(i+number_of_lightcurves_for_each_thread,NUMBER1); k++) {
					        
					   // process one star
        				   snprintf(tmpNAME, OUTFILENAME_LENGTH,  "out%05d.dat", STAR1[k].n); // Generate lightcurve filename
					   file_out=fopen(tmpNAME,"a"); //====================file_out===========================
					   if( file_out==NULL ){
					    fprintf(stderr,"ERROR: can't open file %s\n",tmpNAME);
					    return 1;
					   }
					   for (j = 0; j < obs_in_RAM; j++) {
					    if( ptr_struct_Obs[j].is_used==0 ){
					     if(ptr_struct_Obs[j].star_num == STAR1[k].n && ptr_struct_Obs[j].star_num != 0){
                                              write_lightcurve_point( file_out, ptr_struct_Obs[j].JD, ptr_struct_Obs[j].mag, ptr_struct_Obs[j].mag_err, ptr_struct_Obs[j].X, ptr_struct_Obs[j].Y, ptr_struct_Obs[j].APER, ptr_struct_Obs[j].filename);
					 			ptr_struct_Obs[j].is_used=1;
        				     }
					    }
					   }
					   fclose(file_out);
        				   // end of process one star
        				  }     
        				  if( pid==0 )exit(0); // exit only if this is actually a child
        				  // end of child 
        				 }
        				 else{
        				  // if parent
        				  child_pids[i_fork-1]=pid;
        				  if( i_fork==MIN(n_fork,NUMBER1) ){
        				   for(;i_fork--;){
        				    pid=child_pids[i_fork];
        				    waitpid(pid,&pid_status,0);
        				   }
        				   // i_fork=-1 after the for
        				   i_fork=0;
        				  }
					 }
					} // for(i = 0; i < NUMBER1; i++) {

					obs_in_RAM=0;
					// !!! Experimental stuff !!!
					// Max_obs_in_RAM may have changed, so we need to re-allocate the memory
					free(ptr_struct_Obs);
					ptr_struct_Obs = malloc(sizeof(struct Observation) * Max_obs_in_RAM); 
				}
				/* Reset variables */
				wpolyfit_exit_code=0;
				if (debug != 0) fprintf(stderr, "DEBUG MSG: free(Pos2); - ");
				free(Pos2);
				if (debug != 0) fprintf(stderr, "Ok\n");
				if (debug != 0) fprintf(stderr, "DEBUG MSG: free(STAR2); - ");
				free(STAR2);
				if (debug != 0) fprintf(stderr, "Ok\n");
			} else {
				fprintf(stderr, "ERROR! APERTURE > 50\n");
				/* Write error to the logfile */
				sprintf(log_output,"rotation=   0.000  *detected= %5d  *matched= %5d  status=ERROR  %s\n", 0, 0, base_star[n]);
				write_string_to_log_file(log_output,sextractor_catalog);
			}
		} else {
			fprintf(stderr, "Error reading file %s\n", base_star[n]);
			/* Write error to the logfile */
			sprintf(log_output,"exp_start= %02d.%02d.%4d %02d:%02d:%02d  exp= %4d  JD= %13.5lf  ap= %4.1lf  rotation= %7.3lf  *detected= %5d  *matched= %5d  status=ERROR  %s\n", 0, 0, 0, 0, 0, 0, 0, 0.0, 0.0, 0.0, 0, 0, base_star[n]);
			write_string_to_log_file(log_output,sextractor_catalog);
			
			fitsfile_read_error = 0;
		}

		progress(n + 1, Num);
	}

	free(bad_stars_X);
	free(bad_stars_Y);

	
	
	fprintf(stderr,"Done with measurements! =)\n\n");
	fprintf(stderr,"Total number of measurements %ld (%ld measurements stored in RAM)\n",TOTAL_OBS,obs_in_RAM);
	fprintf(stderr,"Writing lightcurve (outNNNNN.dat) files...\n");

	/* Write observation to disk */

	i_fork=0; // counter for fork 
	number_of_lightcurves_for_each_thread=(int)(NUMBER1/n_fork)+1;
	for(i = 0; i < NUMBER1; i+=number_of_lightcurves_for_each_thread) {
         i_fork++;
	 pid=fork();
         if( pid==0 || pid==-1 ){
	  // if child or parent cannot fork()
          if( pid==-1 )fprintf(stderr,"WARNING: cannot fork()! Continuing in the streamline mode...\n");

	  for(k=i; k < MIN(i+number_of_lightcurves_for_each_thread,NUMBER1); k++) {
	   // process one star
           snprintf(tmpNAME, OUTFILENAME_LENGTH,  "out%05d.dat", STAR1[k].n); // Generate lightcurve filename
	   file_out=fopen(tmpNAME,"a"); //====================file_out===========================
	   if( file_out==NULL ){
	    fprintf(stderr,"ERROR: can't open file %s for appending!\n",tmpNAME);
	    return 1;
	   }
	   for (j = 0; j < obs_in_RAM; j++) {
	    if( ptr_struct_Obs[j].is_used==0 ){
	     if(ptr_struct_Obs[j].star_num == STAR1[k].n && ptr_struct_Obs[j].star_num != 0){
                                write_lightcurve_point( file_out, ptr_struct_Obs[j].JD, ptr_struct_Obs[j].mag, ptr_struct_Obs[j].mag_err, ptr_struct_Obs[j].X, ptr_struct_Obs[j].Y, ptr_struct_Obs[j].APER, ptr_struct_Obs[j].filename);
	 			ptr_struct_Obs[j].is_used=1;
             }
	    }
	   }
	   fclose(file_out);
           // end of process one star
          }     
          if( pid==0 )exit(0); // exit only if this is actually a child
          // end of child 
         }
         else{
          // if parent
          child_pids[i_fork-1]=pid;
          if( i_fork==MIN(n_fork,NUMBER1) ){
           for(;i_fork--;){
            pid=child_pids[i_fork];
            waitpid(pid,&pid_status,0);
           }
           // i_fork=-1 after the for
           i_fork=0;
          }
	 }
	} // for(i = 0; i < NUMBER1; i++) {

	free(child_pids);

        // Dump STAR1 structure to a log file
        vast_list_of_all_stars_log=fopen("vast_list_of_all_stars.log","w");
        if(NULL==vast_list_of_all_stars_log){fprintf(stderr,"WARNING! cannot create vast_list_of_all_stars.log\n");}
        else{
         for(i = 0; i < NUMBER1; i++){
          fprintf(vast_list_of_all_stars_log,"%05d %8.3lf %8.3lf\n",STAR1[i].n,STAR1[i].x,STAR1[i].y);
         }
         fclose(vast_list_of_all_stars_log);
        }
        // Same as above, but in DS9 format
        vast_list_of_all_stars_ds9=fopen("vast_list_of_all_stars.ds9.reg","w");
        if(NULL==vast_list_of_all_stars_ds9){fprintf(stderr,"WARNING! cannot create vast_list_of_all_stars.log\n");}
        else{
         fprintf(vast_list_of_all_stars_ds9,"# Region file format: DS9 version 4.0\n# Filename: vast_list_of_all_stars.ds9.reg\nglobal color=green font=\"sans 10 normal\" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source\nimage\n");
         for(i = 0; i < NUMBER1; i++){
          fprintf(vast_list_of_all_stars_ds9,"circle(%8.3lf,%8.3lf,%8.3lf)\n# text(%8.3lf,%8.3lf) text={%05d}\n",STAR1[i].x,STAR1[i].y,STAR1[i].star_size,STAR1[i].x,STAR1[i].y,STAR1[i].n);
         }
         fclose(vast_list_of_all_stars_ds9);
        }
 
 
	fprintf(stderr,"The lightcurve files have been written, freeing up some memory...\n");
	if (debug != 0) fprintf(stderr, "DEBUG MSG: free(ptr_struct_Obs);\n");
	free(ptr_struct_Obs);
	
	if (debug != 0) fprintf(stderr, "DEBUG MSG: Delete_Preobr_Sk(preobr);\n");
	Delete_Preobr_Sk(preobr);
	
	if (debug != 0) fprintf(stderr, "DEBUG MSG: free(Pos1);\n");
	free(Pos1);
	
	if (debug != 0) fprintf(stderr, "DEBUG MSG: free(STAR1);\n");
	free(STAR1);
	if (debug != 0) fprintf(stderr, "DEBUG MSG: free(STAR3);\n");
	free(STAR3); 
	if (debug != 0) fprintf(stderr, "DEBUG MSG: free(starfile);\n");
	free(starfile);
	if (debug != 0) fprintf(stderr, "DEBUG MSG: for(n = 0; n < Num; n++)free(base_star[n]);\n");
	//for(n = 0; n < Num; n++)
	for(n = Num; n--; )
		free(base_star[n]);
        if (debug != 0) fprintf(stderr, "DEBUG MSG: free(base_star);\n");
	free(base_star);
        
        if (debug != 0) fprintf(stderr, "DEBUG MSG: freeing coordinate arrays\n");
        //for(coordinate_array_counter=0;coordinate_array_counter<NUMBER1;coordinate_array_counter++){
        for(coordinate_array_counter=NUMBER1;coordinate_array_counter--;){
         free(coordinate_array_x[coordinate_array_counter]);
         free(coordinate_array_y[coordinate_array_counter]);
        }
        if (debug != 0) fprintf(stderr, "DEBUG MSG: free(coordinate_array_x);\n");
        free(coordinate_array_x);
        if (debug != 0) fprintf(stderr, "DEBUG MSG: free(coordinate_array_y);\n");
        free(coordinate_array_y);
        if (debug != 0) fprintf(stderr, "DEBUG MSG: free(star_numbers_for_coordinate_arrays);\n");
        free(star_numbers_for_coordinate_arrays);
        if (debug != 0) fprintf(stderr, "DEBUG MSG: free(number_of_coordinate_measurements_for_star);\n");
        free(number_of_coordinate_measurements_for_star);
	
        /* Save details about magnitude calibration */
        if (debug != 0) fprintf(stderr, "DEBUG MSG: vast.c is starting lib/save_magnitude_calibration_details.sh\n");
//        if( param_nocalib!=1 )
         system("lib/save_magnitude_calibration_details.sh");
	
	/* Create vast_image_details.log */
	system("echo \"Creating vast_image_details.log ...\"; for i in image*.log ;do cat $i >> vast_image_details.log; rm -f $i ; done");
	
	/* Generate summary log */
	if (debug != 0) fprintf(stderr, "DEBUG MSG: vast.c is starting echo and lib/vast_image_details_log_parser.sh > vast_summary.log && echo Ok\n");
	fprintf(stderr,"\nWriting summary file: vast_summary.log ...  ");
	system("lib/vast_image_details_log_parser.sh > vast_summary.log && echo Ok ");
	
	/* Remove lightcurves with less than HARD_MIN_NUMBER_OF_POINTS points */
	if (debug != 0) fprintf(stderr, "DEBUG MSG: vast.c is starting lib/remove_lightcurves_with_small_number_of_points\n");
	system("lib/remove_lightcurves_with_small_number_of_points");
	
        /* Remove bad (nan) measurements from lightcurves */
        if (debug != 0) fprintf(stderr, "DEBUG MSG: vast.c is starting lib/clean_lightcurves_from_nan\n");
        system("lib/clean_lightcurves_from_nan");
        
        /* Remove points with large errors */
        if (debug != 0) fprintf(stderr, "DEBUG MSG: vast.c is starting lib/remove_points_with_large_errors\n");
        if( Num>2*HARD_MIN_NUMBER_OF_POINTS )
         system("lib/remove_points_with_large_errors");
        

        /* Apply sigma filter if we want... */
//	if(param_nocalib != 1 && param_nofilter != 1 && Num>2*HARD_MIN_NUMBER_OF_POINTS ) {
	if(param_nofilter != 1 && Num>2*HARD_MIN_NUMBER_OF_POINTS ) {
	        if (debug != 0) fprintf(stderr, "DEBUG MSG: vast.c is starting lib/new_lightcurve_sigma_filter\n");
		system("lib/new_lightcurve_sigma_filter");
	}

        /* Remove lightcurves with less than HARD_MIN_NUMBER_OF_POINTS points (again) */
        if (debug != 0) fprintf(stderr, "DEBUG MSG: vast.c is starting lib/remove_lightcurves_with_small_number_of_points\n");
        system("lib/remove_lightcurves_with_small_number_of_points");

        /* Choose string to describe time system */
        if( timesys==2 )
	 sprintf(tymesys_str,"TT");
	else
	 if( timesys==1 )
	  sprintf(tymesys_str,"UTC");
	 else
	  sprintf(tymesys_str,"UNKNOWN");	                                                                       

        if (debug != 0) fprintf(stderr, "DEBUG MSG: tymesys_str= _%s_\n",tymesys_str);

	/* Write more comments to log file to log file */	
	vast_image_details=fopen("vast_summary.log","a");
	if( vast_image_details==NULL ){fprintf(stderr,"ERROR: cannot open vast_summary.log for writing!\n");return 1;}
	fprintf(vast_image_details,"JD time system (TT/UTC/UNKNOWN): %s\n",tymesys_str);
	fclose(vast_image_details);


	/* Prepare list of possible transients */
	if( param_failsafe==0 ){
	  system("lib/transient_list");
	  //system("lib/remove_blends_from_transient_candidates");
	  //if( Num==3 || Num==4 )
	  if( Num<10 ){
	   // add flares to the list of transients
	   system("lib/find_flares >> candidates-flares.lst ");
	   system("while read A B ;do echo $A `grep -c \"\" $A` `tail -n1 $A |awk '{printf \"%s %8.3f %8.3f  \",$7,$4,$5}'` `head -n1 $A |awk '{printf \"%s %8.3f %8.3f  \",$7,$4,$5}'` ;done < candidates-flares.lst >> candidates-transients.lst");
          }
	  system("/bin/echo -n \"Transient candidates found: \" >> vast_summary.log ; if [ -f candidates-transients.lst ];then grep -c \" \" candidates-transients.lst >> vast_summary.log ;else /bin/echo 0 >> vast_summary.log ;fi"); /* Write to summary log */
	}

	for(n=0;n<number_of_sysrem_iteration;n++){
	 fprintf(stderr,"Starting %d SysRem iteration...\n",number_of_sysrem_iteration);
	 system("util/sysrem");
	}
	
        /* Stop timer */
        end_time=time(NULL);
        elapsed_time=difftime(end_time, start_time);
        /* Write more comments to log file to log file */
        vast_image_details=fopen("vast_summary.log","a");
        if( vast_image_details==NULL ){fprintf(stderr,"ERROR: cannot open vast_summary.log for writing!\n");return 1;}
        fprintf(vast_image_details,"Number of SysRem iterations: %d\n",number_of_sysrem_iteration);
        fprintf(vast_image_details,"Computation time: %.0lf seconds\n",elapsed_time);
        fprintf(vast_image_details,"SExtractor parameter file: default.sex\n"); // actually, for now this is always default.sex
        fclose(vast_image_details);

        /* Write more stats */
        if( Num>3 ){
         if (debug != 0) fprintf(stderr, "DEBUG MSG: vast.c is starting util/observations_per_star >> vast_summary.log\n");
         system("util/observations_per_star >> vast_summary.log"); // will use util/nopgplot.sh to compute stats!!
        }
        if (debug != 0) fprintf(stderr, "DEBUG MSG: vast.c is starting util/vast-mc.sh >> vast_summary.log\n");
        system("util/vast-mc.sh >> vast_summary.log");


        /* Write information about maximum memory usage (if /proc/PID/status is readable) */
        if (debug != 0) fprintf(stderr, "DEBUG MSG: vast.c is getting memory usage stats from /proc\n");
        sprintf(stderr_output,"/proc/%d/status",getpid());
        if( 1==is_file(stderr_output) ){
	 sprintf(stderr_output,"/bin/echo -n \"Memory usage \" >> vast_summary.log ; head -n1 vast_memory_usage.log >> vast_summary.log");
         system(stderr_output);
        }
        
        /* Write version information to the log file */
        version(stderr_output);
        vast_image_details=fopen("vast_summary.log","a");
        if( vast_image_details==NULL ){fprintf(stderr,"ERROR: cannot open vast_summary.log for writing!\n");return 1;}
        fprintf(vast_image_details,"Software: %s ",stderr_output);
        compiler_version(stderr_output);
        fprintf(vast_image_details,"compiled with %s",stderr_output);
        fclose(vast_image_details);
        system("sex -v >> vast_summary.log");
        system("/bin/echo -n \"Processing completed on  \" >> vast_summary.log ; export LANG=C ; date >> vast_summary.log"); // write date to the log file
	
	
	/* Search for variability candidates */	
	if( param_nofind == 0 ) {
	  strcpy(stderr_output,"./find_candidates ");
	  if( period_search_switch==1 )strcat(stderr_output,"--tsearch ");
	  if( use_ds9_instead_of_pgfv==1 )strcat(stderr_output,"--ds9 ");
	  fprintf(stderr, "%s\n",stderr_output);
	  if (!system(stderr_output))
	       return 1;
	}
	else{
	 /* Print summary log */
	 fprintf(stdout,"####################################################################################\n");
	 system("cat vast_summary.log");
	 fprintf(stdout,"####################################################################################\n\n");
	}
	
	//unsetenv("MALLOC_CHECK_");
	
	// Decide on the overall outcome of the processing (was it OK or not)
	if( MATCH_SUCESS<(int)(0.75*Num) )return 1;
	
	return 0;
}
